@@ -15,13 +15,20 @@
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+ */
+
+/**
+ * @file
  * Network Functions
- */ 
+ */
 
 #define EXPORTING
+
 #include <includes.h>
 #include <stdarg.h>
+
+#include <glib.h>
+
 #include "libnessus.h"
 #include "network.h"
 #include "resolve.h"
@@ -44,25 +51,24 @@
  * Low-level connection management                                *
  *----------------------------------------------------------------*/
  
-/* Nessus "FILE" structure */
+/** Nessus "FILE" structure */
 typedef struct {
- int fd;		/* socket number, or whatever */
- int transport;	/* "transport" layer code when stream is encapsultated. 
+ int fd;		/**< socket number, or whatever */
+ int transport;	/**< "transport" layer code when stream is encapsultated.
 		 * Negative transport signals a free descriptor */
- int timeout;	  /* timeout, in seconds
-		   * special values: -2 for default */
- int options;			/* Misc options - see libnessus.h */
+ int timeout;	  /**< timeout, in seconds special values: -2 for default */
+ int options;			/**< Misc options - see libnessus.h */
   
  int port;			 
-  SSL_CTX* 	ssl_ctx;	/* SSL context 	*/
-  SSL_METHOD* 	ssl_mt;		/* SSL method   */
-  SSL* 		ssl;		/* SSL handler  */
-  int		last_ssl_err;	/* Last SSL error code */
- pid_t		pid;		/* Owner - for debugging only */
+  SSL_CTX* 	ssl_ctx;	/**< SSL context 	*/
+  SSL_METHOD* 	ssl_mt;		/**< SSL method   */
+  SSL* 		ssl;		/**< SSL handler  */
+  int		last_ssl_err;	/**< Last SSL error code */
+ pid_t		pid;		/**< Owner - for debugging only */
 #if 0
-  int		last_sock_err;	/* last socket level error */
+  int		last_sock_err;	/**< last socket level error */
 #endif
-  char*		buf;		/* NULL if unbuffered */
+  char*		buf;		/**< NULL if unbuffered */
   int		bufsz, bufcnt, bufptr;
 } nessus_connection;
 
@@ -94,25 +100,26 @@ static int	lock_cnt = 0;
 static int	lock_fd = -1;
 #endif
 
-/*
+/**
  * NESSUS_STREAM(x) is TRUE if <x> is a Nessus-ified fd
  */
 #define NESSUS_STREAM(x) (((x - NESSUS_FD_OFF) < NESSUS_FD_MAX) && ((x - NESSUS_FD_OFF) >=0))
 
 
-/*
- * Same as perror(), but prefixes the data by our pid
+/**
+ * @brief Same as perror(), but prefixes the data by our pid.
  */
 static int 
-nessus_perror(error)
- const char* error;
+nessus_perror (const char* error)
 {
   fprintf(stderr, "[%d] %s : %s\n", getpid(), error, strerror(errno));
   return 0;
 }
 
-/*
- * Returns a free file descriptor
+/**
+ * @brief Returns a free file descriptor.
+ * 
+ * @return A free file descriptor.
  */
 static int
 get_connection_fd()
@@ -135,8 +142,7 @@ get_connection_fd()
 }
 
 static int
-release_connection_fd(fd)
- int fd;
+release_connection_fd (int fd)
 {
  nessus_connection *p;
  
@@ -184,9 +190,7 @@ if (p->fd >= 0)
 /* ******** Compatibility function ******** */
 
 ExtFunc int
-nessus_register_connection(s, ssl)
-     int	s;
-     SSL	*ssl;
+nessus_register_connection (int s, SSL* ssl)
 {
   int			fd;
   nessus_connection	*p;
@@ -205,8 +209,7 @@ nessus_register_connection(s, ssl)
 }
 
 ExtFunc int
-nessus_deregister_connection(fd)
- int fd;
+nessus_deregister_connection (int fd)
 {
  nessus_connection * p;
  if(!NESSUS_STREAM(fd))
@@ -267,9 +270,7 @@ static int block_socket(int soc)
 
 /* Adapted from stunnel source code */
 ExtFunc
-void sslerror2(txt, err)
-     char	*txt;
-     int	err;
+void sslerror2 (char* txt, int err)
 {
   char string[120];
 
@@ -278,15 +279,16 @@ void sslerror2(txt, err)
 }
 
 void
-sslerror(txt)
-     char	*txt;
+sslerror (char* txt)
 {
   sslerror2(txt, ERR_get_error());
 }
 
+/**
+ * @param path Entropy pool file name.
+ */
 ExtFunc int
-nessus_SSL_init(path)
-     char	*path;		/* entropy pool file name */
+nessus_SSL_init (char* path)
 {
   SSL_library_init();
   SSL_load_error_strings();
@@ -351,8 +353,7 @@ nessus_SSL_init(path)
 
 
 ExtFunc int
-nessus_get_socket_from_connection(fd)
-     int	fd;
+nessus_get_socket_from_connection (int fd)
 {
   nessus_connection	*fp;
 
@@ -395,9 +396,7 @@ nessus_SSL_password_cb(buf, size, rwflag, userdata)
 
 
 ExtFunc void
-nessus_install_passwd_cb(ssl_ctx, pass)
-     SSL_CTX	*ssl_ctx;
-     char	*pass;
+nessus_install_passwd_cb (SSL_CTX* ssl_ctx, char* pass)
 {
   SSL_CTX_set_default_passwd_cb(ssl_ctx, nessus_SSL_password_cb);
   SSL_CTX_set_default_passwd_cb_userdata(ssl_ctx, pass);
@@ -406,8 +405,7 @@ nessus_install_passwd_cb(ssl_ctx, pass)
 
 
 ExtFunc SSL*
-stream_get_ssl(fd)
-     int	fd;
+stream_get_ssl (int fd)
 {
  nessus_connection * fp;
  if(!NESSUS_STREAM(fd))
@@ -423,9 +421,7 @@ stream_get_ssl(fd)
 }
 
 ExtFunc int
-stream_set_timeout(fd, timeout)
- int fd;
- int timeout;
+stream_set_timeout (int fd, int timeout)
 {
  int old;
  nessus_connection * fp;
@@ -441,8 +437,7 @@ stream_set_timeout(fd, timeout)
 }
 
 ExtFunc int
-stream_set_options(fd, reset_opt, set_opt)
-     int	fd, reset_opt, set_opt;
+stream_set_options (int fd, int reset_opt, int set_opt)
 {
  nessus_connection * fp;
  if(!NESSUS_STREAM(fd))
@@ -458,10 +453,7 @@ stream_set_options(fd, reset_opt, set_opt)
 
 
 static int 
-read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
- int fd;
- void* buf0;
- int min_len, max_len;
+read_stream_connection_unbuffered (int fd, void* buf0, int min_len, int max_len)
 {
   int			ret, realfd, trp, t, err;
   int			total = 0, flag = 0, timeout = TIMEOUT, waitall = 0;
@@ -643,10 +635,7 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
 }
 
 ExtFunc int 
-read_stream_connection_min(fd, buf0, min_len, max_len)
- int fd;
- void* buf0;
- int min_len, max_len;
+read_stream_connection_min (int fd, void* buf0, int min_len, int max_len)
 {
   nessus_connection	*fp;
 
@@ -704,20 +693,13 @@ read_stream_connection_min(fd, buf0, min_len, max_len)
 }
 
 ExtFunc int 
-read_stream_connection(fd, buf0, len)
- int fd;
- void* buf0;
- int len;
+read_stream_connection (int fd, void* buf0, int len)
 {
  return read_stream_connection_min(fd, buf0, -1, len);
 }
 
 static int
-write_stream_connection4(fd, buf0, n, i_opt) 
- int fd;
- void * buf0;
- int n;
- int	i_opt;
+write_stream_connection4 (int fd, void* buf0, int n, int i_opt)
 {
   int			err, ret, count;
  unsigned char* buf = (unsigned char*)buf0;
@@ -836,19 +818,13 @@ write_stream_connection4(fd, buf0, n, i_opt)
 }
 
 ExtFunc int
-write_stream_connection(fd, buf0, n) 
- int fd;
- void * buf0;
- int n;
+write_stream_connection (int fd, void* buf0, int n)
 {
   return write_stream_connection4(fd, buf0, n, 0);
 }
 
 ExtFunc int
-nsend (fd, data, length, i_opt)
- int fd;
- void * data;
- int length, i_opt;
+nsend (int fd, void* data, int length, int i_opt)
 {
   int		n = 0;
 
@@ -886,10 +862,7 @@ nsend (fd, data, length, i_opt)
  }
  
 ExtFunc int
-nrecv (fd, data, length, i_opt)
- int fd;
- void * data;
- int length, i_opt;
+nrecv (int fd, void* data, int length, int i_opt)
 {
   int e;
  if(NESSUS_STREAM(fd))
@@ -912,8 +885,7 @@ nrecv (fd, data, length, i_opt)
  
 
 ExtFunc int
-close_stream_connection(fd)
- int fd;
+close_stream_connection (int fd)
 {
   if(!NESSUS_STREAM(fd))
    {
@@ -932,8 +904,8 @@ close_stream_connection(fd)
 
 
 static int
-open_socket(struct sockaddr_in *paddr, 
-	    int port, int type, int protocol, int timeout)
+open_socket (struct sockaddr_in *paddr,
+             int port, int type, int protocol, int timeout)
 {
   fd_set		fd_w;
   struct timeval	to;
@@ -1023,15 +995,11 @@ again:
 
 
 ExtFunc 
-int open_sock_opt_hn(hostname, port, type, protocol, timeout)
- const char * hostname; 
- unsigned int port; 
- int type;
- int protocol;
- int timeout;
+int open_sock_opt_hn (const char* hostname, unsigned int port, int type,
+                      int protocol, int timeout)
 {
- struct sockaddr_in addr;
-  
+  struct sockaddr_in addr;
+
   bzero((void*)&addr, sizeof(addr));
   addr.sin_family=AF_INET;
   addr.sin_port=htons((unsigned short)port);
@@ -1047,9 +1015,7 @@ int open_sock_opt_hn(hostname, port, type, protocol, timeout)
 
 
 ExtFunc
-int open_sock_tcp_hn(hostname, port)
- const char * hostname;
- unsigned int port;
+int open_sock_tcp_hn (const char* hostname, unsigned int port)
 {
   return open_sock_opt_hn(hostname, port, SOCK_STREAM, IPPROTO_TCP, TIMEOUT);
 }
@@ -1058,17 +1024,14 @@ int open_sock_tcp_hn(hostname, port)
 
 
 
-/* This function reads a text from the socket stream into the
-   argument buffer, always appending a '\0' byte.  The return
-   value is the number of bytes read, without the trailing '\0'.
+/**
+ * @brief Reads a text from the socket stream into the argument buffer, always
+ * @brief appending a '\\0' byte.
+ * 
+ * @return Number of bytes read, without the trailing '\\0'.
  */
-
-
 ExtFunc
-int recv_line(soc, buf, bufsiz)
- int soc;
- char * buf;
- size_t bufsiz;
+int recv_line (int soc, char* buf, size_t bufsiz)
 {
   int n, ret = 0;
   
@@ -1165,11 +1128,10 @@ int recv_line(soc, buf, bufsiz)
     }
   }
   return ret;
-} 
+}
 
 ExtFunc int
-socket_close(soc)
-int soc;
+socket_close (int soc)
 {
 #if defined NESSUS_CNX_LOCK
   if (lock_cnt > 0)
@@ -1185,32 +1147,11 @@ int soc;
   return close(soc);
 }
 
-/*
- * auth_printf()
- *
- * Writes data to the global socket of the thread
- */
-ExtFunc void 
-auth_printf(struct arglist * globals, char * data, ...)
-{
-  va_list param;
-  char buffer[65535];
-  
-  bzero(buffer, sizeof(buffer));
-
-  va_start(param, data);
-  vsnprintf(buffer, sizeof(buffer) - 1, data, param);
-  
-  va_end(param);
-  auth_send(globals, buffer);
-}                    
-
-
 ExtFunc void
-auth_send(struct arglist * globals, char * data)
+auth_send (struct arglist * globals, char * data)
 {
- int soc = (int)arg_get_value(globals, "global_socket");
- int confirm = (int)arg_get_value(globals, "confirm");
+ int soc = GPOINTER_TO_SIZE(arg_get_value(globals, "global_socket"));
+ int confirm = GPOINTER_TO_SIZE(arg_get_value(globals, "confirm"));
  int n = 0;
  int length;
  int sent = 0;
@@ -1261,44 +1202,19 @@ out:
 #endif
 }
 
-/*
- * auth_gets()
- *
- * Reads data from the global socket of the thread
- */
-ExtFunc char * 
-auth_gets(globals, buf, bufsiz)
-     struct arglist * globals;
-     char * buf;
-     size_t bufsiz;
-{
-  int soc = (int)arg_get_value(globals, "global_socket");
-  int n;
-  /* bzero(buf, bufsiz); */
-  n = recv_line(soc, buf, bufsiz);
-  if(n <= 0)
-	  return NULL;
-  
-  return(buf);
-}
-
-
 /*
  * Select() routines
  */
- 
+
 ExtFunc int
-stream_zero(set)
- fd_set * set;
-{ 
+stream_zero (fd_set* set)
+{
  FD_ZERO(set);
  return 0;
 }
 
 ExtFunc int
-stream_set(fd, set)
- int fd;
- fd_set * set;
+stream_set (int fd, fd_set* set)
 {
  int soc = nessus_get_socket_from_connection(fd);
  if(soc >= 0)
@@ -1307,16 +1223,13 @@ stream_set(fd, set)
 }
 
 ExtFunc int
-stream_isset(fd, set)
- int fd;
- fd_set * set;
+stream_isset (int fd, fd_set* set)
 {
  return FD_ISSET(nessus_get_socket_from_connection(fd), set);
 }
 
 ExtFunc int
-fd_is_stream(fd)
-     int	fd;
+fd_is_stream (int fd)
 {
   return NESSUS_STREAM(fd);	/* Should probably be smarter... */
 }
@@ -1334,8 +1247,7 @@ stream_get_buffer_sz ( int fd )
 
 
 ExtFunc int
-stream_set_buffer(fd, sz)
-     int	fd, sz;
+stream_set_buffer (int fd, int sz)
 {
   nessus_connection	*p;
   char			*b;
@@ -1384,7 +1296,8 @@ stream_set_buffer(fd, sz)
 /*------------------------------------------------------------------*/
 
 
-int os_send(int soc, void * buf, int len, int opt )
+int
+os_send (int soc, void * buf, int len, int opt )
 {
  char * buf0 = (char*)buf;
  int e, n;
@@ -1399,7 +1312,8 @@ int os_send(int soc, void * buf, int len, int opt )
  return n;
 }
 
-int os_recv(int soc, void * buf, int len, int opt )
+int
+os_recv (int soc, void * buf, int len, int opt )
 {
  char * buf0 = (char*)buf;
  int e, n;
@@ -1417,7 +1331,7 @@ int os_recv(int soc, void * buf, int len, int opt )
 
 
 int
-get_and_clear_stream_socket_errno(int fd)
+get_and_clear_stream_socket_errno (int fd)
 {
   nessus_connection	*fp = NULL;
   int		err;
@@ -1458,7 +1372,9 @@ get_and_clear_stream_socket_errno(int fd)
     return err;
 }
 
-ExtFunc int stream_pending(int fd)
+ExtFunc
+int
+stream_pending (int fd)
 {
   nessus_connection * fp;
  if ( ! NESSUS_STREAM(fd) )
@@ -1475,4 +1391,3 @@ ExtFunc int stream_pending(int fd)
 
  return 0;
 }
-
