@@ -34,13 +34,13 @@
  */
  
 #include <includes.h>
-#include <zlib.h>
 #include "globals.h"
 
 #include "nessus.h"
 #include "context.h"
 #include "preferences.h"
 #include "parser.h"
+#include "parseutils.h"
 #include "nessus_plugin.h"
 #include "cli.h"
 
@@ -85,70 +85,6 @@ is_server_present(soc)
 	return 1;
 }
 
-
-
-static harglst* build_plugins_order_table(order)
- char * order;
-{
- int num = 0;
- char * t;
- int i = 0;
- int * plugins_order_table_int;
- harglst * plugins_order_table = harg_create(4000);
-
-
- t = order;
-
- 
- while((t = strchr(t+1, ';')))num++;
- plugins_order_table_int = emalloc((num+1)*sizeof(int));
- t = order;
- if(num)
- while(t)
- {
-  char * next = strchr(t, ';');
-  if(next)next[0]=0;
-  plugins_order_table_int[i++] = atoi(t);
-  if(next)next[0]=';';
-  t = next+1;
-  next = strchr(next+1, ';');
-  if(!next)t = NULL;
- }
- 
- /*
-  * Now, create the names table
-  */
-  plugins_order_table = harg_create(num*3 + 1);
-  
-  for(i=0;i<num;i++)
-  {
-   struct nessus_plugin * plugs = Context->plugins;
-   
-   while(plugs  != NULL  &&
-         plugs->id != plugins_order_table_int[i])
-         plugs = plugs->next;
-
-
-   if( plugs == NULL )
-   {
-    plugs = Context->scanners;
-    while(plugs &&
-	  plugs->id != plugins_order_table_int[i] )
-         	plugs = plugs->next;
-   }
-   if(plugs != NULL ){
-   	char id[32];
-	snprintf(id, sizeof(id), "%d", i+1);
-   	harg_add_string(plugins_order_table, id, plugs->name);
-	efree(&id);
-	}
-  }
-  efree(&plugins_order_table_int);
-  return plugins_order_table;
-}
-
-
-
 void 
 cli_sigterm(s)
  int s;
@@ -156,8 +92,9 @@ cli_sigterm(s)
  cli_report(g_cli);
  exit(5);
 }
-/*
- * Monitor the test - read data from the client, and process it
+
+/**
+ * @brief Monitor the test - read data from the client, and process it
  */
 static void
 cli_test_monitor(cli)
@@ -201,11 +138,6 @@ cli_test_monitor(cli)
 			   char * plug = NULL;
 			  
 			   parse_nessusd_short_status(&(buf[2]), &hostname, &action, &current, &max);
-			   if(cli->plugins_order_table)
-			   {
-			    plug = harg_get_string(cli->plugins_order_table, current);
-			   }
-			   
 			   if(!strcmp(action, "portscan"))plug="";
 			   printf("%s|%s|%s|%d\n", action,hostname,current,max); 
 			   efree(&hostname);
@@ -227,9 +159,6 @@ cli_test_monitor(cli)
 			   efree(&current);
 			   }
 			   break;
-		   case MSG_PLUGINS_ORDER:
-		   	   cli->plugins_order_table = build_plugins_order_table(msg);
-			   break;
 		   case MSG_FINISHED:
 			   printf("finished|%s||||\n", msg);
 			   break;
@@ -327,17 +256,6 @@ cli_args_results(args, results)
  args->extension = strdup(ftype);
 }
 
-void 
-cli_args_cipher(args, cipher)
- struct cli_args * args;
- char * cipher;
-{
- if(args->cipher)free(args->cipher);
- args->cipher = strdup(cipher);
-}
-
-
-
 void 
 cli_args_verbose(args, verbose)
  struct cli_args * args;
@@ -345,24 +263,33 @@ cli_args_verbose(args, verbose)
 { 
  args->verbose = verbose;
 }
-void
-cli_args_auth_pwd(args, auth_pwd)
- struct cli_args * args;
- cli_auth_pwd_t  auth_pwd;
-{
- args->auth_pwd = auth_pwd;
-}
-
 
+/**
+ * Sets the output function pointer in an cli_args.
+ * The output function is chosen according to the type, and currently one of
+ *  - backend_to_nbe (type: "nbe", NULL)
+ *  - arglist_to_html_graph ("html_graph", "html_pie")
+ *  - arglist_to_html ("html")
+ *  - arglist_to_latex ("tex", "latex")
+ *  - arglist_to_text ("txt", "txt")
+ *  - backend_to_xml_ng ("xml")
+ *
+ * @param args The cli_args of interest.
+ * @param type Sets the type of output created when calling the output function
+ *             ("xml", "nbe" etc).
+ * @see backend_to_nbe
+ * @see backend_to_xml_ng
+ * @see arglist_to_latex
+ * @see arglist_to_html
+ * @see arglist_to_html_graph
+ * @see arglist_to_text 
+ */
 void
 cli_args_output(args, type)
  struct cli_args * args;
  char * type;
 {
  char * ftype = args->extension;
-
- 
- 
  
  if(type)ftype = type;
 
@@ -373,13 +300,13 @@ cli_args_output(args, type)
     return;
   }
 
-#ifndef _NO_PIES
+#ifndef NO_GDCHART
  if(!strncmp(ftype, "html_pie", 8)||
      !strncmp(ftype, "html_graph", 10)) {
     args->output = arglist_to_html_graph;
     }
      else 
-#endif /* _NO_PIES */
+#endif /* NO_GDCHART */
      if (!strncmp(ftype, "html", 4)) {
 	args->output = arglist_to_html;
      } else if (!strcmp(ftype, "latex") ||
@@ -456,7 +383,7 @@ cli_report(cli)
  if(!cli->backend_output_func)
     cli->output(backend_convert(cli->backend), cli->results);
  else
-    cli->output((struct arglist*)cli->backend, cli->results);
+    cli->output((struct arglist*)GSIZE_TO_POINTER(cli->backend), cli->results);
 }
 
 
@@ -503,6 +430,7 @@ sql_addslashes(in)
  }
  return realloc(ret, strlen(ret) + 1);
 }
+
 static void 
 _cli_sql_dump_plugins(p)
  struct nessus_plugin * p;
@@ -511,7 +439,7 @@ _cli_sql_dump_plugins(p)
  {
  char * m, * n;
 
- printf("INSERT INTO plugins VALUES ('%d', ", p->id);
+ printf("INSERT INTO plugins VALUES ('%s', ", p->oid);
 
 
  m = sql_addslashes(p->name);
@@ -572,13 +500,14 @@ _cli_sql_dump_plugins(p)
  p = p->next;
  }
 }
+
 static void
 _cli_dump_plugins(p)
  struct nessus_plugin * p;
 {
  while( p != NULL )
  {
-  char * var = p->asc_id;
+  char * var = p->oid;
   char * n;
 
   printf("%s|", var);
@@ -630,9 +559,9 @@ void
 cli_sql_dump_plugins(cli)
  struct cli_args * cli;
 {
- printf("DROP TABLE plugins;\n");
+ printf("DROP TABLE IF EXISTS plugins;\n");
  printf("CREATE TABLE plugins (\n");
- printf(" id int NOT NULL,\n");
+ printf(" oid varchar(50) NOT NULL,\n");
  printf(" name varchar(255),\n");
  printf(" family varchar(255),\n");
  printf(" category varchar(255),\n");
@@ -643,7 +572,7 @@ cli_sql_dump_plugins(cli)
  printf(" cve_id varchar(255),\n");
  printf(" bugtraq_id varchar(255),\n");
  printf(" xref blob,\n");
- printf(" primary key (id));\n");
+ printf(" primary key (oid));\n");
  _cli_sql_dump_plugins(Context->plugins);
  _cli_sql_dump_plugins(Context->scanners);
 }
@@ -703,7 +632,7 @@ cli_dump_prefs(cli)
   switch(p->type)
   {
    case ARG_INT :
-    printf("%s = %d\n", p->name, (int)p->value);
+    printf("%s = %d\n", p->name, (int)GPOINTER_TO_SIZE(p->value));
     break;
    case ARG_STRING:
    printf("%s = %s\n", p->name, (char*)p->value);
@@ -718,6 +647,7 @@ cli_dump_prefs(cli)
  
  return;
 }
+
 int 
 cli_close_connection(cli)
  struct cli_args * cli;
@@ -734,9 +664,9 @@ cli_restore_session(cli, session)
   restore_attack(session, Context);
   cli_test_monitor(cli);
 }
+
 void
-cli_list_sessions(cli)
- struct cli_args * cli;
+cli_list_sessions(struct cli_args* cli)
 {
  hargwalk * hw;
  if(!comm_server_restores_sessions(Context))
@@ -753,6 +683,6 @@ cli_list_sessions(cli)
    {
     printf("%s | %s\n", key, harg_get_string(Context->sessions, key));
    }
-  } 
+  }
 }
 #endif
