@@ -26,6 +26,10 @@
 #include "error_dlg.h"
 #include "plugin_cache.h"
 #include "comm.h"
+#include "hash_table_file.h"
+#include "openvas_certificate_file.h"
+#include "openvas_ssh_login.h"
+#include "openvas_ssh_key_create.h"
 
 #ifdef USE_GTK
 #include <gtk/gtk.h>
@@ -34,10 +38,16 @@
 struct context *Global;
 struct context *Context;
 
+/**
+ * Initializes a Context child of a parent Context, including memory
+ * allocation.
+ * The context_type will be the parents context_type +1.
+ * 
+ * @param context Double pointer to child (memory will be allocated).
+ * @param parent The parent context.
+ */
 void
-context_init(context, parent)
-  struct context **context;
-  struct context *parent;
+context_init (struct context **context, struct context *parent)
 {
   *context = emalloc(sizeof(struct context));
   (*context)->parent = parent;
@@ -60,14 +70,17 @@ context_init(context, parent)
   (*context)->pbar = NULL;
   (*context)->plugin_tree_store = NULL;
   (*context)->plugin_tree_model = NULL;
-#endif  
+  (*context)->gui_sshlogins_per_target = NULL;
+#endif
   (*context)->plugin_cache_loaded = 0;
+  (*context)->signer_fp_certificates = NULL;
+  (*context)->sshkeys = NULL;
+  (*context)->map_target_sshlogin = NULL;
+  (*context)->is_severity_mapped = FALSE;
 }
 
-struct context *
-context_by_type(context, type)
-  struct context *context;
-  context_type type;
+struct context*
+context_by_type (struct context *context, context_type type)
 {
   while(context && context->parent && context->type > type)
     context = context->parent;
@@ -79,11 +92,13 @@ context_by_type(context, type)
 }
 
 
-/* reset the tree store and model for the plugins.
+/**
+ * @brief Resets the tree store and model for the plugins.
+ * 
  * When compiling without USE_GTK, this function does nothing.
  */
 void
-context_reset_plugin_tree(struct context *context)
+context_reset_plugin_tree (struct context *context)
 {
 #ifdef USE_GTK
   if (context->plugin_tree_store != NULL)
@@ -99,9 +114,11 @@ context_reset_plugin_tree(struct context *context)
 #endif
 }
 
-/* Force a redraw of the plugin prefs widgets.  The redraw is not
- * directly here.  It will happen as soon as prefs_plugins_prefs_redraw
- * is called.
+/**
+ * @brief Force a redraw of the plugin prefs widgets.
+ * 
+ * The redraw is not directly here. It will happen as soon as 
+ * prefs_plugins_prefs_redraw is called.
  * When compiling without USE_GTK, this function does nothing.
  */
 void
@@ -120,9 +137,11 @@ context_force_plugin_prefs_redraw(struct context *context)
 }
 
 
-/* Reset the plugin information of the context */
+/**
+ * @brief Reset the plugin information of the context.
+ */
 void
-context_reset_plugins(struct context *context)
+context_reset_plugins (struct context *context)
 {
   nessus_plugin_free(context->plugins);
   nessus_plugin_free(context->scanners);
@@ -132,59 +151,70 @@ context_reset_plugins(struct context *context)
   context_reset_plugin_tree(context);
 }
 
-/*
- * Add a plugin to the context.  If the plugin is a scanner, it's added
- * to context->scanners and the corresponding scanner set.  Otherwise
- * it's a normal plugin and is added to context->plugins list and the
+/**
+ * @brief Add a plugin to the context.
+ * 
+ * If the plugin is a scanner, it's added to context->scanners and the
+ * corresponding scanner set.
+ * Otherwise it's a normal plugin and is added to context->plugins list and the
  * plugin set.
+ * In case that the plugin can not be found in the cache and is not a scanner
+ * it is added in an en/disabled state according to the 
+ * "auto_enable_new_plugins" preference for the scope at hand.
  *
  * XXX: do we need hashing for pluginset?
+ * @param context Cotnext to which to add the plugin to.
+ * @param plugin Plugin to add to context.
+ * 
+ * @return 1 if the added plugin was known before (was in cache), 0 otherwise.
  */
-void
-context_add_plugin(struct context *context, struct nessus_plugin *plugin)
+int
+context_add_plugin (struct context *context, struct nessus_plugin *plugin)
 {
   char *category = plugin->category;
-  char *asc_id =   plugin->asc_id;
+  char *oid =   plugin->oid;
   int is_scanner = (strcmp(category, "scanner") == 0);
   struct nessus_plugin *plugins = is_scanner ? context->scanners
 					     : context->plugins;
   struct arglist *pluginset = prefs_get_pluginset(context,
       is_scanner ? "SCANNER_SET" : "PLUGIN_SET", NULL);
-  int in_pluginset = (int)(arg_get_type(pluginset, asc_id) >= 0);
+  int in_pluginset = (int)(arg_get_type(pluginset, oid) >= 0);
   int enabled = 0;
 
+  /* Already known, enabled plugins are added in enabled state */
   if (in_pluginset)
   {
-    if (arg_get_value(pluginset, asc_id))
+    if (arg_get_value(pluginset, oid))
       enabled = 1;
   }
+  /* Unknown plugins are added in enabled state depending on a scope pref.    */
   else if (!is_scanner)
-    enabled = 1;
+    enabled = prefs_get_int(context, "auto_enable_new_plugins");
 
   plugin->enabled = enabled;
   plugin->next = plugins;
 
   if (!in_pluginset)
-    arg_add_value(pluginset, asc_id, ARG_INT, sizeof(int), (void *)enabled);
+    arg_add_value(pluginset, oid, ARG_INT, sizeof(gpointer), GSIZE_TO_POINTER(enabled));
 
   if (is_scanner)
     context->scanners = plugin;
   else
     context->plugins = plugin;
+  /* Return whether the just added plugin was known from the cache before  */
+  return (!in_pluginset && !is_scanner);
 }
 
 
 void
-context_set_plugins_md5sum(struct context *context, const char *md5sum)
+context_set_plugins_md5sum (struct context *context, const char *md5sum)
 {
   efree(&context->plugins_md5sum);
   context->plugins_md5sum = estrdup(md5sum);
 }
 
 struct context **
-context_find_child_ptr(context, child)
-  struct context *context;
-  struct context *child;
+context_find_child_ptr (struct context *context, struct context *child)
 {
   struct context **ptr;
 
@@ -201,8 +231,7 @@ context_find_child_ptr(context, child)
 }
 
 struct context *
-context_create_child(context)
-  struct context *context;
+context_create_child (struct context *context)
 {
   struct context **child_ptr = context_find_child_ptr(context, NULL);
 
@@ -211,9 +240,7 @@ context_create_child(context)
 }
 
 void
-context_remove_child(context, child)
-  struct context *context;
-  struct context *child;
+context_remove_child (struct context* context, struct context* child)
 {
   struct context **child_ptr = context_find_child_ptr(context, child);
   struct context *next = child->next;
@@ -227,10 +254,118 @@ context_remove_child(context, child)
   *child_ptr = next;
 }
 
+/**
+ * @brief Warns the user if a login is invalid (e.g. misses key file) and asks
+ *        how to solve the situation.
+ * 
+ * Used in a g_hash_table_foreach to check all logins for validity.
+ * Adds keys that should be removed to a list.
+ * 
+ * @param accountname[in] User-defined name of a openvas_ssh_login (key of hashtable).
+ * @param login[in]       User-defined ssh login.
+ * @param removelist[out] List to that keys to removed get appended.
+ */
+static void
+verify_sshlogin_integrity (char* accountname, openvas_ssh_login* login,
+                           GSList** removelist)
+{
+  #ifdef USE_GTK
+  if (login && login->valid == FALSE)
+    {
+      GtkWidget* dialog = gtk_message_dialog_new (NULL, 
+                            GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
+                            GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
+                            _("SSH Login '%s' misses files.\nDelete this account and all information or recreate the files?"),
+                             accountname);
+      gtk_dialog_add_buttons (GTK_DIALOG(dialog), _("Delete Account"), 12,
+                                                  _("Recreate Files"), 13, NULL);
+
+      gtk_widget_show_all (dialog);
+
+      int response = gtk_dialog_run (GTK_DIALOG(dialog));
+
+      if (response == 12) // "Delete"
+        {
+          (*removelist) = g_slist_prepend ((*removelist), g_strdup(accountname));
+        }
+      else if (response == 13) // "Create new"
+        {
+          openvas_ssh_login* login = g_hash_table_lookup (Global->sshkeys, accountname);
+          openvas_ssh_key_create_unlink_files (login);
+          if (openvas_ssh_key_create (login) == FALSE)
+            show_warning (_("Recreation of SSH login '%s' failed.\nCheck your console and remove or repair this login manually."), accountname);
+          else
+            show_info (_("Recreation of SSH login '%s' successful."), accountname);
+        }
+      else
+        {
+          // Seems that dialog (or main-gui) was simply closed, do nothing.
+          // Put notification into the log message area.
+        }
+
+      gtk_widget_destroy (dialog);
+    }
+  #else
+    if (login && login->valid == FALSE)
+      show_warning (_("SSH Login %s seems to miss files. (The GUI can help you to repair or delete it)"), login->name);
+  #endif
+}
+
+/**
+ * @brief Load all ssh login information from the directory sshdir to the 
+ *        contexts sshkeys hashtable.
+ * 
+ * sshdir usually points to .openvas/contex/scope/.ssh.
+ * For each invalid key (public or private keyfile is missing) the user will be
+ * asked whether to delete or recreate the account.
+ * 
+ * @param context The context that shall get the sshkey filenames added.
+ * @param sshdir Directory to scan for files.
+ */
+static void
+context_pickup_sshkeys (struct context* context, const char* sshdir)
+{
+  gchar* loginsfile = g_build_filename(sshdir, ".logins", NULL);
+  GHashTable* loginfos = openvas_ssh_login_file_read (loginsfile, FALSE);
+  GSList* invalidkeylist = NULL;
+  GSList* listit = NULL;
+
+  if(Global->sshkeys != NULL)
+    g_hash_table_destroy(Global->sshkeys);
+
+  context->sshkeys = loginfos;
+
+  // Check if all logins are valid (files exist).
+  g_hash_table_foreach (loginfos, (GHFunc) verify_sshlogin_integrity, &invalidkeylist);
+
+  // Remove the keys that the user wanted to destroy
+  if (invalidkeylist != NULL)
+    {
+      listit = invalidkeylist;
+      while (listit != NULL)
+        {
+          openvas_ssh_key_create_undo (g_hash_table_lookup (Global->sshkeys, listit->data));
+          free (listit->data);
+          listit = listit->next;
+        }
+      g_slist_free (invalidkeylist);
+      // Rewrite the file, to remove eventually deleted information immidiately.
+      openvas_ssh_login_file_write(Global->sshkeys, loginsfile);
+    }
+
+  g_free (loginsfile);
+}
+
+/**
+ * Collects contexts from a directory (recursively descends in the directory).
+ * Contexts are recognized by a openvasrc or report.nbe file, initialized and
+ * added as childs to the context given as parameter "context".
+ * 
+ * @param context The to-be-parent context of found contexts.
+ * @param dir Directory to start looking for contexts.
+ */
 void
-context_collect_recurse(context, dir)
-  struct context *context;
-  const char *dir;
+context_collect_recurse (struct context *context, const char* dir)
 {
   DIR *odir;
   struct dirent *dirent;
@@ -248,18 +383,36 @@ context_collect_recurse(context, dir)
 
       path = g_build_filename(dir, file, NULL);
 
-      if(context->type < CONTEXT_REPORT && check_is_dir(path))
+      /* 5 possible cases: .ssh directory, child directory, report or rc file
+         found, .host_logins and "no hit" */
+      if(check_is_dir(path) && strcmp(file, ".ssh") == 0)
+      {
+        // Just the global context respects .ssh subfolders
+        if( context->type == CONTEXT_GLOBAL)
+          context_pickup_sshkeys(context, path);
+      }
+      else if(context->type < CONTEXT_REPORT && check_is_dir(path))
       {
 	struct context *child = context_create_child(context);
 
 	child->dir = estrdup(path);
-	context_collect_recurse(child, path);
+	context_collect_recurse (child, path);
 	if(!child->prefs && !child->children)
 	  context_remove_child(context, child);
       }
-      else if((!strcmp(file, "nessusrc") && context->type < CONTEXT_REPORT)
-	|| (!strcmp(file, "report.nbe") && context->type == CONTEXT_REPORT))
+      else if((!strcmp(file, "openvasrc") && context->type < CONTEXT_REPORT)
+	    || (!strcmp(file, "report.nbe") && context->type == CONTEXT_REPORT))
+      {
 	context_found = TRUE;
+      }
+      else if( !strcmp(file, ".host_sshlogins") )
+      {
+        context->map_target_sshlogin = hash_table_file_read (path);
+      }
+      if(context->type == CONTEXT_REPORT && strcmp(file, "certificates") == 0)
+      {
+        context->signer_fp_certificates = openvas_certificate_file_read(path);
+      }
       g_free(path);
       path = NULL;
     }
@@ -269,21 +422,22 @@ context_collect_recurse(context, dir)
   }
 }
 
+/**
+ * @brief Recursively collects contexts found in and under the nessus_dir.
+ * 
+ * @param context Is queried for the nessus_dir preference.
+ */
 void
-context_collect(context)
-  struct context *context;
+context_collect (struct context *context)
 {
   const char *dir = estrdup(prefs_get_string(context, "nessus_dir"));
 
-  context_collect_recurse(context, dir, 0);
+  context_collect_recurse (context, dir);
 }
 
 
 char *
-context_newpath(oldpath, dir, name)
-  const char *oldpath;
-  const char *dir;
-  const char *name;
+context_newpath (const char *oldpath, const char *dir, const char *name)
 {
   char *newname = estrdup(name);
   char *pos = newname;
@@ -331,8 +485,7 @@ context_newpath(oldpath, dir, name)
 }
 
 void
-context_update_children_dirs(context)
-  struct context *context;
+context_update_children_dirs (struct context *context)
 {
   char *newpath;
 
@@ -350,9 +503,7 @@ context_update_children_dirs(context)
 }
 
 void
-context_rename(context, newname)
-  struct context *context;
-  const char *newname;
+context_rename (struct context *context, const char *newname)
 {
   char *newpath;
   char *parentdir;
@@ -394,9 +545,7 @@ context_rename(context, newname)
 }
 
 int
-context_move(context, new_parent)
-  struct context *context;
-  struct context *new_parent;
+context_move (struct context *context, struct context *new_parent)
 {
   struct context **child_ptr = context_find_child_ptr(context->parent, context);
   struct context **new_ptr = context_find_child_ptr(new_parent, NULL);
@@ -430,11 +579,19 @@ context_move(context, new_parent)
   return 0;
 }
 
+/**
+ * @brief Creates and initializes a context.
+ * 
+ * @param parent   New contexts parent.
+ * @param name     (Optional) name of the new context, if NULL will be set to
+ *                 unnamed_(scope|task).
+ * @param filename (Optional) name of the openvasrc file which can be used to
+ *                 initialize the new context.
+ * 
+ * @return Fresh context.
+ */
 struct context*
-context_new(parent, name, filename)
-  struct context *parent;
-  const char *name;
-  const char *filename;
+context_new (struct context* parent, const char* name, const char* filename)
 {
   const char *dir;
   char *context_dir;
@@ -496,13 +653,17 @@ context_new(parent, name, filename)
 
   preferences_generate_new_file(context, name);
 
+  context->signer_fp_certificates = g_hash_table_new_full(g_str_hash, 
+                                    g_str_equal, NULL, (GDestroyNotify) openvas_certificate_free);
+  context->map_target_sshlogin =  g_hash_table_new_full (g_str_hash, g_str_equal,
+                                                        NULL, NULL);
+
   return context;
 }
 
 
 void
-context_delete_directory(dir)
-  const char *dir;
+context_delete_directory (const char* dir)
 {
   DIR *odir;
   struct dirent *dirent;
@@ -535,8 +696,7 @@ context_delete_directory(dir)
 }
 
 void
-context_delete(context)
-  struct context *context;
+context_delete (struct context *context)
 {
   while(context->children)
     context_delete(context->children);
@@ -547,6 +707,10 @@ context_delete(context)
   arg_free_all(context->dependencies);
   context->dependencies = NULL;
   context_delete_directory(context->dir);
+  if(context->signer_fp_certificates)
+    g_hash_table_destroy(context->signer_fp_certificates);
+  if (context->map_target_sshlogin)
+    g_hash_table_destroy (context->map_target_sshlogin);
   context_remove_child(context->parent, context);
   if(context == Context)
   {
@@ -555,9 +719,16 @@ context_delete(context)
   }
 }
 
+/**
+ * @brief Recurses throught contexts and saves preferences and ssh login
+ *        information.
+ * 
+ * This function should be called just before OpenVAS-client quits.
+ * 
+ * @param context The context to start with (usually Global).
+ */
 void
-context_save_recurse(context)
-  struct context *context;
+context_save_recurse (struct context *context)
 {
   struct context *child = context->children;
 
@@ -566,15 +737,28 @@ context_save_recurse(context)
     context_save_recurse(child);
     child = child->next;
   }
+  // Save the target_sshlogin map of Tasks and Reports
+  if (context->map_target_sshlogin && context->type >= CONTEXT_SCOPE)
+  {
+    char* fileloc = g_build_filename (context->dir, ".host_sshlogins", NULL);
+    gboolean success = hash_table_file_write (context->map_target_sshlogin, fileloc);
+    if (success == FALSE)
+      {
+        show_warning (_("Could not save ssh-login selection per target!\n"));
+      }
+    efree (&fileloc);
+  }
+
   preferences_save(context);
 }
 
 
-/* Fill the plugin preferences from the information read from the
- * report's nessusrc.
+/**
+ * @brief Fill the plugin preferences from the information read from the
+ * @brief report's nessusrc.
  */
 static void
-fill_plugin_prefs(struct context *context)
+fill_plugin_prefs (struct context *context)
 {
   struct arglist *pref = arg_get_value(context->prefs, "PLUGINS_PREFS");
   while (pref && pref->next)
@@ -600,12 +784,14 @@ fill_plugin_prefs(struct context *context)
 }
 
 
-/* Load the plugin cache of the context if it has one, the context
- * doesn't have plugin information yet and the user actually wants to
+/**
+ * @brief Load the plugin cache of the context if it has one.
+ * 
+ * The context doesn't have plugin information yet and the user actually wants to
  * load plugin information for the given type of context.
  */
 void
-context_load_plugin_cache(struct context *context)
+context_load_plugin_cache (struct context *context)
 {
   if (context->plugins == NULL && !context->plugin_cache_loaded
       &&
@@ -621,8 +807,10 @@ context_load_plugin_cache(struct context *context)
   }
 }
 
-/* This function sync the plugin preferences (for plugins and scanners)
- * for the given context.
+/**
+ * @brief Syncs the plugin preferences (for plugins and scanners) for the given 
+ *        context.
+ * 
  * In fact, the plugin preferences are copied from
  * context->plugins[plugin][plugin-pref]
  * and
@@ -631,8 +819,7 @@ context_load_plugin_cache(struct context *context)
  * context->prefs["PLUGINS_PREFS"][plugin]
  */
 void
-context_sync_plugin_prefs(context)
-  struct context * context;
+context_sync_plugin_prefs (struct context* context)
 {
   struct arglist * plugins_prefs = arg_get_value(context->prefs, "PLUGINS_PREFS");
   struct nessus_plugin * plugins[2];
@@ -696,16 +883,15 @@ context_sync_plugin_prefs(context)
 }
 
 
-/*
- * replacements for g_file_test which is unreliable on windows
+/**
+ * Replacement for g_file_test which is unreliable on windows
  * if nessus and gtk are compiled with a different libc.
  *
  * FIXME: handle symbolic links
+ * @return 1 if file exists, 0 otherwise.
  */
-
 int
-check_exists(name)
-  const char *name;
+check_exists (const char *name)
 {
   struct stat sb;
 
@@ -715,9 +901,13 @@ check_exists(name)
     return 1;
 }
 
+/**
+ * Replacement for g_file_test which is unreliable on windows
+ * if nessus and gtk are compiled with a different libc.
+ * FIXME: handle symbolic links
+ */
 int
-check_is_file(name)
-  const char *name;
+check_is_file (const char *name)
 {
   struct stat sb;
 
@@ -727,9 +917,15 @@ check_is_file(name)
     return(S_ISREG(sb.st_mode));
 }
 
+/**
+ * Replacement for g_file_test which is unreliable on windows
+ * if nessus and gtk are compiled with a different libc.
+ *
+ * FIXME: handle symbolic links
+ * @return 0 if parameter is directory, 1 if it is not (or does not exist).
+ */
 int
-check_is_dir(name)
-  const char *name;
+check_is_dir (const char* name)
 {
   struct stat sb;
 
