@@ -29,9 +29,11 @@
 #include <glib.h>
 #endif
 
+#include "openvas_certificates.h"
+
 typedef enum
 {
-  CONTEXT_GLOBAL = 0,
+  CONTEXT_GLOBAL = 0, /**< The GLOBAL scope is parent of all other scopes and acts to define defaults. */
   CONTEXT_TASK,
   CONTEXT_SCOPE,
   CONTEXT_REPORT,
@@ -44,7 +46,20 @@ typedef enum {
   CONTEXT_SCANNING
 } context_action;
 
-
+/**
+ * @brief A context bundles information and Gtk-Widgets (if USE_GTK).
+ * 
+ * Basically everything that needs to be accessed from any point in the client
+ * is in here.
+ * It implements a tree and a list, as it has a parent, a child and a next.
+ * So a contexts children are its primary child and the 'next's of its primary
+ * child.
+ * Practically, as you decend the tree, you also descend in the context_type
+ * (root is CONTEXT_GLOBAL, roots children are CONTEXT_TASKs and so on).
+ * 
+ * The currently active context can be normally accessed as through the global
+ * variable Context.
+ */
 struct context {
   context_type type;
   context_action action;
@@ -73,12 +88,33 @@ struct context {
   GtkWidget *pbar;
   GtkTreeStore *plugin_tree_store;
   GtkTreeModel *plugin_tree_model;
-#endif
-  /* reports may have plugin information too.  They can be quite large,
-   * so we avoid loading them.  This flag indicates whether the plugin
-   * information has been loaded.
-   */
+  // The PREF_SSH_CREDENTIALS preference of a target is handled differently than
+  // other preferences. Since the gui for it is flexible (depends on managed
+  // ssh- accounts and defined targets) we need to access it more often then
+  // other nvt-preference gui elements. Walking through two arglists each time
+  // seems to be an overkill, so the widget is hardwired in a context.
+  /** GtkTable with logins per target selection, if triggered by a nvt. */
+  GtkTable* gui_sshlogins_per_target;
+#endif /* USE_GTK */
+
+  /** Maps openvas_certificate* (value) to their fingerprints (key). */
+  GHashTable* signer_fp_certificates;
+
+  /** Maps names of ssh public key information bundles to openvas_ssh_logins */
+  GHashTable* sshkeys;
+  
+  /** Maps target names to pointers to ssh_logins. */
+  GHashTable* map_target_sshlogin;
+
+  /** Indicates whether the plugin information has been loaded. Useful because 
+    * Reports may have plugin information too. They can be quite large, and we
+    * want to avoid loading them. */
   int plugin_cache_loaded;
+
+  /** Indicates wether a severity_filter was applied to a report (this flag
+   * makes only sense for reports).
+   */
+  gboolean is_severity_mapped;
 };
 
 extern struct context *Global;
@@ -88,20 +124,11 @@ void context_init(struct context**, struct context*);
 struct context *context_by_type(struct context*, context_type);
 void context_reset_plugin_tree(struct context*);
 void context_force_plugin_prefs_redraw(struct context *context);
-void context_add_plugin(struct context*, struct nessus_plugin*);
+int context_add_plugin(struct context*, struct nessus_plugin*);
 void context_load_plugin_cache(struct context *context);
 void context_set_plugins_md5sum(struct context *context, const char *md5sum);
 void context_reset_plugins(struct context *context);
 
-/* This function sync the plugin preferences (for plugins and scanners)
- * for the given context.
- * In fact, the plugin preferences are copied from
- * context->plugins[plugin][plugin-pref]
- * and
- * context->scanners->[plugin][plugin-pref]
- * to
- * context->prefs["PLUGINS_PREFS"][plugin]
- */
 void context_sync_plugin_prefs(struct context *);
 
 void context_collect(struct context*);
@@ -111,7 +138,7 @@ struct context *context_new(struct context*, const char*, const char*);
 void context_delete(struct context*);
 void context_save_recurse(struct context*);
 
-
+/* TODO move to e.g. file_check.c */
 int check_exists(const char *);
 int check_is_file(const char *);
 int check_is_dir(const char *);
