@@ -64,15 +64,19 @@
 	- HTML report
 
  -------------------------------------------------------------------*/	
+
+
 #include <includes.h>
 #include "nessus_i18n.h"
-#include "gdchart0.94b/gdc.h"
-#include "gdchart0.94b/gdchart.h"
-#include "gdchart0.94b/gdcpie.h"
+#ifndef NO_GDCHART
+#include <gdc.h>
+#include <gdchart.h>
+#include <gdcpie.h>
 #include "report.h"
 #include "report_utils.h"
 #include "error_dlg.h"
 #include "preferences.h"
+#include "html_output.h"
 
 
 static void insert_img(FILE *, char*);
@@ -490,242 +494,6 @@ most_used_operating_systems(hosts, names, values, n)
 			
 -----------------------------------------------------------------------------*/
 
-/*
- * print URL linking to CVE/BID/Nessus database to file.
- * '%s' in urlfmt (from preferences) will be replaced by
- * item IDs (or their parts split at minus characters).
- * '%%' will insert a percent character.
- * Everything else after '%' aborts parsing.
- * If urlfmt yields an empty string (e.g. it is only "%"),
- * only the item ID will be printed.
- */
-static void
-fprint_link(file, key, item)
-  FILE *file;
-  const char *key;
-  const char *item;
-{
-  char *itemparts = estrdup(item);
-  char *itemsrc = itemparts;
-  char *urlfmt = estrdup(prefs_get_string(Global, key));
-  char *src = urlfmt;
-  char *url = emalloc(strlen(urlfmt)+strlen(item)+1);
-  char *dst = url;
-  char *tmp;
-
-  while((tmp = strchr(src, '%')))
-  {
-    *tmp = '\0';
-    strcpy(dst, src);
-    dst += strlen(src);
-    src = tmp+1;
-    switch(*src)
-    {
-      case 's':
-	if((tmp = strchr(itemsrc, '-')))
-	  *tmp = '\0';
-	strcpy(dst, itemsrc);
-	dst += strlen(itemsrc);
-	if(tmp)
-	  itemsrc = tmp+1;
-	src++;
-	break;
-      case '%':
-	strcpy(dst++, "%");
-	src++;
-	break;
-      default:
-	*src = '\0';
-    }
-  }
-  strcpy(dst, src);
-
-  efree(&itemparts);
-  efree(&urlfmt);
-  if(*url)
-    fprintf(file, "<a href=\"%s\">%s</a>", url, item);
-  else
-    fprintf(file, "%s", item);
-  efree(&url);
-}
-
-/* All the cross references (CVE, BID) have the same format - XREF: <num>,...<br> */
-static char * 
-extract_xref(file, str, key)
- FILE * file;
- char * str, * key;
-{
- while(str != NULL && strncmp(str, "<br>", 4) != 0)
-   {
-    char * e1 = strchr(str, ',');
-    char * e2 = strchr(str, '<');
-    char tmp = '\0';
-    if((e1 > e2) || (e1 == NULL))e1 = e2;
-   
-   
-    if(e1 != NULL)
-    {
-     tmp = e1[0];
-     e1[0] = '\0';
-    }
-    fprint_link(file, key, str);
-    str = e1;
-    if(e1 != NULL)
-    {
-     e1[0] = tmp;
-   
-     if(tmp == ','){
-     	fputc(',', file);
-	fputc(' ', file);
-	str ++;
-	str ++;
-	}
-     else
-        fputc('<', file);
-    }
-   }
-  return str;
-}
- 
-static void 
-print_data_with_links(file, str, plugin_id)
- FILE * file;
- char * str, * plugin_id;
-{
- while(str != NULL && str[0] != '\0')
- {
-  if(strncmp(str, "http:", 5) == 0 || strncmp(str, "https:", 6) == 0 )
-  {
-   char * e1, * e2;
-   char tmp = 0;
-   
-   e1 = strchr(str, ' ');
-   e2 = strstr(str, "<br>");
-   if((e1 > e2) || (e1 == NULL))e1 = e2;
-   
-   if(e1 != NULL)
-   {
-    tmp = e1[0];
-    e1[0] = '\0';
-   }
-   fprintf(file, "<a href=\"%s\">%s</a>", str, str);
-   str += strlen(str) - 1;
-   if(e1 != NULL)
-   {
-    e1[0] = tmp;
-   }
-  }
-  else if(strncmp(str, "CVE : ", 6) == 0)
-  {
-   fprintf(file, "CVE : ");
-   str += 6;
-   str = extract_xref(file, str, "url_cve");
-  }
-  else if(strncmp(str, "BID : ", 6) == 0)
-  {
-  fprintf(file, "BID : ");
-  str += 6;
-  str = extract_xref(file, str, "url_bid");
-  }
-  else fputc(str[0], file);
-  if(str != NULL)
-    str++;
- }
- 
- fprintf(file, "Nessus ID : ");
- fprint_link(file, "url_nessus", plugin_id);
-}
-
-static char * convert_cr_to_html(str)
- char * str;
-{
- int num = 0;
- char * t;
- char * ret;
- int i, j = 0;
- /*
-  * Compute the size we'll need
-  */
-  
-  t = str;
-  while(t[0])
-  {
-   if((t[0]=='\n')||(t[0]=='>')||(t[0]=='<'))num++;
-   t++;
-  }
- 
-  ret = emalloc(strlen(str)+5*num+1);
-  for(i=0, j=0;str[i];i++,j++)
-  {
-   if(str[i]=='\n'){
-   	ret[j++]='<';
-	ret[j++]='b';
-	ret[j++]='r';
-	ret[j++]='>';
-	ret[j]='\n';
-	}
-   else if(str[i]=='>') {
-    	ret[j++]='&';
-	ret[j++]='g';
-	ret[j++]='t';
-	ret[j]=';';
-	}
-  else if(str[i]=='<')
-  	{
-	ret[j++]='&';
-	ret[j++]='l';
-	ret[j++]='t';
-	ret[j]=';';
-	}
-  else ret[j] = str[i];
-  }
-  return ret;
-}
-
-
-   
-static char * portname_to_ahref(name, hostname)
- char * name;
- char * hostname;
-{
-  char *t, *k;
-
-  /*
-   * Convert '192.168.1.1' to '192_168_1_1' or
-   * 'prof.openvas.org' to 'prof_openvas_org'
-   */
-  hostname = 
-    t = estrdup (hostname) ;
-  while ((t = strchr (t, '.')) != 0)
-    t [0] = '_' ;
-  if (name == 0)
-    return hostname ;
-
-  /*
-   * Convert 'telnet (21/tcp)' to '21_tcp'
-   */
-  name =
-    k = estrdup (name);
-  if ((t = strrchr (k, '(')) != 0) 
-    k = t + 1;
-  if ((t = strchr (k, ')')) != 0)
-    t [0] = '\0' ;
-  while ((t = strchr (k, '/')) != 0)
-    t [0] = '_' ;
- 
-  /*
-   * append: "name" + "_" + "hostname"
-   */
-  t = emalloc (strlen (hostname) + strlen (k) + 15);
-  strcat (strcat (strcpy (t, hostname), "_"), k);
-  efree (&hostname);
-  efree (&name);
-  return t ;
-}
-
-
-  
-
 static int 
 host_details(file, hosts)
  FILE * file;
@@ -757,7 +525,7 @@ host_details(file, hosts)
    fprintf(file, "[<a href=\"../index.html\">Back to the index</a>]<p>\n");
  hostname = hosts->name;
  ports = arg_get_value(hosts->value, "PORTS");
- name = portname_to_ahref("toc", hostname);
+ name = html_output_portname_to_ahref("toc", hostname);
  fprintf(file, "<a name=\"%s\"></a>\n", name);
  efree(&name);
 
@@ -771,20 +539,22 @@ host_details(file, hosts)
  
     fprintf(file, "<ul><ul><i>\n");
     while(open && open->next){
-    	name = portname_to_ahref(open->name, hostname);
+    	name = html_output_portname_to_ahref(open->name, hostname);
 	if(name)
 	{
 	if(arg_get_value(open->value, "REPORT") ||
 	   arg_get_value(open->value, "INFO") ||
+	   arg_get_value(open->value, "FALSE") ||
 	   arg_get_value(open->value, "NOTE"))
 	   {
 	     fprintf(file, "<li><a href=\"#%s\">%s</a>\n",
 	   		name, open->name);
 	     if(arg_get_value(open->value, "REPORT")){
-	      fprintf(file, "<FONT COLOR=\"#FF0000\"> (Security hole found)</FONT>\n");
+	      fprintf(file, "<FONT COLOR=\"#FF0000\"> (Security hole(s) found)</FONT>\n");
 	     }		
-	     else if (arg_get_value(open->value, "INFO")) fprintf(file, "<FONT COLOR=\"#660000\"> (Security warnings found)</FONT>\n");
-	     else fprintf(file, " (Security notes found)\n");
+	     else if (arg_get_value(open->value, "INFO")) fprintf(file, "<FONT COLOR=\"#660000\"> (Security warning(s) found)</FONT>\n");
+	     else if (arg_get_value(open->value, "FALSE")) fprintf(file, "<FONT COLOR=\"#660000\"> (False positive(s) found)</FONT>\n");
+	     else fprintf(file, " (Security note(s) found)\n");
 	   }	 
 	 else		  
 		fprintf(file, "<li>%s\n", open->name);
@@ -803,11 +573,12 @@ host_details(file, hosts)
     struct arglist * report;
     struct arglist * info;
     struct arglist * note;
+    struct arglist * false_positives;
     port = ports->name;
     report = arg_get_value(ports->value, "REPORT");
     if(report)while(report && report->next)
      {
-     char * name = portname_to_ahref(ports->name, hostname);
+     char * name = html_output_portname_to_ahref(ports->name, hostname);
      fprintf(file, "<a name=\"%s\"></a>",name);
      efree(&name);
      if(strlen(report->value))
@@ -816,8 +587,8 @@ host_details(file, hosts)
      /*
       * Convert the \n to <p> 
       */
-     desc = convert_cr_to_html(report->value);
-     name = portname_to_ahref("toc", hostname);
+     desc = html_output_convert_cr_to_html(report->value);
+     name = html_output_portname_to_ahref("toc", hostname);
           
      fprintf(file, "<font size=\"-1\"><div align=\"right\"><a href=\"#%s\">\
 [ back to the list of ports ]</a></div></font><p>\n",
@@ -825,7 +596,7 @@ host_details(file, hosts)
       efree(&name);		
       fprintf(file, "<FONT COLOR=\"#FF0000\">");
       fprintf(file, "<b>Vulnerability found on port %s</b></FONT><p><ul>\n",port);
-      print_data_with_links(file, desc, report->name);
+      html_output_print_data_with_links(file, desc, report->name, FALSE);
       fprintf(file, "<p></ul>\n");
       efree(&desc);
      }
@@ -836,7 +607,7 @@ host_details(file, hosts)
    if(info)while(info && info->next)
     {
      if(!report){
-      	char * name = portname_to_ahref(ports->name, hostname);
+      	char * name = html_output_portname_to_ahref(ports->name, hostname);
      	fprintf(file, "<a name=\"%s\"></a>",name);
 	efree(&name);
 	}
@@ -848,15 +619,15 @@ host_details(file, hosts)
      /*
       * Convert the \n to <p> 
       */
-     desc = convert_cr_to_html(info->value); 
-     name = portname_to_ahref("toc", hostname);
+     desc = html_output_convert_cr_to_html(info->value); 
+     name = html_output_portname_to_ahref("toc", hostname);
      fprintf(file, "<FONT SIZE=\"-1\"><div align=\"right\"><a href=\"#%s\">\
 [ back to the list of ports ]</a></div></FONT><p>\n",
       		name);
       fprintf(file, "<FONT COLOR=\"#660000\">");		
      fprintf(file, "<b>Warning found on port %s</b></FONT><p><ul><p>\n", port);
      efree(&name);	
-     print_data_with_links(file, desc, info->name);
+     html_output_print_data_with_links(file, desc, info->name, FALSE);
      fprintf(file, "<p></ul>\n");
      efree(&desc);
      }  
@@ -866,7 +637,7 @@ host_details(file, hosts)
    if(note)while(note && note->next)
     {
      if(!report){
-      	char * name = portname_to_ahref(ports->name, hostname);
+      	char * name = html_output_portname_to_ahref(ports->name, hostname);
      	fprintf(file, "<a name=\"%s\"></a>",name);
 	efree(&name);
 	}
@@ -878,19 +649,48 @@ host_details(file, hosts)
      /*
       * Convert the \n to <p> 
       */
-     desc = convert_cr_to_html(note->value); 
-     name = portname_to_ahref("toc", hostname);
+     desc = html_output_convert_cr_to_html(note->value); 
+     name = html_output_portname_to_ahref("toc", hostname);
      fprintf(file, "<FONT SIZE=\"-1\"><div align=\"right\"><a href=\"#%s\">\
 [ back to the list of ports ]</a></div></FONT><p>\n",
       		name);
      fprintf(file, "<b>Information found on port %s</b><p><ul><p>\n", port);
      efree(&name);	
-     print_data_with_links(file, desc, note->name);
+     html_output_print_data_with_links(file, desc, note->name, FALSE);
      fprintf(file, "<p></ul>\n");
      efree(&desc);
      }  
      note = note->next;
     }
+   false_positives = arg_get_value(ports->value, "FALSE");
+   if(false_positives)while(false_positives && false_positives->next)
+    {
+     if(!report){
+      	char * name = html_output_portname_to_ahref(ports->name, hostname);
+     	fprintf(file, "<a name=\"%s\"></a>",name);
+	efree(&name);
+	}
+     if(strlen(false_positives->value))
+     {
+     char * name;
+     desc = emalloc(strlen(false_positives->value)+1);
+     strncpy(desc, false_positives->value, strlen(false_positives->value));
+     /*
+      * Convert the \n to <p> 
+      */
+     desc = html_output_convert_cr_to_html(false_positives->value); 
+     name = html_output_portname_to_ahref("toc", hostname);
+     fprintf(file, "<FONT SIZE=\"-1\"><div align=\"right\"><a href=\"#%s\">\
+[ back to the list of ports ]</a></div></FONT><p>\n",
+      		name);
+     fprintf(file, "<b>False positive found on port %s</b><p><ul><p>\n", port);
+     efree(&name);	
+     html_output_print_data_with_links(file, desc, false_positives->name, FALSE);
+     fprintf(file, "<p></ul>\n");
+     efree(&desc);
+     }  
+     false_positives = false_positives->next;
+    }
 
     ports = ports->next;
    }
@@ -1072,6 +872,10 @@ static int html_make_index(hosts)
  fprintf(f, "<li><b> %d security note%s %s been found</b>\n",
  					n, n>1?"s":"", n>1?"have":"has");
 					
+ n = number_of_false_positives(hosts);
+ fprintf(f, "<li><b> %d false positive%s %s been found</b>\n",
+ 					n, n>1?"s":"", n>1?"have":"has");
+					
 
  fprintf(f, "</ul><br>\n<hr>\n");
  
@@ -1160,6 +964,11 @@ static int html_make_index(hosts)
      insert_img(f, "chart_dangerous_services.gif");
      for(i=0;i<num;i++)free(services[i]);
      }
+     else
+     {
+     fclose(chart);
+     }
+
     }
 
    /*
@@ -1193,6 +1002,10 @@ static int html_make_index(hosts)
      insert_img(f, "chart_services_occurences.gif");
      for(i=0;i<num;i++)free(services[i]);
      }
+    else
+     {
+     fclose(chart);
+     }
     }
     
    /*
@@ -1223,6 +1036,10 @@ static int html_make_index(hosts)
      fclose(chart);
      insert_img(f, "chart_operating_systems.gif");
      }
+    else
+     {
+     fclose(chart);
+     }
     }
     
    /*
@@ -1366,5 +1183,4 @@ int arglist_to_html_graph(hosts, directory)
  return 0;
 }
 
-
- 
+#endif /* NO_GDCHART */
