@@ -44,8 +44,6 @@
 #include "preferences.h"
 
 
-static char * convert_cr_to_html(char *);
-static char * portname_to_ahref(char *, char *);
 void summary_to_file(FILE *, struct arglist *);
 
 
@@ -53,8 +51,8 @@ void summary_to_file(FILE *, struct arglist *);
  * Handy functions
  */
 
-/*
- * print URL linking to CVE/BID/Nessus database to file.
+/**
+ * print URL linking to CVE/BID/OpenVAS database to file.
  * '%s' in urlfmt (from preferences) will be replaced by
  * item IDs (or their parts split at minus characters).
  * '%%' will insert a percent character.
@@ -62,11 +60,8 @@ void summary_to_file(FILE *, struct arglist *);
  * If urlfmt yields an empty string (e.g. it is only "%"),
  * only the item ID will be printed.
  */
-static void
-fprint_link(file, key, item)
-  FILE *file;
-  const char *key;
-  const char *item;
+void
+html_output_fprint_link (FILE* file, const char* key, const char*  item)
 {
   char *itemparts = estrdup(item);
   char *itemsrc = itemparts;
@@ -107,16 +102,17 @@ fprint_link(file, key, item)
   efree(&urlfmt);
   if(*url)
     fprintf(file, "<a href=\"%s\">%s</a>", url, item);
+    // In pdf_output was: strcmp("url_nessus", key) ? "" : "1.3.6.1.4.1.25623.1.0.", item);
   else
     fprintf(file, "%s", item);
   efree(&url);
 }
 
-/* All the cross references (CVE, BID) have the same format - XREF: <num>,...<br> */
-static char * 
-extract_xref(file, str, key)
- FILE * file;
- char * str, * key;
+/**
+ * All the cross references (CVE, BID) have the same format - XREF: <num>,...<br> 
+ */
+char*
+html_output_extract_xref (FILE* file, char* str, char* key)
 {
  while(str != NULL && strncmp(str, "<br>", 4) != 0)
    {
@@ -131,7 +127,7 @@ extract_xref(file, str, key)
      tmp = e1[0];
      e1[0] = '\0';
     }
-    fprint_link(file, key, str);
+    html_output_fprint_link(file, key, str);
     str = e1;
     if(e1 != NULL)
     {
@@ -149,11 +145,10 @@ extract_xref(file, str, key)
    }
   return str;
 }
- 
-static void 
-print_data_with_links(file, str, plugin_id)
- FILE * file;
- char * str, * plugin_id;
+
+void
+html_output_print_data_with_links (FILE* file, char* str, char* plugin_oid,
+                                   gboolean internal_link)
 {
  while(str != NULL && str[0] != '\0')
  {
@@ -182,26 +177,34 @@ print_data_with_links(file, str, plugin_id)
   {
    fprintf(file, "CVE : ");
    str += 6;
-   str = extract_xref(file, str, "url_cve");
+   str = html_output_extract_xref(file, str, "url_cve");
   }
   else if(strncmp(str, "BID : ", 6) == 0)
   {
   fprintf(file, "BID : ");
   str += 6;
-  str = extract_xref(file, str, "url_bid");
+  str = html_output_extract_xref(file, str, "url_bid");
   }
   else fputc(str[0], file);
   if(str != NULL)
     str++;
  }
  
- fprintf(file, "Nessus ID : ");
- fprint_link(file, "url_nessus", plugin_id);
+ fprintf(file, "OpenVAS ID : ");
+ if (internal_link == TRUE)
+   fprintf(file, "<a href=\"#nvt%s\">%s</a>", plugin_oid, plugin_oid);
+ else
+   html_output_fprint_link(file, "url_nessus", plugin_oid);
 }
 
-
-static char * convert_cr_to_html(str)
- char * str;
+/**
+ * Replaces newlines and \<, \> by "\<br\>" and "\&gt;"" , "\&lt;".
+ * 
+ * @param str String to convert.
+ * @return String str where newlines and \<, \> have been replaced by html tags.
+ */
+char*
+html_output_convert_cr_to_html (char* str)
 {
  int num = 0;
  char * t;
@@ -247,10 +250,8 @@ static char * convert_cr_to_html(str)
 }
 
 
-   
-static char * portname_to_ahref(name, hostname)
- char * name;
- char * hostname;
+char*
+html_output_portname_to_ahref (char* name, char* hostname)
 {
   char *t, *k;
 
@@ -288,12 +289,19 @@ static char * portname_to_ahref(name, hostname)
 }
 
 
-  
-
-int 
-arglist_to_html(hosts, filename)
- struct arglist * hosts;
- char * filename;
+ 
+/* As pdf export works via intermediate html export, it might be worth looking
+   at arglist_to_plainhtml (pdf_output.c), refactor and merge. */
+/**
+ * @brief Writes a report to an html file.
+ * 
+ * @param hosts Arglist with hosts containing issue arglists.
+ * @param filename Name of file to write to.
+ * 
+ * @return -1 if file could not be openend, 0 otherwise.
+ */
+int
+arglist_to_html (struct arglist* hosts, char* filename)
 {
  FILE * file;
  struct arglist * h;
@@ -316,7 +324,7 @@ arglist_to_html(hosts, filename)
  while(h && h->next)
  {
   int result;
-  char * href = portname_to_ahref(NULL, h->name);
+  char * href = html_output_portname_to_ahref(NULL, h->name);
   fprintf(file, "   <tr>\n\t <td class=default width=\"60%%\"><a href=\"#%s\">%s</a></td>\n", href, h->name);
   result = is_there_any_hole(h->value);
  
@@ -326,6 +334,8 @@ arglist_to_html(hosts, filename)
 	fprintf(file, "\t<td class=default width=\"40%%\">Security warning(s) found</td></tr>\n");
   else if(result == NOTE_PRESENT)
 	fprintf(file, "\t<td class=default width=\"40%%\">Security note(s) found</td></tr>\n");
+  else if(result == FALSE_PRESENT)
+	fprintf(file, "\t<td class=default width=\"40%%\">False positive(s) found</td></tr>\n");
   else fprintf(file, "\t<td class=default width=\"40%%\">No noticeable information found</td></tr>\n");
   efree(&href);
   h = h->next;
@@ -346,10 +356,10 @@ arglist_to_html(hosts, filename)
   char * name;
   hostname = hosts->name;
 
-  href = portname_to_ahref(NULL, hostname);
+  href = html_output_portname_to_ahref(NULL, hostname);
   fprintf(file, "<a name=\"%s\"></a>\n", href);
   efree(&href);
-  name = portname_to_ahref("toc", hostname);
+  name = html_output_portname_to_ahref("toc", hostname);
   fprintf(file, "<a name=\"%s\"></a>\n", name);
   efree(&name);
 
@@ -370,19 +380,21 @@ arglist_to_html(hosts, filename)
      {
   
         while(open && open->next){
-        	name = portname_to_ahref(open->name, hostname);
+        	name = html_output_portname_to_ahref(open->name, hostname);
 	  if(name)
 	  {
 	      if(arg_get_value(open->value, "REPORT") ||
 	         arg_get_value(open->value, "INFO") ||
+	         arg_get_value(open->value, "FALSE") ||
 	         arg_get_value(open->value, "NOTE")) 
 	      {
 	             fprintf(file, "   <tr>\n\t<td class=default width=\"20%%\">%s</td>\n", hostname);
 	             fprintf(file, "\t<td class=default width=\"30%%\"><a href=\"#%s\">%s</a></td>\n",
 	   		name, open->name);
-	             if(arg_get_value(open->value, "REPORT")) fprintf(file, "\t<td class=default width=\"30%%\"><font color=red>Security hole found</font></td></tr>\n");
+	             if(arg_get_value(open->value, "REPORT")) fprintf(file, "\t<td class=default width=\"30%%\"><font color=red>Security hole(s) found</font></td></tr>\n");
 	             else if(arg_get_value(open->value, "INFO")) fprintf(file, "\t<td class=default width=\"30%%\">Security warning(s) found</td></tr>\n");
-	             else fprintf(file, "\t<td class=default width=\"30%%\">Security notes found</td></tr>\n");
+	             else if(arg_get_value(open->value, "FALSE")) fprintf(file, "\t<td class=default width=\"30%%\">False positive(s) found</td></tr>\n");
+	             else fprintf(file, "\t<td class=default width=\"30%%\">Security note(s) found</td></tr>\n");
 		
 	      }	 
 	        else {	
@@ -416,6 +428,7 @@ arglist_to_html(hosts, filename)
     struct arglist * report;
     struct arglist * info;
     struct arglist * note;
+    struct arglist * false_positive;
 
     port = ports->name;
     report = arg_get_value(ports->value, "REPORT");
@@ -426,17 +439,17 @@ arglist_to_html(hosts, filename)
      /*
       * Convert the \n to <p> 
       */
-      desc = convert_cr_to_html(report->value);
-      name = portname_to_ahref("toc", hostname);
+      desc = html_output_convert_cr_to_html(report->value);
+      name = html_output_portname_to_ahref("toc", hostname);
       efree(&name);		
       fprintf(file, "\t\t<tr>\n\t<td valign=top class=default width=\"10%%\"><font color=red>Vulnerability</font></td>\n");
 
-      name = portname_to_ahref(ports->name, hostname);
+      name = html_output_portname_to_ahref(ports->name, hostname);
       fprintf(file, "\t\t<td valign=top class=default width=\"10%%\"><a name=\"%s\"></a>%s</td>\n", name, port);
       efree(&name);
 
       fprintf(file, "\t\t<td class=default width=\"80%%\">"); 	
-      print_data_with_links(file, desc, report->name);
+      html_output_print_data_with_links(file, desc, report->name, FALSE);
       fprintf(file, "\t</td></tr>\n");
       efree(&desc);
      }
@@ -450,23 +463,51 @@ arglist_to_html(hosts, filename)
      /*
       * Convert the \n to <p> 
       */
-     desc = convert_cr_to_html(info->value); 
-     name = portname_to_ahref("toc", hostname);
+     desc = html_output_convert_cr_to_html(info->value); 
+     name = html_output_portname_to_ahref("toc", hostname);
       efree(&name); 
       fprintf(file, "   <tr>\t\n<td valign=top class=default width=\"10%%\">Warning</td>\n");
 
-      name = portname_to_ahref(ports->name, hostname);
+      name = html_output_portname_to_ahref(ports->name, hostname);
       fprintf(file, "\t<td valign=top class=default width=\"10%%\"><a name=\"%s\"></a>%s</td>\n", name, port);
       efree(&name);
       
      fprintf(file, "\t<td class=default width=\"80%%\">");
-     print_data_with_links(file, desc, info->name);
+     html_output_print_data_with_links(file, desc, info->name, FALSE);
      fprintf(file, "</td></tr>\n");
      efree(&desc);
      }  
      info = info->next;
     }
 
+   false_positive = arg_get_value(ports->value, "FALSE");
+   if(false_positive)while(false_positive->next)
+    {
+     if(strlen(false_positive->value))
+     {
+     char * name;
+     desc = emalloc(strlen(false_positive->value)+1);
+     strncpy(desc, false_positive->value, strlen(false_positive->value));
+     /*
+      * Convert the \n to <p> 
+      */
+     desc = html_output_convert_cr_to_html(false_positive->value); 
+     name = html_output_portname_to_ahref("toc", hostname);
+     efree(&name); 
+     fprintf(file, "   <tr>\n\t<td valign=top class=default width=\"10%%\">Informational</td>\n");
+
+     name = html_output_portname_to_ahref(ports->name, hostname);
+     fprintf(file, "\t<td valign=top class=default width=\"10%%\"><a name=\"%s\"></a>%s</td>\n", name, port);
+     efree(&name);
+
+     fprintf(file, "\t<td class=default width=\"80%%\">");
+     html_output_print_data_with_links(file, desc, false_positive->name, FALSE);
+     fprintf(file, "</td></tr>\n");
+     efree(&desc);
+     }  
+     false_positive = false_positive->next;
+    }
+
    note = arg_get_value(ports->value, "NOTE");
    if(note)while(note->next)
     {
@@ -478,22 +519,23 @@ arglist_to_html(hosts, filename)
      /*
       * Convert the \n to <p> 
       */
-     desc = convert_cr_to_html(note->value); 
-     name = portname_to_ahref("toc", hostname);
+     desc = html_output_convert_cr_to_html(note->value); 
+     name = html_output_portname_to_ahref("toc", hostname);
       efree(&name); 
       fprintf(file, "   <tr>\n\t<td valign=top class=default width=\"10%%\">Informational</td>\n");
 
-      name = portname_to_ahref(ports->name, hostname);
+      name = html_output_portname_to_ahref(ports->name, hostname);
       fprintf(file, "\t<td valign=top class=default width=\"10%%\"><a name=\"%s\"></a>%s</td>\n", name, port);
       efree(&name);
 
       fprintf(file, "\t<td class=default width=\"80%%\">");
-     print_data_with_links(file, desc, note->name);
+     html_output_print_data_with_links(file, desc, note->name, FALSE);
      fprintf(file, "</td></tr>\n");
      efree(&desc);
      }  
      note = note->next;
     }
+
     ports = ports->next;
    }
   }
@@ -510,13 +552,14 @@ arglist_to_html(hosts, filename)
 }
 
 
-void summary_to_file(FILE *file, struct arglist *hosts)
+void
+summary_to_file (FILE *file, struct arglist *hosts)
 {
 
  fprintf(file, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n");
  fprintf(file, "<HTML>\n");
  fprintf(file, " <HEAD>\n");
- fprintf(file, " <TITLE>Nessus Scan Report</TITLE>\n");
+ fprintf(file, " <TITLE>OpenVAS Scan Report</TITLE>\n");
  fprintf(file, " <meta http-equiv=\"Content-Type\" content=\"text/html; charset=\"iso-8859-1\">\n");
  fprintf(file, " <style type=\"text/css\">\n");
  fprintf(file, " <!--\n");
@@ -556,7 +599,7 @@ void summary_to_file(FILE *file, struct arglist *hosts)
  fprintf(file, "<table bgcolor=\"#a1a1a1\" border=0 cellpadding=0 cellspacing=0 width=\"95%%\">\n");
  fprintf(file, "<tbody>\n\t<tr><td>\n<table border=0 cellpadding=2 cellspacing=1 width=\"100%%\">\n");
  fprintf(file, "\t<tbody>\n   <tr>\n");
- fprintf(file, "\t<td class=title>Nessus Scan Report</td></tr>\n");
+ fprintf(file, "\t<td class=title>OpenVAS Scan Report</td></tr>\n");
  fprintf(file, "   <tr>\n\t<td class=content>This report gives details on hosts that were tested\n");
  fprintf(file, "\t\tand issues that were found. Please follow the recommended\n");
  fprintf(file, "\t\tsteps and procedures to eradicate these threats.\n");
@@ -581,6 +624,12 @@ void summary_to_file(FILE *file, struct arglist *hosts)
  fprintf(file, "   <tr>\n\t<td class=default width=\"60%%\">Number of security warnings found</td>\n");
  fprintf(file, "\t<td class=default width=\"30%%\">%d</td></tr>\n",
                                 number_of_warnings(hosts));
+ fprintf(file, "   <tr>\n\t<td class=default width=\"60%%\">Number of security notes found</td>\n");
+ fprintf(file, "\t<td class=default width=\"30%%\">%d</td></tr>\n",
+                                number_of_notes(hosts));
+ fprintf(file, "   <tr>\n\t<td class=default width=\"60%%\">Number of false positives found</td>\n");
+ fprintf(file, "\t<td class=default width=\"30%%\">%d</td></tr>\n",
+                                number_of_false_positives(hosts));
  fprintf(file, "</tbody></table></td></tr></tbody></table><br><br>\n");
 
  fprintf(file, "<a name=\"toc\"></a>");
