@@ -28,8 +28,14 @@
  
 #ifndef _NESSUSC_HTML_OUTPUT_H
 #define _NESSUSC_HTML_OUTPUT_H
+#include <glib.h>
 
-int arglist_to_html(struct arglist * , char * );
-
+int   arglist_to_html(struct arglist * , char * );
+void  html_output_fprint_link(FILE* file, const char* key, const char*  item);
+char* html_output_extract_xref(FILE* file, char* str, char* key);
+char* html_output_convert_cr_to_html(char* str);
+char* html_output_portname_to_ahref(char* name, char* hostname);
+void  html_output_print_data_with_links(FILE* file, char* str, char* plugin_oid,
+                      gboolean internal_link);
 
 #endif
