@@ -83,13 +83,14 @@ static int latex_report_category(hosts, holes, warnings, notes)
   else return CATEGORY_A;
 }
 
-   
-/*
- * Print the LaTeX header
+/**
+ * @brief Print the LaTeX header to file f.
+ * 
+ * @param f File to print LATEX header to.
  */
 static void latex_print_header(FILE * f)
 {
-/*
+/**
  * Fancy headers
  */
  fprintf(f, "\\documentclass{article}\n");
@@ -97,7 +98,7 @@ static void latex_print_header(FILE * f)
  fprintf(f, "\\ifx\\pdfoutput\\undefined\\else\\usepackage{times}\\fi\n");
  fprintf(f, "\\usepackage{fancyhdr}\n");
  fprintf(f, "\\pagestyle{fancy}\n");
- fprintf(f, "\\fancyhead[LE,RO]{\\textit{Nessus Report}}\n");
+ fprintf(f, "\\fancyhead[LE,RO]{\\textit{OpenVAS Report}}\n");
  fprintf(f, "\\fancyfoot[LE,RO]{}\n");
  
  /*
@@ -105,9 +106,9 @@ static void latex_print_header(FILE * f)
   * table of contents
   */
  fprintf(f, "\\pagenumbering{roman}\n");
- fprintf(f, "\\title{\\vspace*{100pt}\\Huge Report of a Nessus scan\\normalsize}\n");
+ fprintf(f, "\\title{\\vspace*{100pt}\\Huge Report of an OpenVAS scan\\normalsize}\n");
  fprintf(f, "%%\n%% You might want to change this : \n%%\n");
- fprintf(f, "\\author{Nessus Security Scanner}\n");
+ fprintf(f, "\\author{OpenVAS Security Scanner}\n");
  fprintf(f, "\\begin{document}\n");
  fprintf(f, "\\maketitle\n");
  fprintf(f, "\\newpage\n");
@@ -115,8 +116,10 @@ static void latex_print_header(FILE * f)
 }
 
 
-/*
- * Print the LaTeX footer
+/**
+ * @brief Print the LaTeX footer to file f.
+ * 
+ * @param f File to print the latex footer to.
  */
 static void latex_print_footer(FILE *  f)
 {
@@ -135,15 +138,15 @@ static void latex_print_footer(FILE *  f)
   
  ****************************************************************************/
 
-/*
+/**
  * Nothing found : excellent
  */
-void latex_introduction_a(FILE * f, struct arglist * hosts, int holes, int warnings, int notes)
+void latex_introduction_a(FILE * f, struct arglist * hosts, int holes, int warnings, int notes, int false_positives)
 {
  int num_of_hosts = arglist_length(hosts);
  fprintf(f, "\\newpage\n");
  fprintf(f, "\\section*{Introduction}\n");
- fprintf(f, "In this test, Nessus has tested %d host",num_of_hosts);
+ fprintf(f, "In this test, OpenVAS has tested %d host",num_of_hosts);
  if(num_of_hosts > 1)fprintf(f, "s");
  fprintf(f," and none of the vulnerabilities tested were present on this ");
  fprintf(f,"network ");
@@ -157,22 +160,23 @@ void latex_introduction_a(FILE * f, struct arglist * hosts, int holes, int warni
  fprintf(f, "complete safety !}. Security Scanners can not test things such as home made ");
  fprintf(f, "CGIs, so if you want the garantee that you network is secure, we recommand ");
  fprintf(f, "that you check these things manually.\\\\\n");
- fprintf(f, "However, your network was given the highest mark that Nessus can give, ");
+ fprintf(f, "However, your network was given the highest mark that OpenVAS can give, ");
  fprintf(f, "that is a A (worst being E). Congratulations !");
 }
 
-/*
+/**
  * Some things found, but not serious on the overall
  */
-void latex_introduction_b(FILE * f, struct arglist * hosts, int holes, int warnings, int notes)
+void latex_introduction_b(FILE * f, struct arglist * hosts, int holes, int warnings, int notes, int false_positives)
 {
  struct arglist * most = most_dangerous_host(hosts);
  int num_of_hosts = arglist_length(hosts);
  fprintf(f, "\\newpage\n");
  fprintf(f, "\\section*{Introduction}\n");
- fprintf(f, "In this test, Nessus has tested %d host", num_of_hosts);
+ fprintf(f, "In this test, OpenVAS has tested %d host", num_of_hosts);
  if(num_of_hosts > 1)fprintf(f, "s");
- fprintf(f, " and found %d security holes, %d warnings and %d notes that can eventually help a cracker ", holes, warnings, notes);
+ fprintf(f, " and found %d security holes, %d warnings, %d notes and %d false positives that can eventually help a cracker ",
+	 holes, warnings, notes, false_positives);
  fprintf(f, "to break into your network. You should have a close look at them and ");
  fprintf(f, "decide of their severity. We strongly suggest that you correct them ");
  fprintf(f, "although we know it is not always possible.\\\\\n");
@@ -188,23 +192,24 @@ void latex_introduction_b(FILE * f, struct arglist * hosts, int holes, int warni
  fprintf(f,"list of what was tested).\\\\\n");
  } 
  fprintf(f, "which is a very good thing.\\\\\n");
- fprintf(f, "On the overall, Nessus has given to the security of this network the mark B. (A being the best ");
+ fprintf(f, "On the overall, OpenVAS has given to the security of this network the mark B. (A being the best ");
  fprintf(f, "and E being the worst).");
 }
 
 
-/*
+/**
  * A lot of small things have been found. That's not good
  */
-int latex_introduction_c(FILE * f, struct arglist * hosts, int holes, int warnings, int notes)
+int latex_introduction_c(FILE * f, struct arglist * hosts, int holes, int warnings, int notes, int false_positives)
 {
  struct arglist * most = most_dangerous_host(hosts);
  int num_of_hosts = arglist_length(hosts);
  fprintf(f, "\\newpage\n");
  fprintf(f, "\\section*{Introduction}\n");
- fprintf(f, "In this test, Nessus has tested %d host", num_of_hosts);
+ fprintf(f, "In this test, OpenVAS has tested %d host", num_of_hosts);
  if(num_of_hosts > 1)fprintf(f, "s");
- fprintf(f, " and found %d security holes, %d warnings an %d notes that can eventually help a cracker ", holes, warnings, notes);
+ fprintf(f, " and found %d security holes, %d warnings, %d notes and %d false positives that can eventually help a cracker ",
+	 holes, warnings, notes, false_positives);
  fprintf(f, "to break into your network. You should have a close look at them and ");
  fprintf(f, "decide of their severity. \\\\\n");
  fprintf(f, "Note that there is a big number of warnings for a single network of this size. ");
@@ -226,21 +231,22 @@ int latex_introduction_c(FILE * f, struct arglist * hosts, int holes, int warnin
  fprintf(f,"list of what was tested).\\\\\n");
  }
  fprintf(f, " which is a very good thing.\\\\\n");
- fprintf(f, "On the overall, Nessus has given to the security of this network the mark C. (A being the best ");
+ fprintf(f, "On the overall, OpenVAS has given to the security of this network the mark C. (A being the best ");
  fprintf(f, "and E being the worst).");
  return 0;
 }
 
 
-int latex_introduction_d(FILE * f, struct arglist * hosts, int holes, int warnings, int notes)
+int latex_introduction_d(FILE * f, struct arglist * hosts, int holes, int warnings, int notes, int false_positives)
 {
  struct arglist * most = most_dangerous_host(hosts);
  int num_of_hosts = arglist_length(hosts);
  fprintf(f, "\\newpage\n");
  fprintf(f, "\\section*{Introduction}\n");
- fprintf(f, "In this test, Nessus has tested %d host", num_of_hosts);
+ fprintf(f, "In this test, OpenVAS has tested %d host", num_of_hosts);
  if(num_of_hosts > 1)fprintf(f, "s");
- fprintf(f, " and found \\textbf{%d severe security holes}, as well as %d security warnings and %d notes.", holes, warnings, notes);
+ fprintf(f, " and found \\textbf{%d severe security holes}, as well as %d security warnings, %d notes and %d false positives.",
+	 holes, warnings, notes, false_positives);
  fprintf(f, "These problems can easily be used to break ");
  fprintf(f, "into your network. You should have a close look at them and ");
  fprintf(f, "correct them as soon as possible.\\\\\n");
@@ -260,22 +266,23 @@ int latex_introduction_d(FILE * f, struct arglist * hosts, int holes, int warnin
  fprintf(f, "(see Appendix A and B page \\pageref{appendix_a} and page \\pageref{appendix_b} for the exhaustive ");
  fprintf(f,"list of what was tested).\\\\\n");
  }
-  fprintf(f, "On the overall, Nessus has given to the security of this network the mark D. (A being the best ");
+  fprintf(f, "On the overall, OpenVAS has given to the security of this network the mark D. (A being the best ");
  fprintf(f, "and E being the worst). There is room for improvement, and ");
  fprintf(f, "\\textbf{we strongly suggest that you take the appropriate measures to ");
  fprintf(f, "solve these problems \\textit{as soon as possible}}\n");
  return 0 ;
 }
 
-int latex_introduction_e(FILE * f, struct arglist * hosts, int holes, int warnings, int notes)
+int latex_introduction_e(FILE * f, struct arglist * hosts, int holes, int warnings, int notes, int false_positives)
 {
  struct arglist * most = most_dangerous_host(hosts);
  int num_of_hosts = arglist_length(hosts);
  fprintf(f, "\\newpage\n");
   fprintf(f, "\\section*{Introduction}\n");
- fprintf(f, "In this test, Nessus has tested %d host", num_of_hosts);
+ fprintf(f, "In this test, OpenVAS has tested %d host", num_of_hosts);
  if(num_of_hosts > 1)fprintf(f, "s");
- fprintf(f, " and found \\textbf{%d severe security holes}, as well as %d security warnings and %d notes.", holes, warnings, notes);
+ fprintf(f, " and found \\textbf{%d severe security holes}, as well as %d security warnings, %d notes and %d false positives.",
+	 holes, warnings, notes, false_positives);
  fprintf(f, "These problems can easily be used to break ");
  fprintf(f, "into your network. You should have a close look at them and ");
  fprintf(f, "correct them as soon as possible.\\\\\n");
@@ -295,7 +302,7 @@ if(Context->parent && Context->parent->scanners && Context->parent->plugins)
  fprintf(f,"list of what was tested).\\\\\n");
  }
  
- fprintf(f, "On the overall, Nessus has given to the security of this network the mark E ");
+ fprintf(f, "On the overall, OpenVAS has given to the security of this network the mark E ");
  fprintf(f, "because of the number of vulnerabilities found. A script kid should be ");
  fprintf(f, "able to break into your network rather easily.\\\\\n");
  fprintf(f, "There is room for improvement, and ");
@@ -333,6 +340,7 @@ void latex_print_host_ports_problems(FILE * f, struct arglist * port)
  struct arglist * holes = arg_get_value(port->value, "REPORT");
  struct arglist * info = arg_get_value(port->value, "INFO");
  struct arglist * note = arg_get_value(port->value, "NOTE");
+ struct arglist * false_positive = arg_get_value(port->value, "FALSE");
  if(!(holes || info || note)){
  	latex_print_host_ports_problems(f, port->next);
 	return;
@@ -384,13 +392,28 @@ void latex_print_host_ports_problems(FILE * f, struct arglist * port)
     }
     fprintf(f, "\\end{itemize}\n");
    }
+   if(false_positive)
+   {
+    fprintf(f, "False_positive :\\\\\n");
+    fprintf(f, "\\begin{itemize}\n");
+    while(false_positive->next)
+    {
+     char * c = false_positive->value;
+     while(c[0]=='\n')c++;
+     fprintf(f, "\\item \\begin{verbatim}");
+     printf_formatted_text(f, c, 70, NULL);
+     fprintf(f, "\n\\end{verbatim}");
+     false_positive = false_positive->next;
+    }
+    fprintf(f, "\\end{itemize}\n");
+   }
   }
   latex_print_host_ports_problems(f, port->next);
  }
 }
 
 
-/*
+/**
  * This function will create the new section that has the name
  * of the current host.
  */
@@ -439,26 +462,26 @@ void latex_conclusion(FILE * f)
 {
  fprintf(f, "\\newpage\n");
  fprintf(f, "\\section*{Conclusion}\n");
- fprintf(f, "A security scanner, such as Nessus, is not a guarantee  ");
+ fprintf(f, "A security scanner, such as OpenVAS, is not a guarantee ");
  fprintf(f, "of the security of your network.\\\\\n");
  fprintf(f, "A lot of factors can not be tested by a security scanner : ");
  fprintf(f, "the practices of the users of the network, the home-made ");
  fprintf(f, "services and CGIs, and so on... So, you should not have ");
- fprintf(f, "a false sense of security now that the test are done. ");
+ fprintf(f, "a false sense of security now that the tests are done. ");
  fprintf(f, "We recommand that you monitor actively what happens on ");
  fprintf(f, "your firewall, and that you use some tools such as ");
  fprintf(f, "tripwire to restore your servers more easily in the case ");
  fprintf(f, "of an intrusion.\\\\\n");
  fprintf(f, "In addition to that, you must know that new security holes ");
- fprintf(f, "are found each week. That is why we recommand that you visit ");
- fprintf(f, "\\verb+http://www.osvdb.org/ and http://www.openvas.org+, for an ");
+ fprintf(f, "are found each week. This is the reason why we recommand that you visit ");
+ fprintf(f, "\\verb+http://www.osvdb.org/+ and \\verb+http://www.openvas.org/+ for an ");
  fprintf(f, "update of security vulnerabilites as they are published ");
  fprintf(f, "on public mailing lists such as BugTraq. (see ");
- fprintf(f, "\\verb+http://www.securityfocus.com+ for details)\\\\\n ");
+ fprintf(f, "\\verb+http://www.securityfocus.com/+ for details)\\\\\n ");
  fprintf(f, "Please test the security of your network on a (at least) weekly basis ");
  fprintf(f, "with the checks that are on this page.\\\\\n");
  fprintf(f, "\\textit{This report was generated by the OpenVAS security ");
- fprintf(f, "scanner. See http://www.openvas.org for more information}");
+ fprintf(f, "scanner. See http://www.openvas.org/ for more information}");
 }
 
 /***************************************************************************
@@ -488,23 +511,28 @@ void latex_appendix(FILE *f)
   if(Context->parent && (Context->parent->scanners || Context->parent->plugins))
   {
   fprintf(f, "\\newpage\n");
-  fprintf(f, "\\label{appendix_a}\n");
   fprintf(f, "\\appendix\n");
-  fprintf(f, "\\section{List of port scanners used during this session}\n");
-  fprintf(f, "\\begin{itemize}\n");
-  latex_appendix_plugins(f, Context->parent->scanners);
-  fprintf(f, "\\end{itemize}\n");
-  fprintf(f, "\\label{appendix_b}\n");
-  fprintf(f, "\\section{List of plugins used during this session}\n");
-  fprintf(f, "\\begin{itemize}\n");
-  latex_appendix_plugins(f, Context->parent->plugins);
-  fprintf(f, "\\end{itemize}\n");
+  if (Context->parent->scanners) {
+    fprintf(f, "\\label{appendix_a}\n");
+    fprintf(f, "\\section{List of port scanners used during this session}\n");
+    fprintf(f, "\\begin{itemize}\n");
+    latex_appendix_plugins(f, Context->parent->scanners);
+    fprintf(f, "\\end{itemize}\n");
+  }
+  if (Context->parent->plugins) {
+    fprintf(f, "\\label{appendix_b}\n");
+    fprintf(f, "\\section{List of plugins used during this session}\n");
+    fprintf(f, "\\begin{itemize}\n");
+    latex_appendix_plugins(f, Context->parent->plugins);
+    fprintf(f, "\\end{itemize}\n");
+  }
  }
 }
  				
 int arglist_to_latex(struct arglist * list, char * filename)
 {
  FILE * f;
+ int false_positives = number_of_false_positives(list);
  int notes = number_of_notes(list);
  int warnings = number_of_warnings(list);
  int holes = number_of_holes(list);
@@ -534,15 +562,15 @@ int arglist_to_latex(struct arglist * list, char * filename)
   switch(latex_report_category(length, holes, warnings, notes))
   {
    case CATEGORY_A :
-   	 latex_introduction_a(f, list, holes, warnings, notes);break;
+   	 latex_introduction_a(f, list, holes, warnings, notes, false_positives);break;
    case CATEGORY_B :
-  	 latex_introduction_b(f, list, holes, warnings, notes);break;
+  	 latex_introduction_b(f, list, holes, warnings, notes, false_positives);break;
    case CATEGORY_C :
-  	 latex_introduction_c(f, list, holes, warnings, notes);break;
+  	 latex_introduction_c(f, list, holes, warnings, notes, false_positives);break;
    case CATEGORY_D :
-  	 latex_introduction_d(f, list, holes, warnings, notes);break;
+  	 latex_introduction_d(f, list, holes, warnings, notes, false_positives);break;
    default :
-   	 latex_introduction_e(f, list, holes, warnings, notes);break;
+   	 latex_introduction_e(f, list, holes, warnings, notes, false_positives);break;
    
   }
   
