@@ -42,7 +42,7 @@
 
 #include <includes.h>
 
-#ifdef USE_GTK 
+#ifdef USE_GTK
 #include <gtk/gtk.h>
 
 #include "backend.h"
@@ -53,6 +53,7 @@
 #include "comm.h"
 #include "auth.h"
 #include "parser.h"
+#include "parseutils.h"
 #include "report.h"
 #include "globals.h"
 #include "error_dlg.h"
@@ -72,8 +73,7 @@ static int monitor_stop_whole_test(GtkWidget * , struct arglist *);
 
 
 static int 
-is_server_present(soc)
-	int soc;
+is_server_present (int soc)
 {
 	fd_set  rd;
 	struct timeval tv = {2,0};
@@ -98,10 +98,7 @@ is_server_present(soc)
 }
 
 
-
-
-
-/*
+/**
  * Function called when the UI is idle, which checks
  * whether the server sent us anything. We use this rather
  * than the traditional gdk input watcher, because it
@@ -140,7 +137,7 @@ idle_socket(struct arglist * ctrls)
   return TRUE;
 }
 
-/* column IDs of scanner TreeView Model */
+/** Column IDs of scanner TreeView Model */
 enum {
   COL_ICON,
   COL_HOSTNAME,
@@ -150,10 +147,7 @@ enum {
 };
 
 static void
-stop_toggled(cell, path_str, data)
-  GtkCellRendererToggle *cell;
-  gchar                 *path_str;
-  gpointer               data;
+stop_toggled (GtkCellRendererToggle *cell, gchar* path_str, gpointer data)
 {
   GtkListStore * store = arg_get_value(data, "MON_STORE");
   struct context * context = arg_get_value(data, "CONTEXT");
@@ -169,14 +163,11 @@ stop_toggled(cell, path_str, data)
     "CLIENT <|> STOP_ATTACK <|> %s <|> CLIENT\n", hostname);
 }
 
-/*
- * monitor_dialog_setup
- *
- * This function draws the window which will
- * show the attack status
+/**
+ * @brief Draws the window which will show the attack status.
  */
-void 
-monitor_dialog_setup(char * victim, struct context * context)
+void
+monitor_dialog_setup (char * victim, struct context * context)
 {
   struct arglist * ctrls = emalloc(sizeof(struct arglist));
   GtkWindow* parent = arg_get_value(MainDialog, "CONTEXT");
@@ -191,7 +182,7 @@ monitor_dialog_setup(char * victim, struct context * context)
   if( backend < 0 )
     return;
 
-  arg_add_value(ctrls, "MONITOR_BACKEND", ARG_INT, -1, (void*)backend);
+  arg_add_value(ctrls, "MONITOR_BACKEND", ARG_INT, -1, GSIZE_TO_POINTER(backend));
   arg_add_value(ctrls, "PARENT", ARG_PTR, -1, parent);
   arg_add_value(ctrls, "CONTEXT", ARG_PTR, -1, context);
   context->action = CONTEXT_SCANNING;
@@ -304,7 +295,7 @@ monitor_dialog_setup(char * victim, struct context * context)
   
   tag = gtk_idle_add((GtkFunction)idle_socket, ctrls);
   
-  arg_add_value(ctrls, "TAG", ARG_INT, sizeof(int), (void*)tag);
+  arg_add_value(ctrls, "TAG", ARG_INT, sizeof(gpointer), GSIZE_TO_POINTER(tag));
   w = gtk_button_new_with_label(_("Stop the whole test"));
   g_signal_connect(GTK_OBJECT(w), "clicked",
 		     (GtkSignalFunc)monitor_stop_whole_test,ctrls);
@@ -323,16 +314,11 @@ monitor_dialog_setup(char * victim, struct context * context)
   }
 }
 
-/*
- * monitor_list_update
- *
- * Updates the progress bars
- */	
-static void 
-monitor_list_update(ctrls,msg, short_status)
-	struct arglist * ctrls;
-	char * msg;
-	int short_status;
+/**
+ * @brief Updates the progress bars.
+ */
+static void
+monitor_list_update (struct arglist * ctrls, char * msg, int short_status)
 {
  char * hostname;
  char * action;
@@ -345,40 +331,23 @@ monitor_list_update(ctrls,msg, short_status)
  GtkWidget * gtkw;
  char * list_hostname;
  int flag = 0;
- static harglst * hosts = NULL;
- int v = 1;
- 
- if(!hosts)
- {
-  hosts = harg_create(65000);
- }
 
- 
  if(!short_status)
   parse_nessusd_status(msg, &hostname, &action, &current, &max);
  else
   parse_nessusd_short_status(msg, &hostname, &action, &current, &max);
- 
- if(hostname)
- {
- v = harg_get_int(hosts, hostname);
- if(!v){
-  harg_add_int(hosts, hostname, 1);
-  v = 1;
-  }
- else
-  {
-  v++;
-  harg_set_int(hosts, hostname, v);
-  }
- }
- 
+
   if (GTK_CHECK_VERSION(2,6,0)) { /* TreeView since GTK 2.6 */
     GtkListStore * store = arg_get_value(ctrls, "MON_STORE");
     GtkTreeIter iter;
-    int f = (atoi(current) * 100) / max;
-    if(f>=100)f=100;
-    if(f<=0)f=0;
+    int f;
+    if (max == 0) {
+      f = 100;
+    } else {
+      f = (atoi(current) / max) * 100;
+      if(f>=100)f=100;
+      if(f<=0)f=0;
+    }
 
     if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter)) {
       do {
@@ -415,16 +384,21 @@ monitor_list_update(ctrls,msg, short_status)
         GtkWidget * progress_bar;
         gfloat f;
    
+        gmax = max;
+	if (gmax == 0) {
+          f = 1.0;
+        } else {
+          gcurrent = atoi(current);
+          f = (gcurrent/gmax);
+          if(f>=1.0)f=1.0;
+          if(f<=0.0)f=0.0;
+        }
+
         if( strcmp(action, "portscan") == 0 )
           progress_bar = gtk_object_get_data(item, "progress_bar_portscan");
         else
           progress_bar = gtk_object_get_data(item, "progress_bar_attack");
 
-        gmax = max;
-        gcurrent = atoi(current);
-        f = (gcurrent/gmax);
-        if(f>=1.0)f=1.0;
-        if(f<=0.0)f=0.0;
         gtk_progress_bar_update (GTK_PROGRESS_BAR(progress_bar), f);
 
         flag = 1;
@@ -443,14 +417,12 @@ monitor_list_update(ctrls,msg, short_status)
  efree(&action);
  if(current)efree(&current);
 }
- 
+
 /*
  * monitor_remove_host
  */
 static void
-monitor_remove_host(ctrls, host)
- struct arglist * ctrls;
- char * host;
+monitor_remove_host (struct arglist * ctrls, char * host)
 {
   if (GTK_CHECK_VERSION(2,6,0)) { /* TreeView since GTK 2.6 */
     GtkListStore * store = arg_get_value(ctrls, "MON_STORE");
@@ -491,22 +463,12 @@ monitor_remove_host(ctrls, host)
   }
 }
 
- 
- 
 
-
-
-/*
- * monitor_add_host
- *
- * this function adds a new hostname and progress bar in
- * the monitor window
+/**
+ * @brief Adds a new hostname and progress bar in the monitor window.
  */
 void 
-monitor_add_host(ctrls,hostname,port)
-	struct arglist * ctrls;
-	char * hostname;
-	int port;
+monitor_add_host (struct arglist * ctrls, char * hostname, int port)
 {
   if (GTK_CHECK_VERSION(2,6,0)) { /* TreeView since GTK 2.6 */
     GtkListStore * store = arg_get_value(ctrls, "MON_STORE");
@@ -552,8 +514,6 @@ monitor_add_host(ctrls,hostname,port)
  /*
   * Host name
   */
-
-    
  box = gtk_vbox_new(TRUE,0);
  gtk_widget_show(box);
  gtk_box_pack_start(GTK_BOX(hbox), box, FALSE, FALSE, 0);
@@ -589,7 +549,6 @@ monitor_add_host(ctrls,hostname,port)
  /*
   *  Progress bars
   */
- 
  box = gtk_vbox_new(FALSE,0);
  gtk_table_attach_defaults(GTK_TABLE(table), box, 1,2,0,3); 
   gtk_widget_show(box);
@@ -607,8 +566,6 @@ monitor_add_host(ctrls,hostname,port)
  gtk_box_pack_start(GTK_BOX(box), progress_bar_attack, TRUE, TRUE,0);
  gtk_widget_show(progress_bar_attack);
  
- 
- 
  /*
   * Stop button
   */
@@ -628,8 +585,6 @@ monitor_add_host(ctrls,hostname,port)
  gtk_widget_show(separator);
 
 
-    
-
  gtk_object_set_data(GTK_OBJECT(item), "hostname", hostname);
  gtk_object_set_data(GTK_OBJECT(item), "progress_bar_attack", progress_bar_attack);
  gtk_object_set_data(GTK_OBJECT(item), "progress_bar_portscan", progress_bar_portscan);
@@ -641,27 +596,20 @@ monitor_add_host(ctrls,hostname,port)
 
   }
  }
- 
- 
-/*  
- * monitor_stop_test
- * 
- * This function will stop the connection between
- * nessusd and the client, and will report the results
- * to the screen
+
+
+/**
+ * @brief Stops the connection between nessusd and the client, and reports the
+ * @brief results to the screen.
  */
- 
 static int
-monitor_stop_whole_test_destroy(a,b,ctrls)
- void * a, * b;
- struct arglist * ctrls;
+monitor_stop_whole_test_destroy (void* a, void* b, struct arglist* ctrls)
 {
  return monitor_stop_whole_test(NULL, ctrls);
 }
+
 static int
-monitor_stop_whole_test(w, ctrls)
- GtkWidget * w;
- struct arglist * ctrls;
+monitor_stop_whole_test (GtkWidget* w, struct arglist* ctrls)
 {
  int type = 0, n;
  char * msg;
@@ -671,7 +619,7 @@ monitor_stop_whole_test(w, ctrls)
  context = arg_get_value(ctrls, "CONTEXT");
 
  network_printf(context->socket, "CLIENT <|> STOP_WHOLE_TEST <|> CLIENT\n");
- gtk_idle_remove((int)arg_get_value(ctrls, "TAG"));
+ gtk_idle_remove(GPOINTER_TO_SIZE(arg_get_value(ctrls, "TAG")));
  
  /*
   * Read the data remaining...
@@ -685,27 +633,24 @@ monitor_stop_whole_test(w, ctrls)
 	}
   if ((n = strlen (buf)) && buf [n-1] == '\n') buf [n-1] = '\0';
   msg  = emalloc(strlen(buf)+1);
-  type = parse_server_message(context, buf, (int)arg_get_value(ctrls, "MONITOR_BACKEND"), msg);
+  type = parse_server_message(context, buf, GPOINTER_TO_SIZE(arg_get_value(ctrls, "MONITOR_BACKEND")), msg);
   efree(&msg);
  }
   
  context->action = CONTEXT_IDLE;
  prefs_context_update(context);
  gtk_widget_hide(arg_get_value(ctrls, "WINDOW"));
- report_save(context, (int)arg_get_value(ctrls, "MONITOR_BACKEND"), NULL);
+ report_save(context, GPOINTER_TO_SIZE(arg_get_value(ctrls, "MONITOR_BACKEND")), NULL);
  return(FALSE);
 }
 
 
-/*
- * monitor_input_callback
- *
+/**
  * This function is called whenever there is new
- * data coming from the server. 
+ * data coming from the server.
  */
-void 
-monitor_input_callback(ctrls)
-    struct arglist * ctrls;
+void
+monitor_input_callback (struct arglist * ctrls)
 {
   int finished = 0;
   static char * buf = NULL;
@@ -735,7 +680,7 @@ monitor_input_callback(ctrls)
         else return;
        }	
 
-  type = parse_server_message(context, buf, (int)arg_get_value(ctrls, "MONITOR_BACKEND"), msg);
+  type = parse_server_message(context, buf, GPOINTER_TO_SIZE(arg_get_value(ctrls, "MONITOR_BACKEND")), msg);
  
   switch(type)
   {
@@ -760,24 +705,22 @@ monitor_input_callback(ctrls)
     {
    scan_finished : 
       gtk_widget_hide(arg_get_value(ctrls, "WINDOW"));
-      gtk_idle_remove((int)arg_get_value(ctrls, "TAG"));
+      gtk_idle_remove(GPOINTER_TO_SIZE(arg_get_value(ctrls, "TAG")));
       gtk_widget_destroy(arg_get_value(ctrls, "WINDOW"));
-      report_save(context, (int)arg_get_value(ctrls, "MONITOR_BACKEND"), NULL);
+      report_save(context, GPOINTER_TO_SIZE(arg_get_value(ctrls, "MONITOR_BACKEND")), NULL);
       context->action = CONTEXT_IDLE;
       prefs_context_update(context);
     }
 }
- 
-/*
- * monitor_stop_test
- *
- * this function stops one test
+
+/**
+ * @brief This function stops one test.
  */
-void 
-monitor_stop_test(GtkWidget * w,struct context * context)
+void
+monitor_stop_test (GtkWidget * w,struct context * context)
 {
- char * hostname = gtk_object_get_data(GTK_OBJECT(w), "hostname");
- network_printf(context->socket, "CLIENT <|> STOP_ATTACK <|> %s <|> CLIENT\n", hostname);
+ char * hostname = gtk_object_get_data (GTK_OBJECT(w), "hostname");
+ network_printf (context->socket, "CLIENT <|> STOP_ATTACK <|> %s <|> CLIENT\n", hostname);
 }
- 
-#endif
+
+#endif /* USE_GTK */
