@@ -28,6 +28,8 @@
  */
 
 #include <includes.h>
+/* for g_sprintf */
+#include <glib/gprintf.h>
 
 #include "nessus_i18n.h"
 
@@ -61,7 +63,9 @@
 #include "latex_output.h"
 #include "text_output.h"
 #include "xml_output_ng.h"
+#include "openvas_ssh_key_create.h" /* for ensure_dir, until we have a file_util module */
 #include "plugin_cache.h"
+#include "severity_filter.h"
 
 #include "cli.h"
 
@@ -77,7 +81,79 @@
 #define inc_optind() (optind++)
 #endif
 
+/**
+ * @mainpage
+ *
+ * @section Introduction
+ * @verbinclude README
+ * 
+ * @section License Information
+ * @verbinclude COPYING
+ */
+
+/**
+ * \file
+ * Starting point for both the GUI and the CLI client.
+ */
+
+/**
+ * @brief Arglist holding the GUI elements (most of them nested in other arglists).
+ * 
+ * Following overview is not complete yet.
+ * 
+ * MainDialog (arg)
+ * - USER (arg)
+ *   - FRAME
+ * - SAVE_KB (arg)
+ *   - FRAME
+ * - SCAN_OPTIONS (arg)
+ *   - FRAME
+ * - PLUGINS (arg)
+ *   - SILENT_DEPTS
+ *   - ENABLE_DEPS_AT_RUNTIME
+ *   - FRAME
+ *   - AUTO_ENABLE_PLUGINS
+ *   - PLUGIN_TREE
+ *   - FILTER_LABEL
+ * - NOTEBOOK
+ * - VBOX
+ * - REPORT (arg)
+ *   - FRAME
+ *   - BE
+ *   - ORDER_COMBOBOX
+ *   - REPORT_CONTEXT
+ *   - REPORT (GtkTextView)
+ *   - REPORT_TREE_VIEW
+ *   - SCAN_TIMESTAMPS
+ *   - VBOX
+ * - TARGET (arg)
+ *   - FRAME
+ *   - DNS_EXPAND
+ *   - READ_FROM_FILE
+ *   - TARGET
+ * - PLUGIN_PREFS (arg)
+ *   - FRAME_CREDENTIALS
+ *   - PLUGIN_CREDENTIALS
+ *   - FRAME
+ *   - PLUGIN_PREFS
+ * - SCAN_OPTIONS (arg)
+ *   - FRAME
+ *   - PING_HOSTS
+ *   - PORT_RANGE
+ *   - UNSCANNED_CLOSED
+ *   - MAX_HOSTS
+ *   - MAX_CHECKS
+ *   - CGI_PATH
+ *   - REVERSE_LOOKUP
+ *   - OPTIMIZE_TEST
+ *   - SAFE_CHECKS
+ *   - USE_MAC_ADDR
+ *   - SCANNERS_LIST
+ */
 struct arglist * MainDialog;
+
+extern severity_filter_t* global_filter;
+
 char * Alt_rcfile = NULL;
 struct plugin_filter Filter;
 static gboolean dont_check_ssl_cert = FALSE;
@@ -87,10 +163,8 @@ int F_show_pixmaps;
 int F_quiet_mode;
 int ListOnly = 0;
 
-void init_globals();
-
-/*
- * Initialize I18N support, if possible
+/**
+ * @brief Initialize I18N support, if possible
  */
 static void
 i18n_init(void)
@@ -134,14 +208,17 @@ verify_callback(preverify_ok, ctx)
 
 #ifdef CLN_AUTH_SRV
 
-/*
- * split a line "var=value" into two components
- * returns 0 if = was not found, 1 if line looks like "var=", 2 if OK
+/**
+ * @brief Split a line "var=value" into two components.
+ * 
+ * @param line A line. Attention, will be modified!
+ * @param[out] var If returned 2 contains the var name (points into \<line\> parameter).
+ * @param[out] val If returned 2 contains the value (points into \<line\> parameter).
+ * 
+ * @return 0 if '=' was not found, 1 if line looks like "var=", 2 if OK.
  */
 static int
-split_var_val(line, var, val)
-     const	char	*line;
-     char	*var, *val;
+split_var_val (const char *line, char* var, char* val)
 {
   const char	*p;
   char	*q;
@@ -158,7 +235,7 @@ split_var_val(line, var, val)
   return q == line ? 1 : 2;
 }
 
-/* 
+/**
  * Returns -1 if error, 0 if hash not found, 1 if found
  */
 static int
@@ -247,7 +324,7 @@ set_server_cert_hash(sname, hash)
     {
       while (fgets(line, sizeof(line), fp1) != NULL)
 	{
-	  x = strlen(line);
+	  x = strlen(line); /* RATS: ignore, fgets ensured nul termination */
 	  if (x > 0 && line[x - 1] != '\n') /* invalid line */
 	    continue;
 
@@ -305,19 +382,17 @@ set_server_cert_hash(sname, hash)
 #endif
 
 
-/*
- * connect_to_nessusd
- *
- * This function establishes the connection between
- * nessus and nessusd, logs in and reads the plugin
- * list from the server.
+/**
+ * @brief Establishes the connection between openvas-client and openvasd, logs
+ * @brief in and reads the plugin list from the server.
  *
+ * @return NULL in case of success, error message text otherwise.
  */
 char *
-connect_to_nessusd(context)
-	struct context * context;
+connect_to_nessusd (struct context * context)
 {
   const char * hostname;
+  gchar * certhash;
   int port;
   const char * login;
 #ifdef CLN_AUTH_SRV
@@ -340,14 +415,7 @@ connect_to_nessusd(context)
   int soc, soc2;
   int opt;
   const char *passwd;
-  /* Initialize the array with a string that will be big enough */
-  char proto[] = PROTO_NAME "md5_caching";
 
-  if (snprintf(proto, sizeof(proto),
-      prefs_get_int(context, "protocol_version") ? PROTO_NAME_OTP : PROTO_NAME,
-      prefs_get_int(Global, "cache_plugin_information") ? "md5_caching":"")
-      >= sizeof(proto))
-    return "programming error: proto_name buffer too small";
 
   if(prefs_get_int(Global, "nessusd_autoconnect"))
     passwd = estrdup(context->passwd);
@@ -502,6 +570,13 @@ Please launch openvasclient-mkrand(1) first !"));
 		  paranoia_level);
 	      paranoia_level = 1;
 	    }
+	  else if (!check_exists(trusted_ca))
+	    {
+	      show_warning(
+		  _("paranoia_level=%d but \"trusted_ca\" file not found:\n%s"),
+		  paranoia_level, trusted_ca);
+	      paranoia_level = 1;
+	    }
 	  else
 	    {
 	      SSL_CTX_set_verify(ssl_ctx, SSL_VERIFY_PEER, verify_callback);
@@ -551,10 +626,25 @@ Please launch openvasclient-mkrand(1) first !"));
 	{
 	  X509	*cert = SSL_get_peer_certificate(ssl);
 	  char	stored_hash[SHA_DIGEST_LENGTH];
-      
-	  if (get_server_cert_hash(hostname, stored_hash) <= 0)
+
+	  /*  Essentially store a key,value pair of 'hostname,port'=certificate hash
+	   *  Allows instances of openvasd to have their own server certs w/out confusing the cli 
+	   *  
+	   *  certhash is len(hostname)+1(comma)+5(port = )+1(\n) 
+	   *   
+	   */
+	  if((certhash = g_try_malloc(strlen(hostname)+10)) ==  NULL)
+	  {
+            fprintf(stderr, "could not allocate memory for certhash");
+            return "Out of memory error";
+	  }
+	  g_sprintf(certhash,"%s,%d",hostname,port);
+    
+	  if (get_server_cert_hash(certhash, stored_hash) <= 0)
 	    memset(stored_hash, 0, sizeof(stored_hash));
 
+	  g_free(certhash);
+
 	  if(cert == NULL)
 	    {
 	      sslerror("SSL_get_peer_certificate");
@@ -573,8 +663,19 @@ Please launch openvasclient-mkrand(1) first !"));
 		return _("Invalid server certificate");
 	      }
 
-	      if (set_server_cert_hash(hostname, cert->sha1_hash) < 0)
+	      /* certhash is len(hostname)+1(comma)+5(16bitport = )+1(\n)  */
+	      certhash = malloc(strlen(hostname)+10);
+
+              if((certhash = g_try_malloc(strlen(hostname)+10)) ==  NULL)
+              {
+                fprintf(stderr, "could not allocate memory for certhash");
+                return "Out of memory error";
+              }
+	      sprintf(certhash,"%s,%d",hostname,port);
+
+	      if (set_server_cert_hash(certhash, cert->sha1_hash) < 0)
 		perror(_("Could not save server certificate"));
+	      g_free(certhash);
 	    }
 	}
 #endif
@@ -592,7 +693,7 @@ Please launch openvasclient-mkrand(1) first !"));
   
   context->socket = soc;
 
-  if(comm_init(context->socket, proto) || (auth_login(context->socket, login, passwd)))
+  if(comm_init(context->socket, PROTO_NAME) || (auth_login(context->socket, login, passwd)))
     {
       close_stream_connection(context->socket);
       efree(&pem_pass);
@@ -609,9 +710,11 @@ Please launch openvasclient-mkrand(1) first !"));
     efree(&passwd);
     return(_("Login failed"));
   }
+
   comm_get_preferences(context);
   comm_get_rules(context);
   comm_get_dependencies(context);
+  comm_get_certificates(context);
 
   if(prefs_get_int(Global, "cache_plugin_information") > 0)
   {
@@ -628,8 +731,8 @@ Please launch openvasclient-mkrand(1) first !"));
   return(NULL);
 }
 
-
-int main(int argc, char * argv[])
+int
+main (int argc, char * argv[])
 {
   char * arg = NULL;
   gchar *myself = NULL;
@@ -650,7 +753,7 @@ int main(int argc, char * argv[])
 #ifdef USE_GTK  
   static gboolean no_pixmap = FALSE;
 #endif
-  static gboolean batch_mode = FALSE;
+  static gchar *batch_mode = NULL;
   static gboolean list_plugins = FALSE;
   static gboolean list_prefs = FALSE;
   static gchar *in_report = NULL;
@@ -671,10 +774,14 @@ int main(int argc, char * argv[])
 #ifdef USE_GTK
     { "no-pixmap", 'n', 0, G_OPTION_ARG_NONE, &no_pixmap, N_("No pixmaps"), NULL },
 #endif
-    { "batch-mode", 'q', 0, G_OPTION_ARG_NONE, &batch_mode, N_("Batch-mode scan"), N_("<host> <port> <user> <pass> <targets-file> <result-file>") },
+    { "batch-mode", 'q', 0, G_OPTION_ARG_STRING, &batch_mode, N_("Batch-mode scan"), N_("<host> <port> <user> <pass> <targets-file> <result-file>") },
     { "make-config-file", 'm', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &make_config_file, "", NULL },
     { "config-file", 'c', 0, G_OPTION_ARG_FILENAME, &config_file, N_("Configuration file"), N_("<.rcfile>") },
+#ifndef NO_GDCHART
     { "output-type", 'T', 0, G_OPTION_ARG_STRING, &output_type, N_("Output format"), N_("[nbe|html|html_graph|text|xml|tex]") },
+#else
+    { "output-type", 'T', 0, G_OPTION_ARG_STRING, &output_type, N_("Output format"), N_("[nbe|html|text|xml|tex]") },
+#endif
     { "verbose", 'V', 0, G_OPTION_ARG_NONE, &verbose, N_("Display status messages in batch mode"), NULL },
     { "list-plugins", 'p', 0, G_OPTION_ARG_NONE, &list_plugins, N_("Obtain list of plugins installed on the server"), NULL },
     { "list-prefs", 'P', 0, G_OPTION_ARG_NONE, &list_prefs, N_("Obtain list of server and plugin preferences"), NULL },
@@ -691,7 +798,7 @@ int main(int argc, char * argv[])
   option_context = g_option_context_new(_("- client for the OpenVAS security scanner"));
   g_option_context_add_main_entries(option_context, entries, NULL);
 #ifdef USE_GTK
-  g_option_context_add_group(option_context, gtk_get_option_group (TRUE));
+  g_option_context_add_group(option_context, gtk_get_option_group (FALSE));
 #endif
   if (!g_option_context_parse(option_context, &argc, &argv, &error))
   {
@@ -719,19 +826,20 @@ int main(int argc, char * argv[])
   {
     printf(_("OpenVAS-Client (%s) %s for %s\n\n"), PROGNAME, NESSUS_VERSION, NESS_OS_NAME);
     printf(_("NessusClient origin: (C) 1998 - 2003 Renaud Deraison <deraison@nessus.org>\n"));
-    printf(_("New code since OpenVAS-Client: (C) 2007, 2008 Intevation GmbH\n"));
+    printf(_("New code since OpenVAS-Client: (C) 2007, 2008, 2009 Intevation GmbH\n"));
     printf ("\n");
     exit(0);
   }
 
-  if (batch_mode)
+  if (batch_mode != NULL)
   {
     quiet_mode = TRUE;
   }
 
-  if (list_prefs)
+  if (list_prefs || list_plugins)
   {
-     ListOnly = 1;
+    ListOnly = 1;
+    quiet_mode = TRUE;
   }
 
 #ifdef ENABLE_SAVE_TESTS
@@ -788,8 +896,10 @@ int main(int argc, char * argv[])
         arglist_to_text(backend_convert(be), out_report);
       else if(!strcmp(type, "html"))
         arglist_to_html(backend_convert(be), out_report);
+#ifndef NO_GDCHART
       else if(!strcmp(type, "html_pie") || !strcmp(type, "html_graph"))
         arglist_to_html_graph(backend_convert(be), out_report);
+#endif
       else if(!strcmp(type, "nbe"))
         backend_to_nbe(be, out_report);
       else if(!strcmp(type, "xml"))
@@ -804,7 +914,7 @@ int main(int argc, char * argv[])
     }
     else
     {
-      fprintf(stderr, _("Could not import '%s' - is it a .nbe file?\n"), 
+      fprintf(stderr, _("Could not import '%s' - is it a .nbe file?\n"),
               in_report);
     }
     exit(0);
@@ -817,8 +927,8 @@ int main(int argc, char * argv[])
     exit(1);
   }
 
-#define BATCH_USAGE "-q host port user pass"
-#define BATCH_ARGC 4
+#define BATCH_USAGE "batch mode and <host> <port> <user> <pass> "
+#define BATCH_ARGC 3
 	
 #ifdef ENABLE_SAVE_TESTS
   if(list_sessions && (remaining_options_count < BATCH_ARGC) && !quiet_mode)
@@ -829,7 +939,7 @@ int main(int argc, char * argv[])
 
   if(restore_session && (remaining_options_count < BATCH_ARGC) && !quiet_mode)
   {
-    fprintf(stderr, _("restore-session requires -q %s result\n"), BATCH_USAGE);
+    fprintf(stderr, _("restore-session requires -q %s <result>\n"), BATCH_USAGE);
     exit(1);
   }
 
@@ -888,7 +998,7 @@ int main(int argc, char * argv[])
     cli_args_verbose(cli, verbose);
 
     /* with, or without ENABLE_CRYPTO_LAYER */
-#define NUM_ARGS 6
+#define NUM_ARGS 5
 
 #ifndef ENABLE_SAVE_TESTS
 
@@ -920,6 +1030,15 @@ int main(int argc, char * argv[])
         exit(1);
       }
     }
+    else if(list_prefs || list_plugins)
+    {
+      if (batch_mode == NULL || remaining_options_count < NUM_ARGS - 2)
+      {
+        fprintf(stderr, "%s" BATCH_USAGE "\n",
+                _("list-prefs and list-plugins require "));
+        exit(1);
+      }
+    }
     else if (remaining_options_count != NUM_ARGS) 
     {
       printf(_("Batch mode requires login information.\n"));
@@ -927,19 +1046,19 @@ int main(int argc, char * argv[])
       exit(0);
     }
 
-#endif
+#endif /* !ENABLE_SAVE_TESTS */
 
     /* next arguments: SERVER PORT */
-    cli_args_server(cli, remaining_options[0]);
-    cli_args_port(cli, atoi(remaining_options[1]));
+    cli_args_server(cli, batch_mode);
+    cli_args_port(cli, atoi(remaining_options[0]));
 
     /* next argument: LOGIN */
-    arg = remaining_options[2];
+    arg = remaining_options[1];
     cli_args_login(cli, arg);
     bzero(arg, strlen(arg));
 
     /* next argument: PASSWORD */
-    arg = remaining_options[3];
+    arg = remaining_options[2];
     cli_args_password(cli, arg);
     bzero(arg, strlen(arg));
 
@@ -978,20 +1097,20 @@ int main(int argc, char * argv[])
     {
       if(restore_session)
       {
-        cli_args_results(cli, remaining_options[4]);
+        cli_args_results(cli, remaining_options[3]);
       }
       else
       {
         if(!list_sessions)
         {
-          char * t = remaining_options[4];
+          char * t = remaining_options[3];
           if(t)
             cli_args_target(cli, t);
           else
           {
             fprintf(stderr, _("Missing parameter\n"));
           }
-          t = remaining_options[5];
+          t = remaining_options[4];
           if(t)
             cli_args_results(cli,  t);
           else
@@ -1031,9 +1150,13 @@ int main(int argc, char * argv[])
       cli_test_network(cli);
       cli_report(cli);
     }
+    /* Free session id. It's no longer needed */
+    g_free(session_id);
     /* end, exit */
     nessus_exit(0);
   }
+  /* Free session id. It's no longer needed */
+  g_free(session_id);
 
   /*
    * Set up the main window
@@ -1048,22 +1171,36 @@ int main(int argc, char * argv[])
 	if(paranoia_level > 0)
  	 prefs_set_int(context, "paranoia_level", paranoia_level);
     }
-#endif    
+#endif
 #ifdef USE_GTK
   prefs_dialog_setup(Global);
   prefs_context_update(Global);
 
+  // Initialize global severity_filter
+  gchar * filter_file = g_build_filename (prefs_get_nessushome(), ".openvas", "severity_overrides.xml", NULL);
+  global_filter = severity_filter_from_xml (filter_file);
+  if (global_filter == NULL)
+    {
+      gchar * filter_file_dir = g_build_filename (prefs_get_nessushome(), ".openvas", NULL); 
+      if (ensure_dir (filter_file_dir) == TRUE)
+        global_filter = severity_filter_new (_("Global Severity Overrides"), filter_file);
+      g_free (filter_file_dir);
+    }
+  g_free (filter_file);
+
+    signal(SIGINT, sighand_exit);
+    signal(SIGQUIT, sighand_exit);
+    signal(SIGKILL, sighand_exit);
+    signal(SIGTERM, sighand_exit);
+    
   gtk_main();
 
-  /* XXX: Server preferences are not saved here */
-  prefs_dialog_apply(Context, MainDialog);
-  context_save_recurse(Global);
+  /* FIXME: Server preferences are not saved here */
+  prefs_dialog_apply (Context, MainDialog);
 
-  /* FIXME: close all open connections
-  close_stream_connection(Context->socket);
-  Context->socket = -1;
-  */
-  nessus_exit(0);
+  context_save_recurse (Global);
+
+  nessus_exit (0);
   return 0; /* make the compiler happy */
 
 #else
@@ -1074,7 +1211,7 @@ int main(int argc, char * argv[])
   the --help option and in the OpenVAS documentation.\n"));
   exit (1);
 #endif
-}
+} /* main */
 
 #ifdef NESSUSNT
 int WINAPI WinMain(HINSTANCE hThisInst, HINSTANCE hPrevInst,
