@@ -26,8 +26,11 @@
  * file, but you are not obligated to do so.  If you do not wish to
  * do so, delete this exception statement from your version.
  *
- * Preferences  -- maps the content of the nessusrc file to memory
- *
+ */
+
+/**
+ * @file
+ * Maps the content of the openvasrc file to memory.
  */
 
 #include <includes.h>
@@ -36,6 +39,8 @@
 # include <gtk/gtk.h>
 #endif
 
+#include <glib.h>
+
 #include "nessus_plugin.h"
 #include "nessus_i18n.h"
 #include "context.h"
@@ -51,34 +56,8 @@ static char *get_username();
 int preferences_process(struct context *);
 char * preferences_get_filename(struct context *);
 
-static int
-plugin_id(plugin)
-  struct arglist *plugin;
-{
-  return (int)arg_get_value(plugin, "ID");
-}
-
-char *
-plugin_asc_id(plugin)
-  struct arglist *plugin;
-{
-  static char asc_id[21];
-  char *ret;
-
-  if((ret = arg_get_value(plugin, "ASC_ID")))
-    return ret;
-  else
-  {
-    bzero(asc_id, sizeof(asc_id));
-    sprintf(asc_id, "%d", plugin_id(plugin));
-    arg_add_value(plugin, "ASC_ID", ARG_STRING, strlen(asc_id), asc_id);
-    return asc_id;
-  }
-}
-
 int
-preferences_init(context)
-  struct context *context;
+preferences_init (struct context *context)
 {
   int result;
 
@@ -94,59 +73,59 @@ preferences_init(context)
  * at a propper place 
  * FIXED: nt used peks, which supports that feature
  */
-
-char *
-preferences_get_filename(context)
-  struct context *context;
+/**
+ * @brief Returns the location of the contexts openvasrc.
+ * 
+ * @param context Context of interest.
+ * 
+ * @return The location of the contexts openvasrc.
+ */
+gchar *
+preferences_get_filename (struct context * context)
 {
-  char *nessusrc;
+  gchar * openvasrc;
 
   if(context->dir)
   {
-    nessusrc = emalloc(strlen(context->dir) + strlen("/nessusrc") + 1);
-    sprintf(nessusrc, "%s/nessusrc", context->dir);
+    openvasrc = g_build_filename(context->dir, "openvasrc", NULL);
   }
   else
   {
     if(Alt_rcfile)
-      nessusrc = estrdup(Alt_rcfile);
+      openvasrc = estrdup(Alt_rcfile);
     else
     {
-      char *home = prefs_get_nessushome();
+      gchar *home = prefs_get_nessushome();
 
-      nessusrc = emalloc(strlen(home) + strlen("/.openvasrc") + 1);
-      sprintf(nessusrc, "%s/.openvasrc", home);
+      openvasrc = g_build_filename (home, ".openvasrc", NULL);
     }
   }
-  return nessusrc;
+  return openvasrc;
 }
 
 char *
-preferences_get_altname(context, ext)
-  struct context *context;
-  const char *ext;
+preferences_get_altname (struct context *context, const char *ext)
 {
-  char *nessusrc = preferences_get_filename(context);
+  gchar *nessusrc = preferences_get_filename (context);
 
-  if(ext && strlen(ext))
+  if (ext && strlen(ext))
   {
-    char *ret;
-
-    ret = emalloc(strlen(nessusrc) + 1 + strlen(ext) + 1);
-    sprintf(ret, "%s.%s", nessusrc, ext);
-    efree(&nessusrc);
+    gchar *ret = g_strdup_printf ("%s.%s", nessusrc, ext);
+    g_free (nessusrc);
     return ret;
   }
   else
     return nessusrc;
 }
 
+/**
+ * @return -1 in case of error, 0 otherwise.
+ */
 static int
-preferences_new(context)
-  struct context *context;
+preferences_new (struct context* context)
 {
   FILE *f;
-  char *fn = preferences_get_filename(context);
+  gchar *fn = preferences_get_filename(context);
 
   if(!fn)
     return (-1);
@@ -163,18 +142,18 @@ preferences_new(context)
     fprintf(f, _("# OpenVAS-Client Preferences File\n\n"));
     fprintf(f, "trusted_ca = cacert.pem\n");
     fprintf(f, "begin(SCANNER_SET)\n");
-    fprintf(f, "10180 = yes\n");
-    fprintf(f, "10278 = no\n");
-    fprintf(f, "10331 = no\n");
-    fprintf(f, "10335 = yes\n");
-    fprintf(f, "10841 = no\n");
-    fprintf(f, "10336 = no\n");
-    fprintf(f, "10796 = no\n");
-    fprintf(f, "11219 = no\n");
-    fprintf(f, "14259 = no\n");
-    fprintf(f, "14272 = no\n");
-    fprintf(f, "14274 = no\n");
-    fprintf(f, "14663 = no\n");
+    fprintf(f, " 1.3.6.1.4.1.25623.1.0.10180 = yes\n");
+    fprintf(f, " 1.3.6.1.4.1.25623.1.0.10278 = no\n");
+    fprintf(f, " 1.3.6.1.4.1.25623.1.0.10331 = no\n");
+    fprintf(f, " 1.3.6.1.4.1.25623.1.0.10335 = yes\n");
+    fprintf(f, " 1.3.6.1.4.1.25623.1.0.10841 = no\n");
+    fprintf(f, " 1.3.6.1.4.1.25623.1.0.10336 = no\n");
+    fprintf(f, " 1.3.6.1.4.1.25623.1.0.10796 = no\n");
+    fprintf(f, " 1.3.6.1.4.1.25623.1.0.11219 = no\n");
+    fprintf(f, " 1.3.6.1.4.1.25623.1.0.14259 = no\n");
+    fprintf(f, " 1.3.6.1.4.1.25623.1.0.14272 = no\n");
+    fprintf(f, " 1.3.6.1.4.1.25623.1.0.14274 = no\n");
+    fprintf(f, " 1.3.6.1.4.1.25623.1.0.14663 = no\n");
     fprintf(f, "end(SCANNER_SET)\n\n");
 
     fprintf(f, "begin(SERVER_PREFS)\n");
@@ -187,28 +166,25 @@ preferences_new(context)
     fprintf(f, " cgi_path = /cgi-bin:/scripts\n");
     fprintf(f, " port_range = default\n");
     fprintf(f, " auto_enable_dependencies = yes\n");
-    fprintf(f, " silent_dependencies = yes\n");
+    fprintf(f, " silent_dependencies = no\n");
     /* end of values from nessusd/preferences.c */
     fprintf(f, "end(SERVER_PREFS)\n");
   }
 
-  fclose(f);
-  chmod(fn, 0600);
-  efree(&fn);
+  fclose (f);
+  chmod (fn, 0600);
+  g_free (fn);
   return (0);
 }
 
 int
-preferences_process(context)
-  struct context *context;
+preferences_process (struct context *context)
 {
   return preferences_process_filename(context, NULL);
 }
 
 int
-preferences_process_filename(context, filename)
-  struct context *context;
-  char *filename;
+preferences_process_filename (struct context *context, gchar *filename)
 {
   FILE *fd;
   char *buffer;
@@ -216,7 +192,7 @@ preferences_process_filename(context, filename)
 
   if(default_filename)
   {
-    filename = preferences_get_filename(context);
+    filename = preferences_get_filename (context);
     chmod(filename, 0600);
   }
 
@@ -266,14 +242,16 @@ preferences_process_filename(context, filename)
     }
   }
 
+  /* Parse file that has been opened without error */
   buffer = emalloc(4096);
   while(!feof(fd) && fgets(buffer, 4096, fd))
   {
     if(strchr(buffer, '='))
       prefs_buffer_parse(buffer, context->prefs, 1);
-    else if(!strncmp(buffer, "begin(", strlen("begin(")))
+    else if(!strncmp(buffer, "begin(",
+                      strlen("begin("))) /* RATS: ignore, string literals are nul terminated */
     {
-      char *t = buffer + (strlen("begin(") * sizeof(char));
+      char *t = buffer + (strlen("begin(") * sizeof(char)); /* RATS: ignore, string literals are nul terminated */
       char *end = strchr(t, ')');
 
       if(!end)
@@ -303,10 +281,7 @@ preferences_process_filename(context, filename)
 }
 
 static int
-prefs_add_subcategory(arglist, fd, keep_order)
-  struct arglist *arglist;
-  FILE *fd;
-  int keep_order;
+prefs_add_subcategory (struct arglist *arglist, FILE *fd, int keep_order)
 {
   char *buffer = emalloc(4096);
   int flag = 0;
@@ -325,18 +300,38 @@ prefs_add_subcategory(arglist, fd, keep_order)
   return (0);
 }
 
+/**
+ * Workaround to cope with internal storage and communication of preference values.
+ * @return TRUE if in argl a key prefname has either the value 'yes' or 1,
+ *         FALSE otherwise.
+ */
+gboolean
+preferences_yes_or_one (struct arglist* argl, char* prefname)
+{
+  // Check various malconditions
+  if (argl == NULL || prefname == NULL)
+    return FALSE;
+
+  // Check for int and 1 or string and 'yes'
+  void* argval = arg_get_value (argl, prefname);
+  int argtype = GPOINTER_TO_INT(arg_get_type (argl, prefname));
+  
+  if (    (argtype == ARG_INT    && GPOINTER_TO_INT(argval) == 1)
+       || (argtype == ARG_STRING && argval && !strcmp(argval, "yes")) )
+    return TRUE;
+  
+  return FALSE;
+}
+
 static int
-prefs_buffer_parse(buffer, arglist, keep_order)
-  char *buffer;
-  struct arglist *arglist;
-  int keep_order;
+prefs_buffer_parse (char *buffer, struct arglist *arglist, int keep_order)
 {
   char *t;
   char *opt;
   char *value;
   int val = -1;
 
-  /* If we need to heep the order, we have to use arg_add_value,
+  /* If we need to keep the order, we have to use arg_add_value,
    * otherwise we can use the faster (for long lists much faster)
    * arg_add_value_at_head */
   void (*arg_add)(struct arglist *, const char *, int, long, void *)
@@ -367,11 +362,11 @@ prefs_buffer_parse(buffer, arglist, keep_order)
       val = 1;
     if(!strcmp(t, "no"))
       val = 0;
-
+    
     if(!strcmp(opt, "paranoia_level") || !strcmp(opt, "nessusd_port") ||
         !strcmp(opt, "sort_order") || !strcmp(opt, "protocol_version"))
     {
-      arg_add(arglist, opt, ARG_INT, sizeof(int), (void *)atoi(t));
+      arg_add(arglist, opt, ARG_INT, sizeof(gpointer), GSIZE_TO_POINTER(atoi(t)));
     }
     else
     {
@@ -383,7 +378,7 @@ prefs_buffer_parse(buffer, arglist, keep_order)
 	arg_add(arglist, opt, ARG_STRING, strlen(value), value);
       }
       else
-	arg_add(arglist, opt, ARG_INT, sizeof(int), (void *)val);
+	arg_add(arglist, opt, ARG_INT, sizeof(gpointer), GSIZE_TO_POINTER(val));
     }
     return (0);
   }
@@ -393,25 +388,26 @@ prefs_buffer_parse(buffer, arglist, keep_order)
 
 
 static void
-new_pluginset(pluginset, plugins)
-  struct arglist *pluginset;
-  struct nessus_plugin *plugins;
+new_pluginset (struct arglist *pluginset, struct nessus_plugin *plugins)
 {
   while(plugins != NULL )
   {
-    char name[32];
-
-    snprintf(name, sizeof(name), "%d", plugins->id);
-    arg_add_value(pluginset, name, ARG_INT, sizeof(int), (void*)(plugins->enabled != 0) );
+    gchar* name = g_strdup (plugins->oid);
+    arg_add_value(pluginset, name, ARG_INT, sizeof(gpointer), GSIZE_TO_POINTER(plugins->enabled != 0) );
     plugins = plugins->next;
+    g_free (name);
   }
 }
 
+/**
+ * @brief Gets or creates the plugin set (arglist) with name \<name\> of a 
+ * @brief context.
+ * 
+ * If the set is created, all plugins in \<plugins\> are used to create the set.
+ */
 struct arglist *
-prefs_get_pluginset(context, name, plugins)
-  struct context *context;
-  char *name;
-  struct nessus_plugin *plugins;
+prefs_get_pluginset (struct context *context, char *name,
+                     struct nessus_plugin *plugins)
 {
   struct arglist *plugin_set = arg_get_value(context->prefs, name);
 
@@ -424,39 +420,47 @@ prefs_get_pluginset(context, name, plugins)
   return plugin_set;
 }
 
+/*
+ * TODO: This whole private hash implementation should be replaced by
+ * a adequate fast search method from glib or a least be consolidated
+ * with other hash implementation that occur in OpenVAS.
+ * XXX: This hashing is probably less performant since it was migrated from
+ * integer names to string names.
+ */
 
 #define MAGIC 8197
 struct hash
 {
-  int name;
+  char * name;
   struct arglist *v;
   struct hash *next;
 };
 
 static struct arglist *
-hash_get(struct hash **hash, int id)
+hash_get(struct hash **hash, const char * oid)
 {
-  int idx = id % MAGIC;
-  struct hash *h = hash[idx];
+  int id, dummy;
+  struct hash *h;
+
+  sscanf(oid, "1.3.6.1.4.1.25623.1.%d.%d", &dummy, &id);
+  h = hash[id % MAGIC];
 
   while(h != NULL)
   {
-    if(h->name == id)
+    if(!strcmp(h->name, oid))
       return h->v;
     h = h->next;
   }
   return NULL;
 }
 
-/*
- * update arglists "PLUGIN_SET" or "SCANNER_SET"
- * from context->plugins or context->scanners
+/**
+ * @brief Update arglists "PLUGIN_SET" or "SCANNER_SET" from context->plugins
+ * @brief or context->scanners.
  */
 void
-pluginset_reload(context, name, plugins)
-  struct context *context;
-  char *name;
-  struct nessus_plugin *plugins;
+pluginset_reload (struct context *context, char *name, 
+                  struct nessus_plugin *plugins)
 {
   struct arglist *pluginset;
   struct hash **hash;
@@ -471,24 +475,24 @@ pluginset_reload(context, name, plugins)
 
   while(pluginset->next)
   {
-    int id = atoi(pluginset->name);
-    int idx = id % MAGIC;
+    int dummy, id;
+
+    sscanf(pluginset->name, "1.3.6.1.4.1.25623.1.%d.%d", &dummy, &id);
 
     h = emalloc(sizeof(struct hash));
-    h->name = id;
+    h->name = pluginset->name;
     h->v = pluginset;
-    h->next = hash[idx];
-    hash[idx] = h;
+    h->next = hash[id % MAGIC];
+    hash[id % MAGIC] = h;
     pluginset = pluginset->next;
   }
 
   while(plugins != NULL )
   {
-    int id = plugins->id;
-    struct arglist *pluginset_entry = hash_get(hash, id);
+    struct arglist *pluginset_entry = hash_get(hash, plugins->oid);
 
     if(pluginset_entry != NULL )
-       pluginset_entry->value = (void *)(plugins->enabled != 0 );
+       pluginset_entry->value = GSIZE_TO_POINTER(plugins->enabled != 0 );
     plugins = plugins->next;
   }
 
@@ -499,8 +503,7 @@ pluginset_reload(context, name, plugins)
 }
 
 void
-preferences_save(context)
-  struct context *context;
+preferences_save (struct context *context)
 {
   char *filename = preferences_get_filename(context);
 
@@ -509,9 +512,7 @@ preferences_save(context)
 }
 
 void
-preferences_save_as(context, filename)
-  struct context *context;
-  char *filename;
+preferences_save_as (struct context *context, char *filename)
 {
   FILE *fd;
   struct arglist *t;
@@ -523,7 +524,7 @@ preferences_save_as(context, filename)
     return;
   }
   chmod(filename, 0600);
-  fprintf(fd, _("# This file was automagically created by OpenVAS-Client\n"));
+  fprintf(fd, _("# This file was automatically created by OpenVAS-Client\n"));
 
   pluginset_reload(context, "PLUGIN_SET", context->plugins);
   pluginset_reload(context, "SCANNER_SET", context->scanners);
@@ -535,7 +536,7 @@ preferences_save_as(context, filename)
     {
       if(!strcmp(t->name, "paranoia_level") || !strcmp(t->name, "nessusd_port")
           || !strcmp(t->name, "sort_order") || !strcmp(t->name, "protocol_version"))
-	fprintf(fd, "%s = %d\n", t->name, (int)t->value);
+	fprintf(fd, "%s = %d\n", t->name, (int)GPOINTER_TO_SIZE(t->value));
       else
 	fprintf(fd, "%s = %s\n", t->name, t->value ? "yes" : "no");
     }
@@ -544,6 +545,8 @@ preferences_save_as(context, filename)
     t = t->next;
   }
 
+  fprintf(fd, "\n");
+
   t = context->prefs;
   while(t && t->next)
   {
@@ -579,9 +582,7 @@ preferences_save_as(context, filename)
 
 
 int
-preferences_generate_new_file(context, name)
-  struct context *context;
-  const char *name;
+preferences_generate_new_file (struct context *context, const char *name)
 {
   struct context *copy_from = context->parent;
 
@@ -595,12 +596,12 @@ preferences_generate_new_file(context, name)
     if(copy_from)
     {
       context_type type = copy_from->type;
-      char *filename = preferences_get_filename(context);
+      gchar *filename = preferences_get_filename (context);
 
       copy_from->type = context->type;
       preferences_save_as(copy_from, filename);
       copy_from->type = type;
-      efree(&filename);
+      g_free (filename);
     }
     else
       preferences_new(context);
@@ -616,25 +617,22 @@ preferences_generate_new_file(context, name)
 
 #ifdef USE_GTK
 gboolean
-prefs_is_executable(name)
-  gchar *name;
+prefs_is_executable (gchar *name)
 {
-  gchar *prog = g_find_program_in_path(name);
+  gchar *prog = g_find_program_in_path (name);
 
-  if(prog)
-  {
-    g_free(prog);
-    return TRUE;
-  }
+  if (prog)
+    {
+      g_free (prog);
+      return TRUE;
+    }
   else
     return FALSE;
 }
 #endif /* USE_GTK */
 
 void *
-prefs_get_default(context, name)
-  struct context *context;
-  const char *name;
+prefs_get_default (struct context *context, const char *name)
 {
   if(!strcmp(name, "name"))
   {
@@ -658,7 +656,7 @@ prefs_get_default(context, name)
   else if(!strcmp(name, "nessusd_user"))
     return get_username();
   else if(!strcmp(name, "nessusd_port"))
-    return (void *)NESIANA_PORT;
+    return (void *)OPENVAS_IANA_OTP_PORT;
   else if(!strcmp(name, "ssl_version"))
     return SSL_VER_DEF_NAME;
   else if(!strcmp(name, "nessus_dir"))
@@ -667,8 +665,7 @@ prefs_get_default(context, name)
     if(!nessus_dir)
     {
       char *nessushome = prefs_get_nessushome();
-      nessus_dir = emalloc(strlen(nessushome) + strlen("/.openvas") + 1);
-      sprintf(nessus_dir, "%s/.openvas", nessushome);
+      nessus_dir = g_build_filename( nessushome, ".openvas", NULL);
     }
     return nessus_dir;
   }
@@ -701,7 +698,7 @@ prefs_get_default(context, name)
   else if(!strcmp(name, "url_bid"))
     return "http://www.securityfocus.com/bid/%s";
   else if(!strcmp(name, "url_nessus"))
-    return "http://www.openvas.org/?oid=1.3.6.1.4.1.25623.1.0.%s";
+    return "http://www.openvas.org/?oid=%s";
   else if(!strcmp(name, "targets"))
     return "localhost";
   else if(!strcmp(name, "tree_autoexpand"))
@@ -716,30 +713,39 @@ prefs_get_default(context, name)
     return (void *)1;
   else if(!strcmp(name, "show_nvt_name_and_oid"))
     return (void *)1;
+  else if(!strcmp(name, "auto_enable_new_plugins"))
+    return (void *)1;
   else
     return NULL;
 }
 
+/**
+ * @brief Returns either the user-set value of a preference or its Default.
+ * 
+ * As this function returns a void pointer, it is wrapped in the macros
+ * prefs_get_int and prefs_get_string. Use these, as you should know which
+ * type you expect.
+ * 
+ * @param context Context to query preferences of.
+ * @param name    Preference name (e.g. "trusted_ca").
+ * 
+ * @return Value of unknown type (thus use prefs_get_int or prefs_get_string).
+ */
 void *
-prefs_get_value(context, name)
-  struct context *context;
-  const char *name;
+prefs_get_value (struct context *context, const char *name)
 {
   void *value = arg_get_value(context->prefs, name);
   int type = arg_get_type(context->prefs, name);
 
   if(type < 0 || (type != ARG_INT && value == NULL)
       || (type == ARG_STRING && !strlen((const char *)value)))
-    value = prefs_get_default(context, name);
+    value = prefs_get_default (context, name);
   return value;
 }
 
 void
-prefs_set_value(context, name, value, type)
-  struct context *context;
-  const char *name;
-  void *value;
-  int type;
+prefs_set_value (struct context *context, const char *name, void *value,
+                 int type)
 {
   int len;
   int arg_type;
@@ -780,19 +786,13 @@ prefs_set_value(context, name, value, type)
 }
 
 void
-prefs_set_int(context, name, value)
-  struct context *context;
-  const char *name;
-  int value;
+prefs_set_int (struct context *context, const char *name, int value)
 {
-  prefs_set_value(context, name, (void *)value, ARG_INT);
+  prefs_set_value(context, name, GSIZE_TO_POINTER(value), ARG_INT);
 }
 
 void
-prefs_set_string(context, name, value)
-  struct context *context;
-  const char *name;
-  const char *value;
+prefs_set_string (struct context *context, const char *name, const char *value)
 {
   const char *default_value = prefs_get_default(context, name);
   if(!default_value || strcmp(value, default_value))
@@ -803,8 +803,7 @@ prefs_set_string(context, name, value)
 
 /* are there any options besides "name" and "comment"? */
 int
-prefs_has_options(context)
-  struct context *context;
+prefs_has_options (struct context *context)
 {
   struct arglist *t = context->prefs;
 
@@ -819,8 +818,8 @@ prefs_has_options(context)
 }
 
 
-/*
- * get_username : returns the name of the current user
+/**
+ * @return The name of the current user or empty string if none found.
  */
 static char *
 get_username(void)
@@ -845,21 +844,21 @@ get_username(void)
 }
 
 char *
-prefs_get_nessushome(void)
+prefs_get_nessushome (void)
 {
   char *home;
   struct passwd *pwd;
 
   home = getenv("NESSUSHOME");
-  if(home)
+  if (home)
     return home;
 
-  home = getenv("HOME");
-  if(home)
+  home = getenv ("HOME");
+  if (home)
     return home;
 
-  pwd = getpwuid(getuid());
-  if(pwd && pwd->pw_dir)
+  pwd = getpwuid (getuid());
+  if (pwd && pwd->pw_dir)
     return pwd->pw_dir;
 
   return "";
