@@ -44,32 +44,62 @@
 #include "../xpm/connect_16.xpm"
 #include "../xpm/disconnect_16.xpm"
 
+/**
+ * Sets sensitivity of a GTK Widget that is registered as name in the arglist
+ * MainDialog.
+ * 
+ * @param name   Name of the entry for the GTK Widget in the arglist MainDialog.
+ * @param enable If true the widget will be sensitive, if false it will be
+ *               insensitive (not reacting to user input).
+ */
 void
-prefs_context_enable_widget(name, enabled)
-  const gchar *name;
-  gboolean enabled;
+prefs_context_enable_widget (const gchar* name, gboolean enabled)
 {
   GtkWidget *widget = GTK_WIDGET(arg_get_value(MainDialog, name));
 
   gtk_widget_set_sensitive(widget, enabled);
 }
 
+/**
+ * @brief Dis/Enables items in menu and buttons, according to context type and
+ * @brief state (action).
+ * 
+ * E.g. disables menu items in the report menu if currently selected context
+ * is not a report.
+ * 
+ * @param context The context according to which switch sensitivity of widgets.
+ */
 void
-prefs_context_update_widgets(context)
-  struct context *context;
+prefs_context_update_widgets (struct context *context)
 {
   static context_type last_type = CONTEXT_ENUM;
   static context_action last_action = CONTEXT_IDLE;
   context_type type = context->type;
   context_type action = context->action;
   int do_enable;
-  
+
+  // Set the text of "apply overrides" menu- item according to state of context
+  // Handier way of doing this (item_set_label) in Gtk 2.16
+  GtkWidget *overrides_menu_item = GTK_WIDGET(arg_get_value(MainDialog, "APPLYOVERRIDESTOREPORT_MENUITEM"));
+  GtkWidget *menu_label = gtk_bin_get_child(GTK_BIN(overrides_menu_item));
+  if (context->is_severity_mapped == TRUE)
+    {
+      gtk_label_set_text(GTK_LABEL(menu_label), _("Undo severity override"));
+    }
+    else
+    {
+      gtk_label_set_text(GTK_LABEL(menu_label), _("Override severities"));
+    }
+
   if(type != last_type || action != last_action )
   {
     prefs_context_enable_widget("RENAMETASK_MENUITEM", type >= CONTEXT_TASK);
     prefs_context_enable_widget("DELETETASK_MENUITEM", type >= CONTEXT_TASK);
 
     prefs_context_enable_widget("EXECSCOPE_MENUITEM", type >= CONTEXT_SCOPE && action == CONTEXT_IDLE );
+#if 0
+    prefs_context_enable_widget("EDITPRIORITIES_MENUITEM", type >= CONTEXT_REPORT);
+#endif
     prefs_context_enable_widget("NEWSCOPE_MENUITEM", type >= CONTEXT_TASK);
     prefs_context_enable_widget("RENAMESCOPE_MENUITEM", type >= CONTEXT_SCOPE);
     prefs_context_enable_widget("DELETESCOPE_MENUITEM", type >= CONTEXT_SCOPE);
@@ -78,6 +108,9 @@ prefs_context_update_widgets(context)
 
     prefs_context_enable_widget("RENAMEREPORT_MENUITEM", type >= CONTEXT_REPORT);
     prefs_context_enable_widget("DELETEREPORT_MENUITEM", type >= CONTEXT_REPORT);
+
+    prefs_context_enable_widget("APPLYOVERRIDESTOREPORT_MENUITEM", type >= CONTEXT_REPORT);
+
     prefs_context_enable_widget("IMPORTREPORT_MENUITEM", type >= CONTEXT_SCOPE);
     prefs_context_enable_widget("EXPORTREPORT_MENUITEM", type >= CONTEXT_REPORT);
     prefs_context_enable_widget("PRINTREPORT_MENUITEM", type >= CONTEXT_REPORT);
@@ -132,11 +165,12 @@ prefs_context_enable_notebookpage(notebook, pagenr, enable)
   gtk_widget_set_sensitive(page, enable);
 }
 
+/**
+ * @brief Dis/enable notebook pages ("tabs" on the right side of the gui like
+ * Comment, Options, Report).
+ */
 void
-prefs_context_update_notebook(comment, options, report)
-  gboolean comment;
-  gboolean options;
-  gboolean report;
+prefs_context_update_notebook (gboolean comment, gboolean options, gboolean report)
 {
   GtkWidget *notebook = arg_get_value(MainDialog, "CONTEXTNOTEBOOK");
 
@@ -153,8 +187,7 @@ prefs_context_update_notebook(comment, options, report)
 }
 
 void
-prefs_context_update_comment(context)
-  struct context *context;
+prefs_context_update_comment (struct context *context)
 {
   fill_comment(context, arg_get_value(MainDialog, "COMMENT"));
 }
@@ -201,37 +234,43 @@ prefs_context_update_options(context)
   }
 }
 
+/**
+ * Rewires MainDialog->'REPORT' arglist BE and REPORT_CONTEXT entries,
+ * then calls scopetreeview_counters_update to update the table in the left part
+ * of the GUI and prefs_report_update
+ */
 void
-prefs_context_update_report(context)
-  struct context *context;
+prefs_context_update_report (struct context *context)
 {
   int be;
   struct arglist *ctrls_report;
   char *fname;
 
-  /*
-   * load the corresponding report and update the report page
-   */
+  /* Load the corresponding report and update the report page */
   fname = report_get_filename(context);
   ctrls_report = arg_get_value(MainDialog, "REPORT");
-  /* but first discard the current one */
-  be = (int)arg_get_value(ctrls_report, "BE");
-  if (be >= 0) {
-    backend_dispose(be);
-    arg_set_value(ctrls_report, "BE", sizeof(int), (void *)-1);
-    arg_set_value(ctrls_report, "REPORT_CONTEXT", -1, (void *)NULL);
-  }
-  if (fname && check_is_file(fname)) {
-    be = backend_import_report(fname);
-    efree(&fname);
-    arg_set_value(ctrls_report, "BE", sizeof(int), (void *)be);
-    arg_set_value(ctrls_report, "REPORT_CONTEXT", -1, (void *)context);
-    scopetreeview_counters_update(context, be, fname);
-  }
+
+  /* But first discard the current one */
+  be = GPOINTER_TO_SIZE (arg_get_value(ctrls_report, "BE"));
+  if (be >= 0)
+    {
+      backend_dispose(be);
+      arg_set_value(ctrls_report, "BE", sizeof(int), (void *)-1);
+      arg_set_value(ctrls_report, "REPORT_CONTEXT", -1, (void *)NULL);
+    }
+  if (fname && check_is_file(fname))
+    {
+      be = backend_import_report(fname);
+      efree(&fname);
+      arg_set_value(ctrls_report, "BE", sizeof(gpointer), GSIZE_TO_POINTER(be));
+      arg_set_value(ctrls_report, "REPORT_CONTEXT", -1, (void *)context);
+      scopetreeview_counters_update(context, be, fname);
+    }
   prefs_report_update(ctrls_report, FALSE);
 }
 
-/* Update the connection state image.
+/**
+ * @brief Update the connection state image.
  *
  * If is_connected is true, the picture shown in the image is set to
  * connect_16, otherwise it is set to disconnect_16.
@@ -257,10 +296,17 @@ update_connection_image(image, is_connected)
       is_connected ? connected : disconnected);
 }
 
-
+/**
+ * @brief Updates connection icon and string in status bar.
+ *
+ * The statusbar in the OpenVAS GUI hosts a small icon showing if the currently
+ * selected task (or Globals) is connected or not and a string showing 
+ * connection details (e.g. "Connection: user@localhost").
+ * 
+ * @param context The context to draw information from.
+ */
 void
-prefs_context_update_statusbar(context)
-  struct context *context;
+prefs_context_update_statusbar (struct context *context)
 {
   GtkStatusbar * statusbar = GTK_STATUSBAR(arg_get_value(MainDialog,
 	"CONNECT_STATUSBAR"));
@@ -275,7 +321,7 @@ prefs_context_update_statusbar(context)
 
   if(context->socket >= 0)
   {
-    if (port == (int)prefs_get_default(context, "nessusd_port"))
+    if (port == GPOINTER_TO_SIZE(prefs_get_default(context, "nessusd_port")))
       connect_status = g_strdup_printf(_("Connection: %s@%s"),
 	username, hostname);
     else if (port >= 0)
@@ -298,8 +344,7 @@ prefs_context_update_statusbar(context)
 }
 
 void
-prefs_context_update(context)
-  struct context *context;
+prefs_context_update (struct context *context)
 {
   GtkWidget *scopetree = GTK_WIDGET(
       arg_get_value(arg_get_value(MainDialog, "SCOPETREE"), "TREEVIEW"));
@@ -342,7 +387,7 @@ prefs_context_update(context)
 	  prefs_get_string(context, "name"));
       break;
     case CONTEXT_SCOPE:
-      /* disable options and comments while scanning */
+      /* Disable options and comments while scanning */
       prefs_context_update_notebook(context->action != CONTEXT_SCANNING,
 	  context->action != CONTEXT_SCANNING, FALSE);
       text = g_strdup_printf(_("Scope: %s (Task: %s)"),
@@ -351,9 +396,9 @@ prefs_context_update(context)
       break;
     case CONTEXT_REPORT:
       prefs_context_update_notebook(TRUE, prefs_has_options(context), TRUE);
-      text = g_strdup_printf(_("Report for scope: %s (Task: %s)"),
-	  prefs_get_string(context->parent, "name"),
-	  prefs_get_string(context->parent->parent, "name"));
+      text = g_strdup_printf (_("Report for scope: %s (Task: %s)"),
+                              prefs_get_string(context->parent, "name"),
+                              prefs_get_string(context->parent->parent, "name"));
       break;
     default:
       show_error(_("prefs_context_update called with illegal context."));
