@@ -26,10 +26,19 @@
  * do so, delete this exception statement from your version.
  */
 
+/**
+ * @file
+ * Module to create large parts of the GUI.\n
+ * Most visible GTK Widgets are hooked into arglist(s) like MainDialog (defined
+ * in nessus.c).
+ * 
+ */
+
 #include <includes.h>
 
 #include "nessus_i18n.h"
 #include "nessus_plugin.h"
+#include "nvt_pref_sshlogin.h"
 
 #ifdef USE_GTK
 #include "comm.h"
@@ -49,6 +58,8 @@
 #include "prefs_options.h"
 #include "prefs_report.h"
 #include "prefs_scan_assistant.h"
+/* We act on the tree-view showing contexts, needed for prefs_scope_tree_update_report  */
+#include "prefs_scope_tree.h"
 #include "prefs_comment.h"
 #include "prefs_dialog_user.h"
 
@@ -57,7 +68,12 @@
 # include "prefs_kb.h"
 #endif
 
+/* We do non-capsulated things on the tree-view showing contexts */
+#include "backend.h"
 #include "error_dlg.h"
+#include "severityfiltermngr_dlg.h"
+#include "severity_filter.h"
+#include "ssh_keys_dialog.h"
 #include "monitor_dialog.h"
 #include "report.h"
 #include "report_save.h"
@@ -82,17 +98,17 @@
 #include "../xpm/info.xpm"
 #include "../xpm/info_small.xpm"
 
+/* temporary, should be part of the global context, comes from severity_filter.c */
+extern severity_filter_t* global_filter;
 
 static void prefs_dialog_set_tooltips(struct arglist *);
 
-/*
- * launch the users manual pdf document with the configure
+/**
+ * Launch the users manual pdf document with the configured
  * pdf viewer.
  */
 void
-show_users_manual(menuitem, ctrls)
-  GtkWidget *menuitem;
-  gpointer ctrls;
+show_users_manual (GtkWidget *menuitem, gpointer ctrls)
 {
 #ifndef CYGWIN
 #ifdef NESSUS_DOCDIR
@@ -105,9 +121,8 @@ show_users_manual(menuitem, ctrls)
                  "share\\doc\\openvas-manual");
 #endif /* CYGWIN */
 
-  /* the pdf file is covered by gettext to allow
-   * speficication of translation in the respective po
-   * files rather than in the source code. */
+  /* PDF file path. String is covered by gettext to allow speficication of 
+   * translation in the respective po files rather than in the source code. */
   char * fname = _("users-manual.pdf");
   char * path = g_build_filename(dir, fname, NULL);
 
@@ -115,13 +130,11 @@ show_users_manual(menuitem, ctrls)
   g_free(path);
 }
 
-/*
- * Save the current global settings to ~/.nessusrc
+/**
+ * @brief Save the current global settings to ~/.openvasrc .
  */
 void
-save_global_settings(menuitem, ctrls)
-  GtkWidget *menuitem;
-  gpointer ctrls;
+save_global_settings (GtkWidget *menuitem, gpointer ctrls)
 {
   prefs_dialog_apply(Global, ctrls);
   prefs_dialog_apply_plugin_prefs(Global->plugins);
@@ -131,13 +144,14 @@ save_global_settings(menuitem, ctrls)
   show_info(_("The global settings have been saved."));
 }
 
-/*
- * Toggle Toolbar on/off
+/**
+ * @brief Toggle Toolbar on/off.
+ * 
+ * @param menuitem Check box menu item for toggling visibility of the toolbar.
+ * @param ctrls    Arglist where the TOOLBAR is hooked into.
  */
 void
-toggle_toolbar(menuitem, ctrls)
-  GtkWidget *menuitem;
-  struct arglist *ctrls;
+toggle_toolbar (GtkWidget *menuitem, struct arglist *ctrls)
 {
   GtkWidget *toolbar = arg_get_value(ctrls, "TOOLBAR");
 
@@ -153,13 +167,15 @@ toggle_toolbar(menuitem, ctrls)
   }
 }
 
-/*
- * Toggle Message-Log on/off
+
+/**
+ * @brief Toggle Message-Log on/off.
+ * 
+ * @param menuitem Check box menu item for toggling the message frame display.
+ * @param ctrls    Arglist where the MSGLOGFRAME is hooked into.
  */
 void
-toggle_msglog(menuitem, ctrls)
-  GtkWidget *menuitem;
-  struct arglist *ctrls;
+toggle_msglog (GtkWidget *menuitem, struct arglist *ctrls)
 {
   GtkWidget *msglogframe = arg_get_value(ctrls, "MSGLOGFRAME");
 
@@ -176,24 +192,18 @@ toggle_msglog(menuitem, ctrls)
 }
 
 
-/*
- * Menu items
+/**
+ * @brief Add a menu item with optional icon and signal-connect it to a function.
+ * 
+ * @param menutitem Menu-Item to add.
+ * @param menu      Menu to add menu-item to.
+ * @param stockicon Name for icon from stock (can be NULL).
+ * @param name      Visible name of the menu item.
+ * @param func      Function to connect to activate-signal.
  */
 void
-menuitem_TODO(menuitem, ctrls)
-  GtkWidget *menuitem;
-  struct arglist *ctrls;
-{
-  show_error(_("Not yet implemented."));
-}
-
-void
-menuitem_add(menuitem, menu, name, stockicon, func)
-  GtkWidget *menuitem;
-  GtkWidget *menu;
-  const gchar *name;
-  const gchar *stockicon;
-  GtkSignalFunc func;
+menuitem_add (GtkWidget *menuitem, GtkWidget *menu, const gchar *name,
+              const gchar *stockicon, GtkSignalFunc func)
 {
   GtkWidget *icon;
 
@@ -211,9 +221,13 @@ menuitem_add(menuitem, menu, name, stockicon, func)
     g_signal_connect(GTK_OBJECT(menuitem), "activate", func, MainDialog);
 }
 
+/**
+ * @brief Adds a separator to a menu.
+ * 
+ * @param menu Menu to add a separator to.
+ */
 void
-menuitem_separator(menu)
-  GtkWidget *menu;
+menuitem_separator (GtkWidget *menu)
 {
   GtkWidget *menuitem;
 
@@ -223,7 +237,8 @@ menuitem_separator(menu)
 }
 
 
-/* Add a pair of new "stock" icons to the factory.
+/**
+ * @brief Add a pair of new "stock" icons to the factory.
  *
  * The parameters small and large are the arrays of char* that make up
  * an xpm.  The icons are added to the factory as an icon set for the
@@ -234,13 +249,13 @@ menuitem_separator(menu)
  *
  * The parameter small may be NULL to indicate that no special small
  * variant exists.
+ * 
+ * @param large Char array that makes the large size icon (xpm).
+ * @param small Char array that makes the small size icon (xpm).
  */
 static void
-add_nessus_icon_set(factory, stock_id, large, small)
-  GtkIconFactory *factory;
-  const char *stock_id;
-  const char **large;
-  const char **small;
+add_nessus_icon_set (GtkIconFactory *factory, const char *stock_id,
+                     const char **large, const char **small_icon)
 {
   GtkIconSet *iconset;
   GtkIconSource *source;
@@ -260,9 +275,9 @@ add_nessus_icon_set(factory, stock_id, large, small)
   g_object_unref(G_OBJECT(pixbuf));
 
   /* The "small" icon is used for the GTK_ICON_SIZE_MENU size */
-  if (small != NULL)
+  if (small_icon != NULL)
   {
-    pixbuf = gdk_pixbuf_new_from_xpm_data(small);
+    pixbuf = gdk_pixbuf_new_from_xpm_data(small_icon);
     source = gtk_icon_source_new();
     gtk_icon_source_set_pixbuf(source, pixbuf);
     gtk_icon_source_set_size_wildcarded(source, FALSE);
@@ -273,11 +288,12 @@ add_nessus_icon_set(factory, stock_id, large, small)
   }
 
   gtk_icon_factory_add(factory, stock_id, iconset);
-}  
+}
 
-/* Create some nessus specific stock icons.
+/**
+ * @brief Create some openvas specific stock icons.
  *
- * This function adds several nessus specific icon sets for use in e.g.
+ * This function adds several openvas specific icon sets for use in e.g.
  * menus and buttons.
  */
 static void
@@ -285,37 +301,76 @@ create_nessus_icon_factory()
 {
   GtkIconFactory* factory = gtk_icon_factory_new();
 
-  add_nessus_icon_set(factory, "nessus-connect", connect_24, connect_16);
-  add_nessus_icon_set(factory, "nessus-disconnect", disconnect_24,
-      disconnect_16);
-  add_nessus_icon_set(factory, "nessus-general-prefs", general_prefs_xpm, NULL);
-  add_nessus_icon_set(factory, "nessus-plugins", plugins_xpm, NULL);
-  add_nessus_icon_set(factory, "nessus-credentials", credentials_xpm, NULL);
-  add_nessus_icon_set(factory, "nessus-target-selection", target_selection_xpm,
-      NULL);
-  add_nessus_icon_set(factory, "nessus-access-rules", access_rules_xpm, NULL);
-  add_nessus_icon_set(factory, "nessus-prefs", properties_xpm, NULL);
-  add_nessus_icon_set(factory, "nessus-knowledge-base", knowledgebase_xpm,
-      NULL);
-  add_nessus_icon_set(factory, "nessus-error", error_xpm, error_small_xpm);
-  add_nessus_icon_set(factory, "nessus-warning", warning_xpm,
-      warning_small_xpm);
-  add_nessus_icon_set(factory, "nessus-info", info_xpm, info_small_xpm);
+  add_nessus_icon_set(factory, "openvas-connect", connect_24, connect_16);
+  add_nessus_icon_set(factory, "openvas-disconnect", disconnect_24,
+                      disconnect_16);
+  add_nessus_icon_set(factory, "nessus-general-prefs", 
+                      (const char**) general_prefs_xpm, NULL);
+  add_nessus_icon_set(factory, "nessus-plugins",
+                      (const char**) plugins_xpm, NULL);
+  add_nessus_icon_set(factory, "nessus-credentials",
+                      (const char**) credentials_xpm, NULL);
+  add_nessus_icon_set(factory, "nessus-target-selection",
+                      (const char**) target_selection_xpm, NULL);
+  add_nessus_icon_set(factory, "nessus-access-rules",
+                      (const char**) access_rules_xpm, NULL);
+  add_nessus_icon_set(factory, "nessus-prefs", (const char**) properties_xpm,
+                      NULL);
+  add_nessus_icon_set(factory, "nessus-knowledge-base",
+                      (const char**) knowledgebase_xpm, NULL);
+  add_nessus_icon_set(factory, "nessus-error",
+                      (const char**) error_xpm, (const char**) error_small_xpm);
+  add_nessus_icon_set(factory, "nessus-warning", (const char**) warning_xpm,
+                      (const char**) warning_small_xpm);
+  add_nessus_icon_set(factory, "nessus-info", (const char**) info_xpm,
+                      (const char**) info_small_xpm);
 
   gtk_icon_factory_add_default(factory);
   g_object_unref(G_OBJECT(factory));
 }
 
-/*
- * prefs_dialog_setup
+/**
+ * @brief Applies/Undos the global severity_filter to current selected report.
  * 
- * This function draws the preferences dialog of the Nessus
- * client
- *
+ * Swaps the current contexts is_severity_mapped flag and the menu items
+ * label.
+ * 
+ * @param source The GtkMenuImageItem. The text will be altered to indicate
+ *               the new possibility after applying/ undoing the overrides.
+ */
+static void
+apply_severity_filter (GtkWidget* source, gpointer ignored)
+{
+  GtkWidget *menu_label = gtk_bin_get_child(GTK_BIN(source));
+
+  if (Context->type != CONTEXT_REPORT)
+    return;
+
+  // Swap the flag and set the text of the menu- item according to state of
+  // context (a handier way of doing this (item_set_label) in Gtk 2.16 )
+  if (Context->is_severity_mapped == TRUE)
+    {
+      gtk_label_set_text(GTK_LABEL(menu_label), _("Override severities"));
+      Context->is_severity_mapped = FALSE;
+    }
+  else
+    {
+      gtk_label_set_text(GTK_LABEL(menu_label), _("Undo severity override"));
+      Context->is_severity_mapped = TRUE;
+    }
+
+  // Refresh scopeview
+  prefs_scope_tree_update_report ();
+}
+
+/**
+ * @brief Draws the preferences dialog of the OpenVAS client (and other things).
+ * 
+ * Also init and draw the OpenVAS client.
+ * Inits the MainDialog from where GUI elements are referenced.
  */
 void
-prefs_dialog_setup(context)
-  struct context *context;
+prefs_dialog_setup (struct context* context)
 {
   GtkWidget *window;
   GtkWidget *mainmenu;
@@ -387,20 +442,16 @@ prefs_dialog_setup(context)
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), submenu);
 
   menuitem = gtk_image_menu_item_new_with_mnemonic(_("_Connect"));
-  menuitem_add(menuitem, submenu, "CONNECT_MENUITEM", "nessus-connect",
+  menuitem_add(menuitem, submenu, "CONNECT_MENUITEM", "openvas-connect",
       GTK_SIGNAL_FUNC(prefs_dialog_auth_connect));
 
   menuitem = gtk_image_menu_item_new_with_mnemonic(_("_Disconnect"));
-  menuitem_add(menuitem, submenu, "DISCONNECT_MENUITEM", "nessus-disconnect",
+  menuitem_add(menuitem, submenu, "DISCONNECT_MENUITEM", "openvas-disconnect",
       GTK_SIGNAL_FUNC(prefs_dialog_auth_disconnect));
   gtk_widget_set_sensitive(menuitem, FALSE);
 
   menuitem_separator(submenu);
 
-  menuitem = gtk_image_menu_item_new_with_mnemonic(_("SLAD _Install"));
-  menuitem_add(menuitem, submenu, NULL, "slad-install",
-      GTK_SIGNAL_FUNC(slad_install));
-
   menuitem = gtk_image_menu_item_new_with_mnemonic(_("_Scan Assistant"));
   menuitem_add(menuitem, submenu, NULL, "gtk-help",
       GTK_SIGNAL_FUNC(prefs_scan_assistant));
@@ -464,13 +515,6 @@ prefs_dialog_setup(context)
   menuitem_add(menuitem, submenu, "EXECSCOPE_MENUITEM", NULL,
       GTK_SIGNAL_FUNC(prefs_dialog_execute));
 
-#if 0
-  menuitem = gtk_image_menu_item_new_from_stock("gtk-stop", accel_group);
-  menuitem_add(menuitem, submenu, "STOPSCOPE_MENUITEM", NULL,
-      GTK_SIGNAL_FUNC(menuitem_TODO));
-  gtk_widget_set_sensitive(menuitem, FALSE);
-#endif
-
   menuitem_separator(submenu);
 
   menuitem = gtk_image_menu_item_new_with_mnemonic(_("_New"));
@@ -519,6 +563,15 @@ prefs_dialog_setup(context)
 
   menuitem_separator(submenu);
 
+  //menuitem = gtk_check_menu_item_new_with_mnemonic (_("_Apply Severity Filter"));
+  menuitem = gtk_image_menu_item_new_with_label (_("Override severities"));
+  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem),
+                                 gtk_image_new_from_stock (GTK_STOCK_CONVERT, GTK_ICON_SIZE_MENU));
+  menuitem_add (menuitem, submenu, "APPLYOVERRIDESTOREPORT_MENUITEM", NULL,
+                                      GTK_SIGNAL_FUNC(apply_severity_filter));
+
+  menuitem_separator(submenu);
+
   menuitem = gtk_menu_item_new_with_mnemonic(_("_Import"));
   menuitem_add(menuitem, submenu, "IMPORTREPORT_MENUITEM", NULL,
       GTK_SIGNAL_FUNC(open_report_selectfile));
@@ -531,6 +584,23 @@ prefs_dialog_setup(context)
   menuitem_add(menuitem, submenu, "PRINTREPORT_MENUITEM", NULL,
       GTK_SIGNAL_FUNC(report_open_pdf));
 
+  /* Extras menu */
+  menuitem = gtk_menu_item_new_with_mnemonic(_("E_xtras"));
+  menuitem_add(menuitem, mainmenu, NULL, NULL, NULL);
+  submenu = gtk_menu_new();
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), submenu);
+
+  menuitem = gtk_image_menu_item_new_with_mnemonic(_("SLAD _Install Manager"));
+  menuitem_add(menuitem, submenu, NULL, "slad-install",
+      GTK_SIGNAL_FUNC(slad_install));
+
+  menuitem = gtk_menu_item_new_with_mnemonic(_("LSC _Credentials Manager"));
+  menuitem_add(menuitem, submenu, "LSCCREDENTIALSMANAGER_MENUITEM", NULL,
+      GTK_SIGNAL_FUNC(ssh_manager_button_cb));
+
+  menuitem = gtk_menu_item_new_with_mnemonic(_("Severity _Overrides Manager"));
+  menuitem_add(menuitem, submenu, "PRIRORITYFILTERMANAGER_MENUITEM", NULL,
+      GTK_SIGNAL_FUNC(severityfiltermanager_dialog));
 
   /* Help menu */
   menuitem = gtk_menu_item_new_with_mnemonic(_("_Help"));
@@ -607,7 +677,7 @@ prefs_dialog_setup(context)
 
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gtk_separator_tool_item_new(), -1);
 
-  tool_button = gtk_tool_button_new_from_stock("nessus-connect");
+  tool_button = gtk_tool_button_new_from_stock("openvas-connect");
   gtk_tool_button_set_label(GTK_TOOL_BUTTON(tool_button), _("Connect"));
   gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(tool_button),
                             toolbar_tips, _("Connect"), NULL);
@@ -617,7 +687,7 @@ prefs_dialog_setup(context)
   gtk_widget_show(GTK_WIDGET(tool_button));
   arg_add_value(ctrls, "CONNECT_BUTTON", ARG_PTR, -1, tool_button);
 
-  tool_button = gtk_tool_button_new_from_stock("nessus-disconnect");
+  tool_button = gtk_tool_button_new_from_stock("openvas-disconnect");
   gtk_tool_button_set_label(GTK_TOOL_BUTTON(tool_button), _("Disconnect"));
   gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(tool_button),
                             toolbar_tips, _("Disconnect"), NULL);
@@ -738,9 +808,10 @@ prefs_dialog_setup(context)
   gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview)),
       _("Welcome to OpenVAS-Client, http://www.openvas.org/"
 	"\nNessusClient origin: Copyright 1998-2007 by Renaud Deraison"
-	"\nNew code since OpenVAS-Client: Copyright 2007 Intevation GmbH"
+	"\nNew code since OpenVAS-Client: Copyright 2007, 2008, 2009 Intevation GmbH"
 	"\nAuthors: Renaud Deraison, Thomas Arendsen Hein, Jan-Oliver Wagner"
 	", Bernhard Herzog, Michel Arboi (SSL-Support), Bruce Verderaime (Pie/Charts)"
+        ", Michael Wiegand, Felix Wolfsteller"
 	"\n"),
       -1);
   arg_add_value(ctrls, "MSGLOGTEXT", ARG_PTR, -1, textview);
@@ -798,11 +869,32 @@ prefs_dialog_setup(context)
   gtk_widget_show(window);
 }
 
+/**
+ * @brief Sets a ARG_STRING value "yes" or "no" in a preference arglist, if the same
+ * @brief key existed as ARG_INT before.
+ *
+ * @param preferences    Preferences arglist.
+ * @param prefrence_name Key to look for and eventually set to "yes" or no.
+ */
+static void
+arg_int_to_yesno(struct arglist* preferences, char* preference_name)
+{
+  char* value = arg_get_value (preferences, preference_name);
+  if (arg_get_type (preferences, preference_name) == ARG_INT)
+  {
+    char *yesno = (value) ? estrdup("yes") : estrdup("no");
 
+    /* RATS: ignore yesno is nul-terminated because of strdup of string literal*/
+    arg_set_value (preferences, preference_name, strlen(yesno), yesno);
+    arg_set_type  (preferences, preference_name, ARG_STRING);
+  }
+}
+
+/**
+ * @brief Sets default values to the GTK Widgets hooked in ctrls.
+ */
 void
-prefs_dialog_set_defaults(context, ctrls)
-  struct context *context;
-  struct arglist *ctrls;
+prefs_dialog_set_defaults (struct context *context, struct arglist *ctrls)
 {
 #define EX_NONE 1
 #define EX_DNS 2
@@ -830,7 +922,7 @@ prefs_dialog_set_defaults(context, ctrls)
       char *s = strdup(v ? "yes" : "no");
 
       arg_set_type(serv, "auto_enable_dependencies", ARG_STRING);
-      arg_set_value(serv, "auto_enable_dependencies", strlen(s), s);
+      arg_set_value(serv, "auto_enable_dependencies", strlen(s), s); /* RATS: ignore s is yes or no and nul terminated */
       v = s;
     }
     if(v && !strcmp(v, "yes"))
@@ -847,7 +939,7 @@ prefs_dialog_set_defaults(context, ctrls)
       char *s = strdup(v ? "yes" : "no");
 
       arg_set_type(serv, "silent_dependencies", ARG_STRING);
-      arg_set_value(serv, "silent_dependencies", strlen(s), s);
+      arg_set_value(serv, "silent_dependencies", strlen(s), s); /* RATS: ignore s is yes or no and nul terminated */
       v = s;
     }
     if(v && !strcmp(v, "yes"))
@@ -855,6 +947,16 @@ prefs_dialog_set_defaults(context, ctrls)
     else
       GTK_TOGGLE_BUTTON(gtkw)->active = FALSE;
   }
+  gtkw = arg_get_value(t, "AUTO_ENABLE_NEW_PLUGINS");
+  if(gtkw)
+  {
+    int iv = prefs_get_int(context, "auto_enable_new_plugins");
+    if(iv)
+      GTK_TOGGLE_BUTTON(gtkw)->active = TRUE;
+    else
+      GTK_TOGGLE_BUTTON(gtkw)->active = FALSE;
+  }
+
   /*
    *  Host expansion options
    */
@@ -889,56 +991,20 @@ prefs_dialog_set_defaults(context, ctrls)
    */
   t = arg_get_value(ctrls, "SCAN_OPTIONS");
   gtkw = arg_get_value(t, "PING_HOSTS");
-  v = arg_get_value(serv, "ping_hosts");
-  if(arg_get_type(serv, "ping_hosts") == ARG_INT)
-  {
-    char *s = emalloc(4);
-
-    if(v)
-      strncpy(s, "yes", 3);
-    else
-      strncpy(s, "no", 2);
-
-    arg_set_value(serv, "ping_hosts", strlen(s), s);
-    arg_set_type(serv, "ping_hosts", ARG_STRING);
-  }
+  arg_int_to_yesno(serv, "ping_hosts");
   v = arg_get_value(serv, "ping_hosts");
   if(v)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtkw), !strcmp(v, "yes"));
 
   gtkw = arg_get_value(t, "REVERSE_LOOKUP");
-  v = arg_get_value(serv, "reverse_lookup");
-  if(arg_get_type(serv, "reverse_lookup") == ARG_INT)
-  {
-    char *s = emalloc(4);
-
-    if(v)
-      strncpy(s, "yes", 3);
-    else
-      strncpy(s, "no", 2);
-
-    arg_set_value(serv, "reverse_lookup", strlen(s), s);
-    arg_set_type(serv, "reverse_lookup", ARG_STRING);
-  }
+  arg_int_to_yesno(serv, "reverse_lookup");
   v = arg_get_value(serv, "reverse_lookup");
   if(v)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtkw), !strcmp(v, "yes"));
 
 
   gtkw = arg_get_value(t, "OPTIMIZE_TEST");
-  v = arg_get_value(serv, "optimize_test");
-  if(arg_get_type(serv, "optimize_test") == ARG_INT)
-  {
-    char *s = emalloc(4);
-
-    if(v)
-      strncpy(s, "yes", 3);
-    else
-      strncpy(s, "no", 2);
-
-    arg_set_value(serv, "optimize_test", strlen(s), s);
-    arg_set_type(serv, "optimize_test", ARG_STRING);
-  }
+  arg_int_to_yesno(serv, "optimize_test");
   v = arg_get_value(serv, "optimize_test");
   if(v)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtkw), !strcmp(v, "yes"));
@@ -947,20 +1013,7 @@ prefs_dialog_set_defaults(context, ctrls)
 
 
   gtkw = arg_get_value(t, "SAFE_CHECKS");
-  v = arg_get_value(serv, "safe_checks");
-  if(arg_get_type(serv, "safe_checks") == ARG_INT)
-  {
-    char *s = emalloc(4);
-
-    if(v)
-      strncpy(s, "yes", 3);
-    else
-      strncpy(s, "no", 2);
-
-    arg_set_value(serv, "safe_checks", strlen(s), s);
-    arg_set_type(serv, "safe_checks", ARG_STRING);
-  }
-
+  arg_int_to_yesno(serv, "safe_checks");
   v = arg_get_value(serv, "safe_checks");
   if(v)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtkw), !strcmp(v, "yes"));
@@ -969,20 +1022,7 @@ prefs_dialog_set_defaults(context, ctrls)
 
 
   gtkw = arg_get_value(t, "USE_MAC_ADDR");
-  v = arg_get_value(serv, "use_mac_addr");
-  if(arg_get_type(serv, "use_mac_addr") == ARG_INT)
-  {
-    char *s = emalloc(4);
-
-    if(v)
-      strncpy(s, "yes", 3);
-    else
-      strncpy(s, "no", 2);
-
-    arg_set_value(serv, "use_mac_addr", strlen(s), s);
-    arg_set_type(serv, "use_mac_addr", ARG_STRING);
-  }
-
+  arg_int_to_yesno(serv, "use_mac_addr");
   v = arg_get_value(serv, "use_mac_addr");
   if(v)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtkw), !strcmp(v, "yes"));
@@ -1005,7 +1045,7 @@ prefs_dialog_set_defaults(context, ctrls)
     char *s;
     int v;
 
-    v = (int)arg_get_value(serv, "unscanned_closed");
+    v = GPOINTER_TO_SIZE(arg_get_value(serv, "unscanned_closed"));
     if(v)
       s = strdup("yes");
     else
@@ -1028,7 +1068,7 @@ prefs_dialog_set_defaults(context, ctrls)
   {
     v = arg_get_value(serv, "max_hosts");
     if(v)
-      gtk_entry_set_text(GTK_ENTRY(gtkw), v);
+      gtk_spin_button_set_value (GTK_SPIN_BUTTON(gtkw), atof(v));
   }
 
   gtkw = arg_get_value(t, "MAX_CHECKS");
@@ -1036,7 +1076,7 @@ prefs_dialog_set_defaults(context, ctrls)
   {
     v = arg_get_value(serv, "max_checks");
     if(v)
-      gtk_entry_set_text(GTK_ENTRY(gtkw), v);
+      gtk_spin_button_set_value (GTK_SPIN_BUTTON(gtkw), atof(v));
   }
 
   gtkw = arg_get_value(t, "CGI_PATH");
@@ -1057,10 +1097,13 @@ prefs_dialog_set_defaults(context, ctrls)
 #endif
 }
 
-
+/**
+ * @brief Sets tooltips for various GTK widgets hooked up in ctrls.
+ *
+ * @param ctrls Arglist with GTK widget hooked into.
+ */
 static void
-prefs_dialog_set_tooltips(ctrls)
-  struct arglist *ctrls;
+prefs_dialog_set_tooltips (struct arglist *ctrls)
 {
   struct arglist *t;
   GtkTooltips *tooltips;
@@ -1076,6 +1119,9 @@ prefs_dialog_set_tooltips(ctrls)
   w = arg_get_value(t, "SILENT_DEPS");
   gtk_tooltips_set_tip(tooltips, w, HLP_SILENT_DEPS, "");
 
+  w = arg_get_value(t, "AUTO_ENABLE_NEW_PLUGINS");
+  gtk_tooltips_set_tip(tooltips, w, HLP_AUTO_ENABLE_NEW_PLUGINS, "");
+
   t = arg_get_value(ctrls, "TARGET");
   w = arg_get_value(t, "DNS_EXPAND");
   gtk_tooltips_set_tip(tooltips, w, HLP_HOST_EXPANSION_DNS, "");
@@ -1112,15 +1158,49 @@ prefs_dialog_set_tooltips(ctrls)
   gtk_tooltips_set_tip(tooltips, w, HLP_MISC_MAX_CHECKS, "");
   w = arg_get_value(t, "CGI_PATH");
   gtk_tooltips_set_tip(tooltips, w, HLP_CGI_PATH, "");
+  // FIXME suspect tooltip for following item is missing:
   t = arg_get_value(ctrls, "USER");
   gtk_tooltips_enable(tooltips);
 }
 
+/**
+ * @brief Sets a string in a preference arglist as either "yes" or "no",
+ * @brief depending on the state (active -> "yes") of a togglebutton.
+ * 
+ * @param preferences  Servers preference arglist.
+ * @param togglebutton A gtk_toggle_button, of which the state will decide the
+ *                     preference value (active -> "yes").
+ * @param preference_name Key for the value in the preference arglist.
+ */
+static void
+set_yesno_pref_from_toggle (struct arglist* preferences, GtkWidget* togglebutton,
+                            char* preference_name)
+{
+ if(togglebutton)
+  {
+    char *value;
+  
+    if(GTK_TOGGLE_BUTTON(togglebutton)->active)
+      value = estrdup("yes");
+    else
+      value = estrdup("no");
+  
+    if(arg_get_value(preferences, preference_name))
+      arg_set_value(preferences, preference_name, strlen(value), value); /* RATS: ignore */
+    else
+      arg_add_value(preferences, preference_name, ARG_STRING, 
+                    strlen(value), value); /* RATS: ignore */
+  }
+}
 
+/**
+ * @brief Applies user choices to a context (and its preferences etc.).
+ * 
+ * @param context The context to manipulate
+ * @param ctrls   Arglist with all the GTK widgets hooked in.
+ */
 void
-prefs_dialog_apply(context, ctrls)
-  struct context *context;
-  struct arglist *ctrls;
+prefs_dialog_apply (struct context *context, struct arglist *ctrls)
 {
   struct arglist *prefs;
   struct arglist *serv;
@@ -1153,41 +1233,22 @@ prefs_dialog_apply(context, ctrls)
 
 
   t = arg_get_value(ctrls, "PLUGINS");
+  
   gtkw = arg_get_value(t, "ENABLE_DEPS_AT_RUNTIME");
-  if(gtkw)
-  {
-    char *s;
-
-    if(GTK_TOGGLE_BUTTON(gtkw)->active)
-      s = estrdup("yes");
-    else
-      s = estrdup("no");
-
-    if(arg_get_value(serv, "auto_enable_dependencies"))
-      arg_set_value(serv, "auto_enable_dependencies", strlen(s), s);
-    else
-      arg_add_value(serv, "auto_enable_dependencies", ARG_STRING, strlen(s),
-	  s);
-  }
+  set_yesno_pref_from_toggle(serv, gtkw, "auto_enable_dependencies");
+  
   gtkw = arg_get_value(t, "SILENT_DEPS");
+  set_yesno_pref_from_toggle(serv, gtkw, "silent_dependencies");
+  
+  gtkw = arg_get_value(t, "AUTO_ENABLE_NEW_PLUGINS");
   if(gtkw)
   {
-    char *s;
-
     if(GTK_TOGGLE_BUTTON(gtkw)->active)
-      s = estrdup("yes");
+      prefs_set_int(context,"auto_enable_new_plugins",1);
     else
-      s = estrdup("no");
-
-    if(arg_get_value(serv, "silent_dependencies"))
-      arg_set_value(serv, "silent_dependencies", strlen(s), s);
-    else
-      arg_add_value(serv, "silent_dependencies", ARG_STRING, strlen(s), s);
+      prefs_set_int(context,"auto_enable_new_plugins",0);
   }
 
-
-
-
   /*
    * Host expansion
    */
@@ -1214,83 +1275,31 @@ prefs_dialog_apply(context, ctrls)
     arg_add_value(serv, "host_expansion", ARG_STRING, strlen(s), s);
 
 
-
-
   /*
    * Scan options
    */
-
   t = arg_get_value(ctrls, "SCAN_OPTIONS");
   gtkw = arg_get_value(t, "PING_HOSTS");
-  s = emalloc(4);
-
-  if(GTK_TOGGLE_BUTTON(gtkw)->active)
-    strncpy(s, "yes", 3);
-  else
-    strncpy(s, "no", 4);
-
-  if(arg_get_value(serv, "ping_hosts"))
-    arg_set_value(serv, "ping_hosts", strlen(s), s);
-  else
-    arg_add_value(serv, "ping_hosts", ARG_STRING, strlen(s), s);
+  set_yesno_pref_from_toggle(serv, gtkw, "ping_hosts");
 
 
   gtkw = arg_get_value(t, "REVERSE_LOOKUP");
-  s = emalloc(4);
-  if(GTK_TOGGLE_BUTTON(gtkw)->active)
-    strncpy(s, "yes", 3);
-  else
-    strncpy(s, "no", 4);
-
-  if(arg_get_value(serv, "reverse_lookup"))
-    arg_set_value(serv, "reverse_lookup", strlen(s), s);
-  else
-    arg_add_value(serv, "reverse_lookup", ARG_STRING, strlen(s), s);
+  set_yesno_pref_from_toggle(serv, gtkw, "reverse_lookup");
 
   gtkw = arg_get_value(t, "OPTIMIZE_TEST");
-  s = emalloc(4);
-  if(GTK_TOGGLE_BUTTON(gtkw)->active)
-    strncpy(s, "yes", 3);
-  else
-    strncpy(s, "no", 4);
-
-  if(arg_get_value(serv, "optimize_test"))
-    arg_set_value(serv, "optimize_test", strlen(s), s);
-  else
-    arg_add_value(serv, "optimize_test", ARG_STRING, strlen(s), s);
-
+  set_yesno_pref_from_toggle(serv, gtkw, "optimize_test");
 
   gtkw = arg_get_value(t, "SAFE_CHECKS");
-  s = emalloc(4);
-  if(GTK_TOGGLE_BUTTON(gtkw)->active)
-    strncpy(s, "yes", 3);
-  else
-    strncpy(s, "no", 4);
+  set_yesno_pref_from_toggle(serv, gtkw, "safe_checks");
 
-  if(arg_get_value(serv, "safe_checks"))
-    arg_set_value(serv, "safe_checks", strlen(s), s);
-  else
-    arg_add_value(serv, "safe_checks", ARG_STRING, strlen(s), s);
 
 
   gtkw = arg_get_value(t, "USE_MAC_ADDR");
-  s = emalloc(4);
-  if(GTK_TOGGLE_BUTTON(gtkw)->active)
-    strncpy(s, "yes", 3);
-  else
-    strncpy(s, "no", 4);
-
-  if(arg_get_value(serv, "use_mac_addr"))
-    arg_set_value(serv, "use_mac_addr", strlen(s), s);
-  else
-    arg_add_value(serv, "use_mac_addr", ARG_STRING, strlen(s), s);
-
+  set_yesno_pref_from_toggle(serv, gtkw, "use_mac_addr");
 
   t = arg_get_value(ctrls, "SCAN_OPTIONS");
   gtkw = arg_get_value(t, "PORT_RANGE");
-  s = emalloc(strlen(gtk_entry_get_text(GTK_ENTRY(gtkw))) + 1);
-  strncpy(s, gtk_entry_get_text(GTK_ENTRY(gtkw)),
-      strlen(gtk_entry_get_text(GTK_ENTRY(gtkw))));
+  s = estrdup (gtk_entry_get_text(GTK_ENTRY(gtkw)));
 
   if(arg_get_value(serv, "port_range"))
     arg_set_value(serv, "port_range", strlen(s), s);
@@ -1299,24 +1308,12 @@ prefs_dialog_apply(context, ctrls)
 
 
   gtkw = arg_get_value(t, "UNSCANNED_CLOSED");
-  s = emalloc(4);
-  if(GTK_TOGGLE_BUTTON(gtkw)->active)
-    strncpy(s, "yes", 3);
-  else
-    strncpy(s, "no", 4);
-
-  if(arg_get_value(serv, "unscanned_closed"))
-    arg_set_value(serv, "unscanned_closed", strlen(s), s);
-  else
-    arg_add_value(serv, "unscanned_closed", ARG_STRING, strlen(s), s);
+  set_yesno_pref_from_toggle(serv, gtkw, "unscanned_closed");
 
 
   gtkw = arg_get_value(t, "MAX_HOSTS");
 
-  s = emalloc(strlen(gtk_entry_get_text(GTK_ENTRY(gtkw))) + 1);
-
-  strncpy(s, gtk_entry_get_text(GTK_ENTRY(gtkw)),
-      strlen(gtk_entry_get_text(GTK_ENTRY(gtkw))));
+  s = estrdup (gtk_entry_get_text(GTK_ENTRY(gtkw)));
 
   if(arg_get_value(serv, "max_hosts"))
     arg_set_value(serv, "max_hosts", strlen(s), s);
@@ -1326,10 +1323,7 @@ prefs_dialog_apply(context, ctrls)
 
   gtkw = arg_get_value(t, "MAX_CHECKS");
 
-  s = emalloc(strlen(gtk_entry_get_text(GTK_ENTRY(gtkw))) + 1);
-
-  strncpy(s, gtk_entry_get_text(GTK_ENTRY(gtkw)),
-      strlen(gtk_entry_get_text(GTK_ENTRY(gtkw))));
+  s = g_strdup (gtk_entry_get_text(GTK_ENTRY(gtkw)));
 
   if(arg_get_value(serv, "max_checks"))
     arg_set_value(serv, "max_checks", strlen(s), s);
@@ -1397,20 +1391,68 @@ prefs_dialog_apply(context, ctrls)
   }
 }
 
+/**
+ * @brief Creates a string from plugin preference radiobuttons.
+ * 
+ * @param list Value of arg_get_value(pref->value, "RADIOBUTTONS)
+ * @param pref The plugin preference
+ * 
+ * @returns Plugin preference string for a radiobutton selection.
+ */
+static char*
+get_plugin_radio_pref(GSList* list, struct arglist* pref)
+{
+  char* value = NULL;
+  /* First pass: find the active entry */
+  while (list)
+  {
+    GtkWidget *button = list->data;
 
-/* XXX Warning:
+    if (GTK_TOGGLE_BUTTON(button)->active)
+    {
+      value = estrdup(gtk_object_get_data(GTK_OBJECT(button), "name"));
+      break;
+    }
+    list = list->next;
+  }
+  /* Second pass: find all other entries and append them separated with ";" */
+  list = arg_get_value(pref->value, "RADIOBUTTONS");
+  while (list)
+  {
+    GtkWidget *button = list->data;
+
+    if (! GTK_TOGGLE_BUTTON(button)->active)
+    {
+      char * v = gtk_object_get_data(GTK_OBJECT(button), "name");
+      value = (char *) erealloc((void *)value, (size_t) (strlen(value) + strlen(";") + strlen(v) + 1));
+      strcat(value, ";");
+      strcat(value, v);
+    }
+    list = list->next;
+  }
+  return value;
+}
+
+
+/**
+ * @brief Applies the input from the plugins preferences widgets to the arglists.
+ * 
+ * Is currently called when an attack is started and when the user deselects
+ * a scope (e.g. by selecting a different one).
+ * XXX Warning:
  * saves to context->plugins and context->scanners,
  * NOT to context->prefs["PLUGINS_PREFS"]
+ * 
+ * @param plugins List of plugins whose preferences to modify.
  */
 void
-prefs_dialog_apply_plugin_prefs(plugins)
-  struct nessus_plugin *plugins;
+prefs_dialog_apply_plugin_prefs (struct nessus_plugin *plugins)
 {
   while(plugins!= NULL )
   {
     struct arglist *pref;
-    char *type;
-    char *value;
+    char *type  = NULL;
+    char *value = NULL;
 
     if((pref = plugins->plugin_prefs) != NULL )
       while(pref && pref->next)
@@ -1437,51 +1479,29 @@ prefs_dialog_apply_plugin_prefs(plugins)
 	    {
 	      char *value =
 		  GTK_TOGGLE_BUTTON(button)->active ? "yes" : "no";
-	      arg_set_value(pref->value, "value", strlen(value),
-		  estrdup(value));
+	      arg_set_value(pref->value, "value", strlen(value), estrdup(value)); /* RATS: ignore , value is yes or no and nul- terminated */
 	    }
 	  }
 	  else if(!strcmp(type, PREF_RADIO))
 	  {
-        GSList *list = NULL;
-        char * value = NULL;
+            GSList *list = NULL;
+            char * value = NULL;
 
-        list = arg_get_value(pref->value, "RADIOBUTTONS");
+            list = arg_get_value(pref->value, "RADIOBUTTONS");
 
-        if (list)
-        {
-          /* First pass: find the active entry */
-          while (list)
-          {
-            GtkWidget *button = list->data;
-
-            if (GTK_TOGGLE_BUTTON(button)->active)
+            if (list)
             {
-              value = estrdup(gtk_object_get_data(GTK_OBJECT(button), "name"));
-              break;
+              value = get_plugin_radio_pref(list, pref);
             }
-            list = list->next;
-          }
-          /* Second pass: find all other entries and append
-           * them separated with ";" */
-          list = arg_get_value(pref->value, "RADIOBUTTONS");
-          while (list)
-          {
-            GtkWidget *button = list->data;
-
-            if (! GTK_TOGGLE_BUTTON(button)->active)
-            {
-              char * v = gtk_object_get_data(GTK_OBJECT(button), "name");
-              value = (char *) erealloc((void *)value, (size_t) (strlen(value) + strlen(";") + strlen(v) + 1));
-              strcat(value, ";");
-              strcat(value, v);
-            }
-            list = list->next;
-          }
-        }
-        if(value)
-          arg_set_value(pref->value, "value", strlen(value), value);
+            if(value)
+              arg_set_value(pref->value, "value", strlen(value), value);
 	  }
+          else if (!strcmp(type, PREF_SSH_CREDENTIALS))
+          {
+            // Workaround to trigger display of this NVT (otherwise invisible 
+            // when not connected)
+            arg_set_value(pref->value, "value", strlen("ignored"), "ignored"); /* RATS: ignore string literals are nul terminated */
+          }
 	}
 	pref = pref->next;
       }
@@ -1490,10 +1510,15 @@ prefs_dialog_apply_plugin_prefs(plugins)
 }
 
 
+/**
+ * @brief To execute, first all preferences (user choices) are applied.
+ * 
+ * Function acts as a callback for menu items and button.
+ * 
+ * @return Always 0.
+ */
 int
-prefs_dialog_execute(w, ctrls)
-  GtkWidget *w;
-  struct arglist *ctrls;
+prefs_dialog_execute (GtkWidget* w, struct arglist* ctrls)
 {
   struct context *context = context_by_type(Context, CONTEXT_SCOPE);
   char *hostname;
