@@ -35,7 +35,6 @@
 #include <gtk/gtk.h>
 #include "../nessus_plugin.h"
 #include "../plugin_infos.h"
-#include "../families.h"
 #include "error_dlg.h"
 #include "../xpm/warning_small.xpm"
 #include "filter.h"
@@ -45,37 +44,40 @@
 
 
 
-/* The columns of the tree store we use */
+/** The columns of the tree store we use. */
 enum {
-  /* The name of the family/plugin */
+  /** The name of the family/plugin. */
   COL_NAME,
 
-  /* struct arglist * of the plugin. For a family the value is NULL.
-   */
+  /** struct arglist * of the plugin. For a family the value is NULL. */
   COL_PLUGIN,
 
-  /* Number of columns */
+  /** Number of columns. */
   NUM_COLS
 };
 
 
-/* Define a new signal, "statistics-changed", to be emitted when the
+/**
+ * Define a new signal, "statistics-changed", to be emitted when the
  * plugin tree changes in a way that may have changed the statistics
  * (number of plugins, number of enabled plugins, number of filtered
  * plugins) tree view.
  *
- * the signal handlers have the following signature:
+ * The signal handlers have the following signature:
  *
- * void statistics_changed_handler(GtkWidget* tree_view, gpointer user_data)
+ * void statistics_changed_handler (GtkWidget* tree_view, gpointer user_data)
  */
 
-/* the id of the signal */
+/** ID of the "statistics-changed"- signal. */
 static gint statistics_changed_signal = 0;
 
-/* register the "statistics-changed" signal with glib.  This function
- * must be called before the signal is used for the first time in any
- * way.  It may be called multiple times as it only registers the signal
- * on the first call.
+/**
+ * @brief Register the "statistics-changed" signal with glib.
+ * 
+ * This function must be called before the signal is used for the first time in
+ * any way.
+ * It may be called multiple times as it only registers the signal on the first
+ * call.
  */
 static void
 create_statistics_changed_signal()
@@ -88,17 +90,22 @@ create_statistics_changed_signal()
   }
 }
 
-/* Emit the "statistics-changed" signal with the tree_view as the
- * instance */
+/**
+ * @brief Emit the "statistics-changed" signal with the tree_view as the instance.
+ * 
+ * @param tree_view The signal- emitting treeview.
+ */
 static void
-emit_statistics_changed(tree_view)
-  GtkWidget *tree_view;
+emit_statistics_changed (GtkWidget* tree_view)
 {
   g_signal_emit(tree_view, statistics_changed_signal, 0);
-}  
+}
 
-/* Handler for the tree model's "row-changed" signal.  Emit the
- * "statistics-changed" signal on the tree model */
+/**
+ * @brief Handler for the tree model's "row-changed" signal.
+ * 
+ * Emit the "statistics-changed" signal on the tree model.
+ */
 static void
 emit_statistics_when_row_changed(treemodel, arg1, arg2, user_data)
   GtkTreeModel *treemodel;
@@ -109,8 +116,11 @@ emit_statistics_when_row_changed(treemodel, arg1, arg2, user_data)
   emit_statistics_changed(GTK_WIDGET(user_data));
 }
 
-/* Handler for the tree model's "row-deleted" signal.  Emit the
- * "statistics-changed" signal on the tree model */
+/**
+ * @brief Handler for the tree model's "row-deleted" signal.
+ * 
+ * Emit the "statistics-changed" signal on the tree model.
+ */
 static void
 emit_statistics_when_row_deleted(treemodel, arg1, user_data)
   GtkTreeModel *treemodel;
@@ -121,7 +131,8 @@ emit_statistics_when_row_deleted(treemodel, arg1, user_data)
 }
 
 
-/* Return whether the family given by the model and iter is enabled.
+/**
+ * @brief Return whether the family given by the model and iter is enabled.
  *
  * The iter argument must be the iter pointing to the family row.
  * A family is enabled when at least one of it's plugins is enabled as
@@ -149,7 +160,8 @@ is_family_enabled(model, iter)
 }
 
 
-/* Determine whether a warning should be shown for the given row
+/**
+ * @brief Determine whether a warning should be shown for the given row.
  *
  * Some plugins are dangerous and a warning sign is shown for the them
  * in the tree view.  This function determines whether a warning is
@@ -164,7 +176,6 @@ is_family_enabled(model, iter)
  * "denial", "kill_host", "flood" or "destructive_attack" and false
  * otherwise.
  */
-
 static int
 should_show_warning(model, path, iter)
   GtkTreeModel *model;
@@ -196,7 +207,8 @@ should_show_warning(model, path, iter)
 }
 
 
-/* Trigger a row update in any view of the given model
+/**
+ * @brief Trigger a row update in any view of the given model.
  *
  * The iter should be an iter for the row in question.  This function is
  * needed because some information shown in the plugin tree is not
@@ -205,9 +217,7 @@ should_show_warning(model, path, iter)
  * model doesn't send any signals.
  */
 static void
-trigger_row_update(model, iter)
-  GtkTreeModel *model;
-  GtkTreeIter *iter;
+trigger_row_update (GtkTreeModel *model, GtkTreeIter *iter)
 {
   GtkTreePath *path = gtk_tree_model_get_path(model, iter);
 
@@ -218,7 +228,8 @@ trigger_row_update(model, iter)
 
 
 
-/* Toggle the active/launch flag of a plugin
+/**
+ * @brief Toggle the active/launch flag of a plugin.
  *
  * This function is called when the checkbox in the tree view widget has
  * been toggled by the user.  There are two cases: The row on which it
@@ -237,10 +248,7 @@ trigger_row_update(model, iter)
  * parent family.
  */
 static void
-active_toggled(cell, path_str, data)
-  GtkCellRendererToggle *cell;
-  gchar                 *path_str;
-  gpointer               data;
+active_toggled (GtkCellRendererToggle *cell, gchar *path_str, gpointer data)
 {
   GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(data));
   GtkTreeIter iter;
@@ -283,8 +291,10 @@ active_toggled(cell, path_str, data)
   }
 }
 
-/* Called when the user double clicks on a row of the tree view.  Pop up
- * the plugin info dialog if the row is a plugin, not a family.
+/**
+ * @brief Called when the user double clicks on a row of the tree view.
+ * 
+ * Pop up the plugin info dialog if the row is a plugin, not a family.
  */
 static void
 row_activated(treeview, path, column, user_data)
@@ -308,7 +318,8 @@ row_activated(treeview, path, column, user_data)
 }
 
 
-/* Make the check boxes and warning "buttons" work on the first mouse click
+/**
+ * @brief Make the check boxes and warning "buttons" work on the first mouse click.
  *
  * Normally, the tree view only selects a line on the first click with
  * the mouse button.  Only clicks on a selected row will toggle a check
@@ -325,10 +336,8 @@ row_activated(treeview, path, column, user_data)
  * all other cases.
  */
 static gboolean
-button_press_event(tree, event, user_data)
-  GtkTreeView *tree;
-  GdkEventButton *event;
-  gpointer user_data;
+button_press_event (GtkTreeView *tree, GdkEventButton *event,
+                    gpointer user_data)
 {
   GtkTreePath * path;
   GtkTreeViewColumn *column;
@@ -360,15 +369,12 @@ button_press_event(tree, event, user_data)
 }
 
 
-/* Set the "active" property of the toggle cell renderer
+/**
+ * @brief Set the "active" property of the toggle cell renderer.
  */
 static void
-active_data_func(tree_column, cell, model, iter, data)
-  GtkTreeViewColumn *tree_column;
-  GtkCellRenderer *cell;
-  GtkTreeModel *model;
-  GtkTreeIter *iter;
-  gpointer data;
+active_data_func (GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
+                  GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
 {
   struct nessus_plugin *plugin;
   gboolean is_active = FALSE;
@@ -393,7 +399,8 @@ active_data_func(tree_column, cell, model, iter, data)
 }
 
 
-/* Set the "pixbuf" property of the warning cell renderer
+/**
+ * @brief Set the "pixbuf" property of the warning cell renderer.
  */
 static void
 warning_data_func(tree_column, cell, model, iter, data)
@@ -419,18 +426,18 @@ warning_data_func(tree_column, cell, model, iter, data)
 
 
 
-/* Create a new plugin tree and model */
+/**
+ * @brief Create a new plugin tree and model.
+ */
 GtkWidget *
-prefs_create_plugins_tree(context, ctrls)
-  struct context *context;
-  struct arglist *ctrls;
+prefs_create_plugins_tree (struct context *context, struct arglist* ctrls)
 {
   GtkWidget *tree;
   GtkTreeSelection *selection;
   GtkTreeViewColumn *column;
   GtkCellRenderer *renderer;
 
-  /* make sure the "statistics-changed" signal exists */
+  /* Make sure the "statistics-changed" signal exists */
   create_statistics_changed_signal();
 
   /* Create the tree and its columns*/
@@ -474,13 +481,12 @@ prefs_create_plugins_tree(context, ctrls)
 }
 
 
-/* Fill the context's tree model with the plugins, applying the filter
- * from ctrls if one is set.
+/**
+ * @brief Fill the context's tree model with the plugins, applying the filter
+ *        from ctrls if one is set.
  */
 static void
-fill_plugin_tree_store(context, ctrls)
-  struct context *context;
-  struct arglist *ctrls;
+fill_plugin_tree_store (struct context *context,   struct arglist *ctrls)
 {
   struct nessus_plugin *plugs = context->plugins;
   GtkTreeModel *store = GTK_TREE_MODEL(context->plugin_tree_store);
@@ -488,7 +494,7 @@ fill_plugin_tree_store(context, ctrls)
   GHashTable* family_hash = g_hash_table_new_full(g_str_hash, g_str_equal,
       NULL, (GDestroyNotify)gtk_tree_path_free);
   
-  /* add the plugins and families.  The family nodes are created when
+  /* Add the plugins and families.  The family nodes are created when
    * the first plugin of that family is encountered */
   while(plugs != NULL )
   {
@@ -515,6 +521,7 @@ fill_plugin_tree_store(context, ctrls)
       g_hash_table_insert(family_hash, plug_family, family_path);
       gtk_tree_store_set(GTK_TREE_STORE(store), &family_iter,
 	  COL_NAME, plug_family, COL_PLUGIN, NULL, -1);
+      
     }
     else
     {
@@ -534,7 +541,10 @@ fill_plugin_tree_store(context, ctrls)
 }
 
 
-/* Update the plugin tree with a different context.  The plugin-tree
+/**
+ * @brief Update the plugin tree with a different context.
+ * 
+ * The plugin-tree
  * maintains a tree store and a sorted model in the context.  Make the
  * tree store of the new context the model shown in the plugin tree view
  * (taken from ctrls).  If the store doesn't exist yet or if the
@@ -544,21 +554,20 @@ fill_plugin_tree_store(context, ctrls)
  * statistics_changed signal.
  */
 void
-prefs_plugin_tree_context_changed(context, ctrls, force_rebuild)
-  struct context *context;
-  struct arglist *ctrls;
-  gboolean force_rebuild;
+prefs_plugin_tree_context_changed (struct context *context,
+                                   struct arglist *ctrls,
+                                   gboolean force_rebuild)
 {
   GtkTreeView *tree_view = arg_get_value(ctrls, "PLUGINS_TREE_VIEW");
   GtkTreeModel *model = context->plugin_tree_model;
   GtkTreeModel *old_model = gtk_tree_view_get_model(tree_view);
 
-  /* if the caller wants, force a rebuild of the store by deleting the
+  /* If the caller wants, force a rebuild of the store by deleting the
    * old one */
   if (force_rebuild)
     context_reset_plugin_tree(context);
 
-  /* if the context doesn't have a tree store yet, create it together
+  /* If the context doesn't have a tree store yet, create it together
    * with the sort model and fill it */
   if (context->plugin_tree_store == NULL)
   {
@@ -597,16 +606,15 @@ prefs_plugin_tree_context_changed(context, ctrls, force_rebuild)
   }
 
   /* When a new model is set, the statistics will have changed: */
-  emit_statistics_changed(tree_view);
+  emit_statistics_changed ( GTK_WIDGET(tree_view));
 }
 
 
-/* enable/disable all plugins in the tree.
+/**
+ * @brief Enable/Disable all plugins in the tree.
  */
 void
-prefs_plugin_tree_enable_all(tree, enable)
-  GtkTreeView *tree;
-  int enable;
+prefs_plugin_tree_enable_all (GtkTreeView *tree, int enable)
 {
   GtkTreeModel *model = gtk_tree_view_get_model(tree);
   GtkTreeIter iter, child_iter;
@@ -633,7 +641,8 @@ prefs_plugin_tree_enable_all(tree, enable)
   }	
 }
 
-/* Compute some statistics for the plugin tree
+/**
+ * @brief Compute some statistics for the plugin tree.
  *
  * The tree_view parameter must be a tree view created by
  * prefs_create_plugins_tree and filled with
@@ -641,12 +650,15 @@ prefs_plugin_tree_enable_all(tree, enable)
  *
  * The function determines the total number of plugins and the number of
  * active plugins and stores these values is *total and *enabled.
+ * 
+ * @param tree_view    The treeview for which to count nvts.
+ * @param[out] total   Will contain the total number of NVTs in the tree
+ *                     model of the tree view tree_view.
+ * @param[out] enabled Will contain the number of enabled NVTs in the tree model
+ *                     of the tree view tree_view.
  */
 void
-prefs_plugin_tree_statistics(tree_view, total, enabled)
-  GtkTreeView *tree_view;
-  int *total;
-  int *enabled;
+prefs_plugin_tree_statistics (GtkTreeView *tree_view, int *total, int *enabled)
 {
   GtkTreeModel *tree_model = gtk_tree_view_get_model(tree_view);
   GtkTreeModel *real_model = gtk_tree_model_sort_get_model(
@@ -667,11 +679,12 @@ prefs_plugin_tree_statistics(tree_view, total, enabled)
   }
 }
 
-/* Make the tree "read only", meaning that the user cannot change the
- * "active" tflag.
+/**
+ * @brief Make the tree "read only", meaning that the user cannot change the
+ *        "active" tflag.
  */
 void
-prefs_plugin_tree_read_only(GtkTreeView* tree_view, gboolean readonly)
+prefs_plugin_tree_read_only (GtkTreeView* tree_view, gboolean readonly)
 {
   g_object_set_data(G_OBJECT(tree_view), "read-only",
       GINT_TO_POINTER(readonly));
