@@ -52,9 +52,9 @@ sslui_paranoia_callback(w, ctrls)
 			GtkWidget * button = list->data;
 			if(GTK_TOGGLE_BUTTON(button)->active)
 			{
-				value = (int)gtk_object_get_data(GTK_OBJECT(button), "level");
+				value = GPOINTER_TO_SIZE(gtk_object_get_data(GTK_OBJECT(button), "level"));
 				arg_add_value(ctrls, "LEVEL", ARG_INT,
-				sizeof(value), (void*)value);
+						sizeof(gpointer), GSIZE_TO_POINTER(value));
 				gtk_grab_remove(arg_get_value(ctrls, "WINDOW"));
 				gtk_widget_destroy(arg_get_value(ctrls, "WINDOW"));
 				gtk_main_quit();
@@ -63,6 +63,7 @@ sslui_paranoia_callback(w, ctrls)
 			list = list->next;
 		}
 }
+
 static void 
 build_dialog(ctrls, prompt)
  struct arglist * ctrls;
@@ -201,9 +202,9 @@ sslui_ask_paranoia_level()
 	 struct arglist * ctrls = emalloc(sizeof(*ctrls));
 	 build_dialog(ctrls, question);
 	 gtk_main();
-        ret =  (int)arg_get_value(ctrls, "LEVEL");
-	arg_free(ctrls);
-	return ret;
+	 ret =  GPOINTER_TO_SIZE(arg_get_value(ctrls, "LEVEL"));
+	 arg_free(ctrls);
+	 return ret;
 	}
   else
 #endif
@@ -257,16 +258,23 @@ showcert_refuse_cb(w, ctrls)
 
 
 
-
+/**
+ * Displays certificate and asks whether to trust it or not.
+ * (Callbacks showcert_accept_cb and showcert_refuse_cb which ultimately call 
+ * showcert_cb).
+ * @param cert The certificate text.
+ * @return Arglist with WINDOW and RESULT entry (RESULT is 1 if the certificate 
+ *         was accepted).
+ */
 static struct arglist * 
-sslui_showcert(cert)
- char * cert;
+sslui_showcert(char* cert)
 {
  struct arglist * ctrls = emalloc(sizeof(*ctrls));
  GtkWidget * w;
  GtkWidget * vbox, *hbox;
- GtkWidget * label, *text, *table, *vsb, *sep,*button;
- GtkAdjustment * vadj;
+ GtkWidget * label, *text, *sep, *button;
+ GtkWidget* scrollwin;
+ 
  w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_default_size(GTK_WINDOW(w), 640, 480);
  gtk_window_set_position(GTK_WINDOW(w), GTK_WIN_POS_CENTER);
@@ -282,27 +290,28 @@ Here it is:"));
  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 5);
  gtk_widget_show(label);
  
- vadj = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 0.0, 0.0, 0.0, 0.0));
- vsb = gtk_vscrollbar_new(vadj);
- table = gtk_table_new(1,2,FALSE);
- gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);
- gtk_widget_show(table);
+ /* Initialize scroll area */
+ scrollwin = gtk_scrolled_window_new(NULL, NULL);
+ gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW (scrollwin),
+                                 GTK_POLICY_AUTOMATIC,
+                                 GTK_POLICY_AUTOMATIC);
+ gtk_box_pack_start(GTK_BOX(vbox), scrollwin, TRUE, TRUE, 0);
  
- text = gtk_text_new(NULL, vadj);
- gtk_table_attach(GTK_TABLE(table), vsb, 1, 2, 0, 1, 0,
- 			GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
- gtk_table_attach(GTK_TABLE(table), text, 0,1,0,1,
- 			GTK_EXPAND | GTK_SHRINK | GTK_FILL,
-			GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
-			
- gtk_container_border_width(GTK_CONTAINER(table), 2);
- gtk_widget_show(vsb);
- gtk_widget_realize(text);
- gtk_text_set_editable(GTK_TEXT(text), FALSE);
- gtk_text_set_word_wrap(GTK_TEXT(text), TRUE);
- gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, cert, -1);
+ /* Initialize text view */
+ text = gtk_text_view_new();
+ gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
+
+ /* Add certificate text to text view */
+ GtkTextBuffer* textbuffer;
+ textbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
+ gtk_text_buffer_set_text(textbuffer, cert, -1);
+ 
+ /* Add and show scroll area and text view */
+ gtk_container_add(GTK_CONTAINER(scrollwin), text);
  gtk_widget_show(text);						
+ gtk_widget_show(scrollwin);
  
+ /* Seperator, buttons and callbacks */
  sep = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 0);
  gtk_widget_show(sep);
@@ -310,9 +319,7 @@ Here it is:"));
  label = gtk_label_new(_("Do you accept this certificate?"));
  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
  gtk_widget_show(label);
- 
 
- 
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show(hbox);
@@ -329,6 +336,9 @@ Here it is:"));
  	GTK_SIGNAL_FUNC(showcert_refuse_cb), ctrls);
  gtk_widget_show(button);
  
+ g_signal_connect (G_OBJECT (w), "destroy", 
+                   GTK_SIGNAL_FUNC(showcert_refuse_cb), ctrls);
+
  
  gtk_widget_show(w);
  gtk_window_set_modal(GTK_WINDOW(w), TRUE);
@@ -382,7 +392,7 @@ sslui_check_cert(ssl)
   struct arglist * ctrls = sslui_showcert(ascii_cert);
   efree(&ascii_cert);
   gtk_main();
-  ret =  (int)arg_get_value(ctrls, "RESULT");
+  ret =  GPOINTER_TO_SIZE(arg_get_value(ctrls, "RESULT"));
   arg_free(ctrls);
   if(ret) 
     return 0;
