@@ -60,7 +60,6 @@ static void xml_info (int, FILE*, int);
 static void xml_config (FILE*, int);
 static void xml_plugins (FILE*, int);
 static void fprintf_report (FILE*, char*, char*);
-static char * ne_strcasestr (char*, char *);
 static char * escape_string (char*);
 
 /* Play with 'name(port/proto)' (ex: 'www(80/tcp)') */
@@ -70,26 +69,41 @@ static char* getproto (char *str);
 static char* getrisk (char*);
 
 #ifndef XML_INDENT
-#define XML_INDENT "\t"
+/** @brief String to use for one level of indentation in an xml file. */
+#define XML_INDENT "\t" 
 #endif
 
-static int 
+/**
+ * @brief Prints in a printf fashion to file, using indent indentation steps.
+ * 
+ * No escaping whatsoever is done, thus expects direct content of xml file.
+ */
+static int
 xml_fprintf( FILE* fd, int indent, char* fmt, ... )
 {
- va_list ap;
- int i = (indent>=0)?indent:0;
- int ret = 0;
+  va_list ap;
+  int i = (indent>=0) ? indent : 0;
+  int ret = 0;
+
+  for (; i; i--)
+    {
+      fprintf (fd, "%s", XML_INDENT); 
+    }
+
+ va_start (ap, fmt);
+ ret = vfprintf (fd, fmt, ap);
+ va_end (ap);
 
- for (; i; i--) {
-  fprintf( fd, "%s", XML_INDENT ); 
- }
- va_start(ap,fmt);
- ret = vfprintf( fd, fmt, ap );
- va_end(ap);
  if (ret>0) ret += indent; 
  return ret;
 } /* xml_fprintf */
 
+/**
+ * @brief Searches needle in haystack, ignoring case and returns pointer into
+ * @brief haystack.
+ * 
+ * @TODO Use a GLib function.
+ */
 static char * 
 ne_strcasestr(char * haystack, char * needle)
 {
@@ -110,52 +124,53 @@ ne_strcasestr(char * haystack, char * needle)
 } /* ne_strcasestr */
 
 
-
+/**
+ * In a copy of str (that is 6 times bigger...), replaces '\>', '\<', ''', '"'
+ * and '\' by "&gt;", "&lt;", "&amp;", "&quot;" and "&apos;" .
+ */
 static char * escape_string (char * str)
 {
- char *ret, *temp;
- int i, y, len;
+  char *ret, *temp;
+  int i, y, len;
 
- temp = str;
- ret = emalloc (5*strlen(temp)+1);
- len = strlen (str);
+  temp = str;
+  ret = emalloc (6 * strlen (temp) + 1);
+  len = strlen (str);
 
- bzero( ret, 5*strlen(temp)+1 );
-
- for (i=0,y=0;i<len;i++,y++)
- {
-  switch (temp[i])
+  bzero (ret, 6 * strlen (temp) + 1);
+  
+  for (i=0,y=0 ; i<len ; i++,y++)
   {
-   case '>':
-    ret = strcat (ret, "&gt;"); 	y+=3;
-    break;
-
-   case '<':
-    ret = strcat (ret, "&lt;"); 	y+=3;
-    break;
-
-   case '&':
-    ret = strcat (ret, "&amp;"); 	y+=4;
-    break;
-    
-   case '\"':
-     ret = strcat(ret, "&quot;"); 	y += 5;
-     break;
-   
-   case '\'' : 
-      ret = strcat(ret, "&apos;"); 	y += 5;
+    switch (temp[i])
+    {
+    case '>':
+      ret = strcat (ret, "&gt;"); 	y+=3;
       break;
 
-   default: 
-    ret[y] = temp[i];
-  }
- }
+    case '<':
+      ret = strcat (ret, "&lt;"); 	y+=3;
+      break;
 
- ret = (char*) erealloc (ret, strlen (ret)+1);
- if (!ret) {
-  fprintf( stderr, "realloc failed" );
- }
- return ret;
+    case '&':
+      ret = strcat (ret, "&amp;"); 	y+=4;
+      break;
+
+    case '\"':
+      ret = strcat(ret, "&quot;"); 	y += 5;
+      break;
+
+    case '\'' : 
+        ret = strcat(ret, "&apos;"); 	y += 5;
+        break;
+
+    default: 
+      ret[y] = temp[i];
+    }
+  }
+
+  ret = (char*) erealloc (ret, strlen (ret) + 1);
+
+  return ret;
 } /* escape_string */
 
 static void
@@ -186,18 +201,17 @@ xml_fprintf_element (FILE *fd, int indent, char *tag, char *value)
 } /* xml_fprintf_element */
 
 static void
-xml_info_host (int be, FILE* fd, int indent, struct arglist * t)
+xml_info_host (int be, FILE* fd, int indent, struct arglist * server_info)
 {
  const char *host = prefs_get_string(Context, "nessusd_host");
 
  xml_fprintf (fd, indent, "<host>\n");
  xml_fprintf_element(fd, indent+1, "name", (char*)host);
 
- t = arg_get_value(Context->prefs, "SERVER_INFO");
- if (t)
+ if (server_info)
  {
-  xml_fprintf_element (fd, indent+1, "osname", (char *) arg_get_value (t, "server_info_os"));
-  xml_fprintf_element (fd, indent+1, "osvers", (char *) arg_get_value (t, "server_info_os_version"));
+  xml_fprintf_element (fd, indent+1, "osname", (char *) arg_get_value (server_info, "server_info_os"));
+  xml_fprintf_element (fd, indent+1, "osvers", (char *) arg_get_value (server_info, "server_info_os_version"));
  }
  xml_fprintf (fd, indent, "</host>\n");
 } /* xml_info_host */
@@ -236,32 +250,32 @@ xml_info_nessusd (int be, FILE* fd, int indent, struct arglist * t)
 {
  if (t)
  {
-  xml_fprintf (fd, indent, "<nessusd>\n");
-  xml_fprintf_element (fd, indent+1, "version", (char *) arg_get_value (t, "server_info_nessusd_version"));
-  xml_fprintf_element (fd, indent+1, "libnasl", (char *) arg_get_value (t, "server_info_libnasl_version"));
-  xml_fprintf_element (fd, indent+1, "libnessus", (char *) arg_get_value (t, "server_info_libnessus_version"));
+  xml_fprintf (fd, indent, "<openvasd>\n");
+  xml_fprintf_element (fd, indent+1, "version", (char *) arg_get_value (t, "server_info_openvasd_version"));
+  xml_fprintf_element (fd, indent+1, "openvas-libnasl", (char *) arg_get_value (t, "server_info_libnasl_version"));
+  xml_fprintf_element (fd, indent+1, "openvas-libraries", (char *) arg_get_value (t, "server_info_libnessus_version"));
   xml_fprintf_element (fd, indent+1, "thread", (char *) arg_get_value (t, "server_info_thread_manager"));
-  xml_fprintf (fd, indent, "</nessusd>\n");
+  xml_fprintf (fd, indent, "</openvasd>\n");
  } else {
-  show_error("xml_output: No nessusd informations.\n");
-  xml_fprintf (fd, indent, "<nessusd>\n");
+  show_error("xml_output: No openvasd informations.\n");
+  xml_fprintf (fd, indent, "<openvasd>\n");
   xml_fprintf (fd, indent+1, "<!-- no version information found -->\n");
-  xml_fprintf (fd, indent, "</nessusd>\n");
+  xml_fprintf (fd, indent, "</openvasd>\n");
  }
 } /* xml_info_nessusd */
 
 static void
 xml_info (int be, FILE* fd, int indent)
 {
- struct arglist * t;
- t = arg_get_value(Context->prefs, "SERVER_INFO");
+ struct arglist * server_info;
+ server_info = arg_get_value(Context->prefs, "SERVER_INFO");
 
- if (t)
+ if (server_info)
  {
   xml_fprintf (fd, indent, "<info>\n");
-   xml_info_nessusd(be, fd, indent+1, t);
-   xml_info_host(be, fd, indent+1, t);
-   xml_info_date(be, fd, indent+1, t);
+   xml_info_nessusd(be, fd, indent+1, server_info);
+   xml_info_host(be, fd, indent+1, server_info);
+   xml_info_date(be, fd, indent+1, server_info);
   xml_fprintf (fd, indent, "</info>\n\n");
  } else {
   xml_fprintf (fd, indent, "<info>\n");
@@ -277,7 +291,7 @@ xml_config_scanners (FILE* fd, int indent, struct arglist* t)
   while( t->next != NULL )
   {
         char* esc_name = escape_string(t->name);
-	xml_fprintf (fd, indent+1, "<plugin id=\"%s\" value=\"%s\"/>\n", esc_name, (t->value?"yes":"no"));
+	xml_fprintf (fd, indent+1, "<nvt oid=\"%s\" value=\"%s\"/>\n", esc_name, (t->value?"yes":"no"));
         efree(&esc_name);
 	t = t->next;
   }
@@ -325,7 +339,7 @@ xml_setting(FILE* fd, int indent, char* name, char* value)
 static void
 xml_config_plugins (FILE* fd, int indent, struct arglist* t)
 {
-  xml_fprintf (fd, indent, "<plugins>\n");
+  xml_fprintf (fd, indent, "<nvts>\n");
   while( t->next )
   {
    if (t->type == ARG_STRING)
@@ -334,7 +348,7 @@ xml_config_plugins (FILE* fd, int indent, struct arglist* t)
      xml_setting (fd, indent+1, t->name, (t->value?"yes":"no"));
    t = t->next;
   }
-  xml_fprintf (fd, indent, "</plugins>\n");
+  xml_fprintf (fd, indent, "</nvts>\n");
 } /* xml_config_server */
 
 static void
@@ -385,12 +399,19 @@ xml_config (FILE* fd, int indent)
   xml_fprintf (fd, indent, "</config>\n");
 }
 
+/**
+ * @brief Writes details about an nvt to file \<fd\>.
+ * 
+ * @param fd     XML file to write to.
+ * @param indent Current indentation level
+ * @param u      NVT of interest.
+ */
 static void 
 xml_plugins_plugin (FILE* fd, int indent, struct nessus_plugin * u )
-{  
+{
     char * s = nessus_plugin_get_description(u);
 
-     xml_fprintf (fd, indent, "<plugin id=\"%d\">\n", u->id);
+     xml_fprintf (fd, indent, "<nvt oid=\"%s\">\n", u->oid);
      xml_fprintf_element (fd, indent+1, "name",  u->name);
      xml_fprintf_element (fd, indent+1, "version", u->version);
      xml_fprintf_element (fd, indent+1, "family",  u->family);
@@ -400,7 +421,7 @@ xml_plugins_plugin (FILE* fd, int indent, struct nessus_plugin * u )
      xml_fprintf_element (fd, indent+1, "risk",  getrisk(s));
      xml_fprintf_element (fd, indent+1, "summary", u->summary);
      xml_fprintf_element (fd, indent+1, "copyright", u->copyright);
-     xml_fprintf (fd, indent, "</plugin>\n");
+     xml_fprintf (fd, indent, "</nvt>\n");
 } /* xml_plugins_plugin */
 
 static void 
@@ -408,7 +429,7 @@ xml_plugins (FILE* fd, int indent)
 { 
  struct nessus_plugin * t;
 
- xml_fprintf (fd, indent, "<plugins>\n");
+ xml_fprintf (fd, indent, "<nvts>\n");
 
   if ( Context->parent != NULL )
 	t = Context->parent->scanners;
@@ -439,10 +460,10 @@ xml_plugins (FILE* fd, int indent)
     t = t->next;
    }
   else
-   xml_fprintf (fd, indent+1, "<error txt=\"No plugins list.\"/>\n");
+   xml_fprintf (fd, indent+1, "<error txt=\"No NVT list.\"/>\n");
 
 
-  xml_fprintf (fd, indent, "</plugins>\n");
+  xml_fprintf (fd, indent, "</nvts>\n");
 }
 
 
@@ -502,7 +523,8 @@ getproto (char *str)
  for (i=0;i<len; i++)
   if (str[i] == '(') break;
  i++;
- for (i=0;i<len; i++)
+ if (i == len) i = 0;
+ for (i=i;i<len; i++)
   if (str[i] == '/') break;
  i++;
  for (i=i;i<len; i++)
@@ -550,7 +572,7 @@ xml_results_result_ports(
    {
     struct subset * si, * i;
 
-    si = i = query_backend(be, "SELECT severity,plugin_id,report FROM results WHERE port = '%s' AND host = '%s' AND subnet = '%s'", subset_value (p), subset_value (h), subset_value (q));
+    si = i = query_backend(be, "SELECT severity,plugin_oid,report FROM results WHERE port = '%s' AND host = '%s' AND subnet = '%s'", subset_value (p), subset_value (h), subset_value (q));
 
     /* port a la nmap 
     <port protocol="tcp" portid="445"><state state="closed" />
@@ -702,32 +724,35 @@ xml_results ( FILE* fd, int indent, int be)
  subset_free(sq);
 } /* xml_results */
 
-int backend_to_xml_ng(be, filename)
- int be;
- char * filename;
+/**
+ * @brief Exports a backend to xml.
+ * 
+ * @param be       Backend-index.
+ * @param filename Path of file to write to.
+ * 
+ * @return -1 on error (error is displayed from within function), 0 on success.
+ */
+int
+backend_to_xml_ng (int be, char* filename)
 {
- FILE* fd = fopen(filename, "w");
+  FILE* fd = fopen(filename, "w");
 
- if(!fd)
- {
-  show_error(_("Can't open %s for writing: %s."), strerror(errno));
-  return -1;
- }
+  if(!fd)
+    {
+      show_error(_("Can't open %s for writing: %s."), filename, strerror(errno));
+      return -1;
+    }
 
- xml_fprintf (fd, 0, "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n");
- xml_fprintf (fd, 0, "<!-- nessus.xsl can be found in the nessus-tools directory -->\n");
+  xml_fprintf (fd, 0, "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n");
 
- xml_fprintf (fd, 0, "<?xml-stylesheet type=\"text/xsl\" href=\"nessus.xsl\"?>\n");
- xml_fprintf (fd, 0, "<report version=\"1.4\">\n");
- /* don't need this: fprintf (fd, "\t<scan>\n"); */
+  xml_fprintf (fd, 0, "<?xml-stylesheet type=\"text/xsl\" href=\"openvas-report.xsl\"?>\n");
+  xml_fprintf (fd, 0, "<openvas-report version=\"1.0\">\n");
   xml_info (be, fd, 1); 
   xml_config (fd, 1); 
   xml_plugins (fd, 1);
-  xml_results(fd, 1, be);
- /* dont't need this: fprintf (fd, "\t</scan>\n"); */
- xml_fprintf (fd, 0, "</report>\n");
+  xml_results (fd, 1, be);
+  xml_fprintf (fd, 0, "</openvas-report>\n");
 
- fclose(fd);
- return 0; 
+  fclose(fd);
+  return 0;
 }
-
